@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -103,19 +103,17 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 
-#include "Programs/misc.h"
-#include "Programs/brltty.h"
+#include "misc.h"
+#include "brltty.h"
 
 #define BRLSTAT ST_AlvaStyle
 #define BRL_HAVE_FIRMNESS
 #define BRLCONST
-#include "Programs/brl_driver.h"
+#include "brl_driver.h"
 #include "braille.h"
 
 static const int logInputBuffer = 0;
@@ -124,99 +122,129 @@ static const int logOutputPackets = 0;
 
 /* Braille display parameters */
 typedef struct {
-  const char *Name;
-  unsigned char ID;
-  unsigned char Flags;
-  unsigned char Cols;
-  unsigned char NbStCells;
-  unsigned char HelpPage;
-} BRLPARAMS;
-#define BPF_CONFIGURABLE 0X01
+  const char *name;
+  unsigned char identifier;
+  unsigned char columns;
+  unsigned char statusCells;
+  unsigned char flags;
+  unsigned char helpPage;
+} ModelEntry;
+#define MOD_FLG__CONFIGURABLE 0X01
 
-static BRLPARAMS Models[] =
-{
-  {
-    /* ID == 0 */
-    "ABT 320", ABT320, 0,
-    20, 3, 0
+static const ModelEntry modelTable[] = {
+  { .identifier = 0X00,
+    .name = "ABT 320",
+    .columns = 20,
+    .statusCells = 3,
+    .flags = 0,
+    .helpPage = 0
   }
   ,
-  {
-    /* ID == 1 */
-    "ABT 340", ABT340, 0,
-    40, 3, 0
+  { .identifier = 0X01,
+    .name = "ABT 340",
+    .columns = 40,
+    .statusCells = 3,
+    .flags = 0,
+    .helpPage = 0
   }
   ,
-  {
-    /* ID == 2 */
-    "ABT 340 Desktop", ABT34D, 0,
-    40, 5, 0
+  { .identifier = 0X02,
+    .name = "ABT 340 Desktop",
+    .columns = 40,
+    .statusCells = 5,
+    .flags = 0,
+    .helpPage = 0
   }
   ,
-  {
-    /* ID == 3 */
-    "ABT 380", ABT380, 0,
-    80, 5, 0
+  { .identifier = 0X03,
+    .name = "ABT 380",
+    .columns = 80,
+    .statusCells = 5,
+    .flags = 0,
+    .helpPage = 0
   }
   ,
-  {
-    /* ID == 4 */
-    "ABT 382 Twin Space", ABT382, 0,
-    80, 5, 0
+  { .identifier = 0X04,
+    .name = "ABT 382 Twin Space",
+    .columns = 80,
+    .statusCells = 5,
+    .flags = 0,
+    .helpPage = 0
   }
   ,
-  {
-    /* ID == 10 */
-    "Delphi 420", DEL420, 0,
-    20, 3, 0
+  { .identifier = 0X0A,
+    .name = "Delphi 420",
+    .columns = 20,
+    .statusCells = 3,
+    .flags = 0,
+    .helpPage = 0
   }
   ,
-  {
-    /* ID == 11 */
-    "Delphi 440", DEL440, 0,
-    40, 3, 0
+  { .identifier = 0X0B,
+    .name = "Delphi 440",
+    .columns = 40,
+    .statusCells = 3,
+    .flags = 0,
+    .helpPage = 0
   }
   ,
-  {
-    /* ID == 12 */
-    "Delphi 440 Desktop", DEL44D, 0,
-    40, 5, 0
+  { .identifier = 0X0C,
+    .name = "Delphi 440 Desktop",
+    .columns = 40,
+    .statusCells = 5,
+    .flags = 0,
+    .helpPage = 0
   }
   ,
-  {
-    /* ID == 13 */
-    "Delphi 480", DEL480, 0,
-    80, 5, 0
+  { .identifier = 0X0D,
+    .name = "Delphi 480",
+    .columns = 80,
+    .statusCells = 5,
+    .flags = 0,
+    .helpPage = 0
   }
   ,
-  {
-    /* ID == 14 */
-    "Satellite 544", SAT544, BPF_CONFIGURABLE,
-    40, 3, 1
+  { .identifier = 0X0E,
+    .name = "Satellite 544",
+    .columns = 40,
+    .statusCells = 3,
+    .flags = MOD_FLG__CONFIGURABLE,
+    .helpPage = 1
   }
   ,
-  {
-    /* ID == 15 */
-    "Satellite 570 Pro", SAT570P, BPF_CONFIGURABLE,
-    66, 3, 1
+  { .identifier = 0X0F,
+    .name = "Satellite 570 Pro",
+    .columns = 66,
+    .statusCells = 3,
+    .flags = MOD_FLG__CONFIGURABLE,
+    .helpPage = 1
   }
   ,
-  {
-    /* ID == 16 */
-    "Satellite 584 Pro", SAT584P, BPF_CONFIGURABLE,
-    80, 3, 1
+  { .identifier = 0X10,
+    .name = "Satellite 584 Pro",
+    .columns = 80,
+    .statusCells = 3,
+    .flags = MOD_FLG__CONFIGURABLE,
+    .helpPage = 1
   }
   ,
-  {
-    /* ID == 17 */
-    "Satellite 544 Traveller", SAT544T, BPF_CONFIGURABLE,
-    40, 3, 1
+  { .identifier = 0X11,
+    .name = "Satellite 544 Traveller",
+    .columns = 40,
+    .statusCells = 3,
+    .flags = MOD_FLG__CONFIGURABLE,
+    .helpPage = 1
   }
   ,
-  {
-    NULL, 0, 0,
-    0, 0, 0
+  { .identifier = 0X13,
+    .name = "Braille System 40",
+    .columns = 40,
+    .statusCells = 0,
+    .flags = MOD_FLG__CONFIGURABLE,
+    .helpPage = 1
   }
+  ,
+  { .name = NULL }
 };
 
 
@@ -237,7 +265,7 @@ static unsigned char *prevdata = NULL;	/* previously sent raw data */
 static unsigned char StatusCells[MAX_STCELLS];		/* to hold status info */
 static unsigned char PrevStatus[MAX_STCELLS];	/* to hold previous status */
 static unsigned char NbStCells;	/* number of status cells */
-static BRLPARAMS *model;		/* points to terminal model config struct */
+static const ModelEntry *model;		/* points to terminal model config struct */
 static int rewriteRequired = 0;		/* 1 if display need to be rewritten */
 static int rewriteInterval;
 static struct timeval rewriteTime;
@@ -360,7 +388,7 @@ static int inputUsed;
 static int
 verifyInputPacket (unsigned char *buffer, int *length) {
   int size = 0;
-  if (logInputBuffer) LogBytes("Input Buffer", inputBuffer, inputUsed);
+  if (logInputBuffer) LogBytes(LOG_DEBUG, "Input Buffer", inputBuffer, inputUsed);
 
 #if ! ABT3_OLD_FIRMWARE
   while (inputUsed > 0) {
@@ -398,7 +426,7 @@ verifyInputPacket (unsigned char *buffer, int *length) {
     }
 
   unrecognized:
-    LogBytes("Unrecognized Packet", inputBuffer, inputUsed);
+    LogBytes(LOG_WARNING, "Unrecognized Packet", inputBuffer, inputUsed);
     memcpy(&inputBuffer[0], &inputBuffer[1], --inputUsed);
   }
 #else /* ABT3_OLD_FIRMWARE */
@@ -406,7 +434,7 @@ verifyInputPacket (unsigned char *buffer, int *length) {
 #endif /* ! ABT3_OLD_FIRMWARE */
 
   if (!size) return 0;
-  if (logInputPackets) LogBytes("Input Packet", inputBuffer, size);
+  if (logInputPackets) LogBytes(LOG_DEBUG, "Input Packet", inputBuffer, size);
 
   if (*length < size) {
     LogPrint(LOG_DEBUG, "truncated input packet: %d < %d", *length, size);
@@ -432,7 +460,7 @@ writeParameter (BrailleDisplay *brl, unsigned char parameter, unsigned char sett
   return io->writePacket(bytes, sizeof(bytes), &brl->writeDelay);
 }
 
-#include "Programs/io_serial.h"
+#include "io_serial.h"
 static SerialDevice *serialDevice = NULL;
 static int serialCharactersPerSecond;
 
@@ -481,8 +509,8 @@ readSerialPacket (unsigned char *buffer, int length) {
 
 static int
 writeSerialPacket (const unsigned char *buffer, int length, unsigned int *delay) {
-  if (logOutputPackets) LogBytes("Output Packet", buffer, length);
-  if (delay) *delay += length * 1000 / serialCharactersPerSecond;
+  if (logOutputPackets) LogBytes(LOG_DEBUG, "Output Packet", buffer, length);
+  if (delay) *delay += (length * 1000 / serialCharactersPerSecond) + 1;
   return serialWriteData(serialDevice, buffer, length);
 }
 
@@ -492,15 +520,19 @@ static const InputOutputOperations serialOperations = {
 };
 
 #ifdef ENABLE_USB_SUPPORT
-#include "Programs/io_usb.h"
+#include "io_usb.h"
 
 static UsbChannel *usb = NULL;
 
 static int
 openUsbPort (char **parameters, const char *device) {
   static const UsbChannelDefinition definitions[] = {
-    {0X06b0, 0X0001, 1, 0, 0, 1, 2, 0},
-    {0}
+    { .vendor=0X06b0, .product=0X0001,
+      .configuration=1, .interface=0, .alternative=0,
+      .inputEndpoint=1, .outputEndpoint=2
+    }
+    ,
+    { .vendor=0 }
   };
 
   rewriteInterval = 0;
@@ -544,7 +576,7 @@ readUsbPacket (unsigned char *buffer, int length) {
       inputUsed += count;
       if (verifyInputPacket(buffer, &length)) return length;
     } else if (inputUsed) {
-      LogBytes("Truncated Packet", inputBuffer, inputUsed);
+      LogBytes(LOG_WARNING, "Truncated Packet", inputBuffer, inputUsed);
       inputUsed = 0;
     }
   }
@@ -552,7 +584,7 @@ readUsbPacket (unsigned char *buffer, int length) {
 
 static int
 writeUsbPacket (const unsigned char *buffer, int length, unsigned int *delay) {
-  if (logOutputPackets) LogBytes("Output Packet", buffer, length);
+  if (logOutputPackets) LogBytes(LOG_DEBUG, "Output Packet", buffer, length);
   return usbWriteEndpoint(usb->device, usb->definition.outputEndpoint, buffer, length, 1000);
 }
 
@@ -562,26 +594,6 @@ static const InputOutputOperations usbOperations = {
 };
 #endif /* ENABLE_USB_SUPPORT */
 
-static void
-brl_identify (void)
-{
-  LogPrint(LOG_NOTICE, "Alva driver, version 2.2");
-  LogPrint(LOG_INFO, "   Copyright (C) 1995-2004 by Nicolas Pitre <nico@cam.org>.");
-  LogPrint(LOG_INFO, "   Compiled for %s with %s version.",
-#if MODEL == ABT_AUTO
-	  "terminal autodetection",
-#else /* MODEL == ABT_AUTO */
-	  Models[MODEL].Name,
-#endif /* MODEL == ABT_AUTO */
-#if ABT3_OLD_FIRMWARE
-	  "old firmware"
-#else /* ABT3_OLD_FIRMWARE */
-	  "new firmware"
-#endif /* ABT3_OLD_FIRMWARE */
-    );
-}
-
-
 int AL_writeData( unsigned char *data, int len )
 {
   if (io->writePacket(data, len, NULL) == len) return 1;
@@ -636,27 +648,26 @@ static int
 identifyModel (BrailleDisplay *brl, unsigned char identifier) {
   /* Find out which model we are connected to... */
   for (
-    model = Models;
-    model->Name && (model->ID != identifier);
+    model = modelTable;
+    model->name && (model->identifier != identifier);
     model++
   );
 
-  if (!model->Name) {
+  if (!model->name) {
     /* Unknown model */
     LogPrint(LOG_ERR, "Detected unknown Alva model with ID %02X (hex).", identifier);
-    LogPrint(LOG_WARNING, "Please fix Models[] in Alva/braille.c and mail the maintainer.");
     return 0;
   }
-  LogPrint(LOG_INFO, "Detected Alva %s: %d columns, %d status cells.",
-           model->Name, model->Cols, model->NbStCells);
+  LogPrint(LOG_INFO, "Detected Alva %s: %d columns, %d status cells",
+           model->name, model->columns, model->statusCells);
 
   /* Set model parameters... */
-  brl->x = model->Cols;
+  brl->x = model->columns;
   brl->y = 1;
-  brl->helpPage = model->HelpPage;			/* initialise size of display */
-  NbStCells = model->NbStCells;
+  brl->helpPage = model->helpPage;			/* initialise size of display */
+  NbStCells = model->statusCells;
 
-  if (model->Flags & BPF_CONFIGURABLE) {
+  if (model->flags & MOD_FLG__CONFIGURABLE) {
     BRLSYMBOL.firmness = brl_firmness;
 
     writeFunction(brl, 0X07);
@@ -686,8 +697,8 @@ identifyModel (BrailleDisplay *brl, unsigned char identifier) {
 }
 
 static int
-brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
-  unsigned char ModelID = MODEL;
+brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
+  unsigned char ModelID;
 
   {
     static const DotsTable dots = {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80};
@@ -696,13 +707,15 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
 
   if (isSerialDevice(&device)) {
     io = &serialOperations;
+  } else
 
 #ifdef ENABLE_USB_SUPPORT
-  } else if (isUsbDevice(&device)) {
+  if (isUsbDevice(&device)) {
     io = &usbOperations;
+  } else
 #endif /* ENABLE_USB_SUPPORT */
 
-  } else {
+  {
     unsupportedDevice(device);
     return 0;
   }
@@ -736,7 +749,7 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
 }
 
 
-static void brl_close (BrailleDisplay *brl)
+static void brl_destruct (BrailleDisplay *brl)
 {
   if (rawdata) {
     free(rawdata);
@@ -949,7 +962,7 @@ GetKey (BrailleDisplay *brl, unsigned int *Keys, unsigned int *Pos) {
   }
 
 #endif /* ! ABT3_OLD_FIRMWARE */
-  LogBytes("Unexpected Packet", packet, length);
+  LogBytes(LOG_WARNING, "Unexpected Packet", packet, length);
   return 0;
 }
 
