@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,13 +22,11 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 
-#include "Programs/misc.h"
+#include "misc.h"
 
 typedef enum {
   PARM_STATUSCELLS
@@ -36,14 +34,14 @@ typedef enum {
 #define BRLPARMS "statuscells"
 
 #define BRL_HAVE_PACKET_IO
-#include "Programs/brl_driver.h"
+#include "brl_driver.h"
 #include "braille.h"
-#include "Programs/tbl.h"
+#include "tbl.h"
 
 static int logInputPackets = 0;
 static int logOutputPackets = 0;
 
-#include "Programs/io_serial.h"
+#include "io_serial.h"
 static SerialDevice *serialDevice = NULL;
 static const int serialBaud = 38400;
 static int charactersPerSecond;
@@ -122,7 +120,7 @@ readPacket (unsigned char *packet, int size) {
     unsigned char byte;
 
     if (!readByte(&byte, (offset > 0))) {
-      if (offset > 0) LogBytes("Partial Packet", packet, offset);
+      if (offset > 0) LogBytes(LOG_WARNING, "Partial Packet", packet, offset);
       return 0;
     }
 
@@ -147,7 +145,7 @@ readPacket (unsigned char *packet, int size) {
             break;
 
           default:
-            LogBytes("Unknown Packet", &byte, 1);
+            LogBytes(LOG_WARNING, "Unknown Packet", &byte, 1);
             offset = 0;
             length = 0;
             continue;
@@ -156,8 +154,8 @@ readPacket (unsigned char *packet, int size) {
 
       packet[offset] = byte;
     } else {
-      if (offset == size) LogBytes("Truncated Packet", packet, offset);
-      LogBytes("Discarded Byte", &byte, 1);
+      if (offset == size) LogBytes(LOG_WARNING, "Truncated Packet", packet, offset);
+      LogBytes(LOG_WARNING, "Discarded Byte", &byte, 1);
     }
 
     if (++offset == length) {
@@ -167,7 +165,7 @@ readPacket (unsigned char *packet, int size) {
         continue;
       }
 
-      if (logInputPackets) LogBytes("Input Packet", packet, offset);
+      if (logInputPackets) LogBytes(LOG_DEBUG, "Input Packet", packet, offset);
       return length;
     }
   }
@@ -192,11 +190,11 @@ writePacket (BrailleDisplay *brl, const unsigned char *packet, int size) {
 
   {
     int count = byte - buffer;
-    if (logOutputPackets) LogBytes("Output Packet", buffer, count);
+    if (logOutputPackets) LogBytes(LOG_DEBUG, "Output Packet", buffer, count);
 
     {
       int ok = serialWriteData(serialDevice, buffer, count) != -1;
-      if (ok) brl->writeDelay += count * 1000 / charactersPerSecond;
+      if (ok) brl->writeDelay += (count * 1000 / charactersPerSecond) + 1;
       return ok;
     }
   }
@@ -379,28 +377,24 @@ visualDisplay (BrailleDisplay *brl, unsigned char byte, BRL_DriverCommandContext
   return EOF;
 }
 
-static void
-brl_identify (void) {
-  LogPrint(LOG_NOTICE, "BrailleNote Driver, version 1.0");
-  LogPrint(LOG_INFO, "   Copyright (C) 2001 by Dave Mielke <dave@mielke.cc>");
-}
-
 static void
 adjustStatusCells (BrailleDisplay *brl, const char *parameter) {
   if (*parameter) {
     const int minimum = 0;
     const int maximum = MIN(BRL_MAX_STATUS_CELL_COUNT, brl->x-1);
-    if (validateInteger(&statusCells, "status cell count", parameter, &minimum, &maximum)) {
+    if (validateInteger(&statusCells, parameter, &minimum, &maximum)) {
       statusArea = dataArea;
       dataArea = statusArea + statusCells;
       brl->x -= statusCells;
       dataCells -= statusCells;
+    } else {
+      LogPrint(LOG_WARNING, "%s: %s", "invalid status cell count", parameter);
     }
   }
 }
 
 static int
-brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
+brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
   {
      static const DotsTable dots = {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80};
      makeOutputTable(dots, outputTable);
@@ -446,7 +440,7 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
                 LogError("cell buffer allocation");
               }
             } else {
-              LogBytes("unexpected packet", response.bytes, size);
+              LogBytes(LOG_WARNING, "Unexpected Packet", response.bytes, size);
             }
           }
         }
@@ -459,7 +453,7 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
 }
 
 static void
-brl_close (BrailleDisplay *brl) {
+brl_destruct (BrailleDisplay *brl) {
   serialCloseDevice(serialDevice);
   serialDevice = NULL;
 
@@ -468,14 +462,14 @@ brl_close (BrailleDisplay *brl) {
 }
 
 static ssize_t
-brl_readPacket (BrailleDisplay *brl, unsigned char *buffer, size_t size) {
+brl_readPacket (BrailleDisplay *brl, void *buffer, size_t size) {
   int count = readPacket(buffer, size);
   if (!count) count = -1;
   return count;
 }
 
 static ssize_t
-brl_writePacket (BrailleDisplay *brl, const unsigned char *packet, size_t length) {
+brl_writePacket (BrailleDisplay *brl, const void *packet, size_t length) {
   return writePacket(brl, packet, length)? length: -1;
 }
 
@@ -824,7 +818,6 @@ interpretBackspaceChord (BrailleDisplay *brl, unsigned char dots, BRL_DriverComm
       break;
     case BNC_SPACE:
       return BRL_BLK_PASSKEY + BRL_KEY_BACKSPACE;
-      return EOF;
     case BNC_A:
       return BRL_CMD_DISPMD | BRL_FLG_TOGGLE_ON;
     case BNC_B:
@@ -877,7 +870,6 @@ interpretEnterChord (BrailleDisplay *brl, unsigned char dots, BRL_DriverCommandC
       break;
     case BNC_SPACE:
       return BRL_BLK_PASSKEY + BRL_KEY_ENTER;
-      return EOF;
     case BNC_B:
       return BRL_CMD_SKPBLNKWINS | BRL_FLG_TOGGLE_ON;
     case BNC_F:
@@ -977,7 +969,7 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
         break;
 
       default:
-        LogBytes("unexpected packet", packet.bytes, size);
+        LogBytes(LOG_WARNING, "Unexpected Packet", packet.bytes, size);
         continue;
     }
 
