@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -16,7 +16,6 @@
  */
 
 /* EuroBraille/braille.c - Braille display library for the EuroBraille family.
- * Copyright (C) 1997-2005 by Yannick Plassiard <yan@mistigri.org>
  *                        and Nicolas Pitre <nico@cam.org>
  * See the GNU General Public License for details in the ../../COPYING file
  * See the README file for details about copyrights and version informations
@@ -25,13 +24,11 @@
 #include "prologue.h"
 
 #include <fcntl.h>
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "Programs/message.h"
-#include "Programs/misc.h"
+#include "message.h"
+#include "misc.h"
 
 /*
 ** For debugging only
@@ -40,9 +37,9 @@
 
 #define BRL_HAVE_PACKET_IO
 #define BRL_HAVE_VISUAL_DISPLAY
-#include "Programs/brl_driver.h"
+#include "brl_driver.h"
 #include "braille.h"
-#include "Programs/io_serial.h"
+#include "io_serial.h"
 
 /*
 ** For debugging only
@@ -279,7 +276,7 @@ static int	alt = 0;
 #define EOT     0x04
 #define ACK     0x06
 #define DLE     0x10
-#define NACK    0x15
+#define NAK    0x15
 
 #define PRT_E_PAR 0x01		/* parity error */
 #define PRT_E_NUM 0x02		/* frame numver error */
@@ -309,53 +306,34 @@ static int sendbyte(unsigned char c)
   return (serialWriteData(serialDevice, &c, 1));
 }
 
-static int WriteToBrlDisplay (BrailleDisplay *brl, int len, const unsigned char *data)
+static ssize_t brl_writePacket(BrailleDisplay *brl, const void *packet, size_t packetSize)
 {
-  unsigned char	buf[1024];
+  static const unsigned char needsEscape[0X100] = {
+    [SOH] = 1, [EOT] = 1, [DLE] = 1, [ACK] = 1, [NAK] = 1
+  };
+  const unsigned char *data = packet;
+  unsigned char	buf[(packetSize + 3) * 2];
   unsigned char		*p = buf;
   unsigned char parity = 0;
 #ifdef	LOG_IO
   int		logfd;
 #endif
 
-   if (!len)
-     return 1;
-
    *p++ = SOH;
-   while (len--)
+   while (packetSize--)
      {
-	switch (*data)
-	  {
-	   case SOH:
-	   case EOT:
-	   case ACK:
-	   case DLE:
-	   case NACK:
-	     *p++ = DLE;
-	  /* no break */
-	   default:
-	     *p++ = *data;
-	     parity ^= *data++;
-	  }
+	if (needsEscape[*data]) *p++ = DLE;
+        *p++ = *data;
+	parity ^= *data++;
      }
-   *p++ = PktNbr;
+   *p++ = PktNbr; /* Doesn't need to be prefixed since greater than 128 */
    parity ^= PktNbr;
    if (++PktNbr >= 256)
      PktNbr = 128;
-   switch (parity)
-     {
-      case SOH:
-      case EOT:
-      case ACK:
-      case DLE:
-      case NACK:
-	*p++ = DLE;
-      /* no break */
-      default:
-	*p++ = parity;
-     }
+   if (needsEscape[parity]) *p++ = DLE;
+   *p++ = parity;
    *p++ = EOT;
-   brl->writeDelay += (p - buf) * 1000 / chars_per_sec;
+   brl->writeDelay += ((p - buf) * 1000 / chars_per_sec) + 1;
 #ifdef		LOG_IO
    logfd = open("/tmp/eb-log.out", O_CREAT | O_APPEND | O_WRONLY, 0600);
    write(logfd, "WritePacket: ", 13);
@@ -363,31 +341,17 @@ static int WriteToBrlDisplay (BrailleDisplay *brl, int len, const unsigned char
    write(logfd, "\n", 1);
    close(logfd);
 #endif
-   return (serialWriteData(serialDevice, buf, p - buf));
-}
-
-static ssize_t brl_writePacket(BrailleDisplay *brl, const unsigned char *p, size_t sz)
-{
-  if (!WriteToBrlDisplay(brl, sz, p))
-    return 0;
-  return sz;
+   return (serialWriteData(serialDevice, buf, p - buf) == p - buf);
 }
 
 static int brl_reset(BrailleDisplay *brl)
 {
   static const unsigned char packet[] = {0X02, 'S', 'I'};
+  LogPrint(LOG_INFO, "EuroBraille: hardware reset requested");
   return (brl_writePacket(brl, packet, sizeof(packet)) == sizeof(packet));
 }
 
-static void brl_identify (void)
-{
-   LogPrint(LOG_NOTICE, "EuroBraille driver, version 1.3.3");
-   LogPrint(LOG_INFO, "  Copyright (C) 1997-2003");
-   LogPrint(LOG_INFO, "      - Yannick PLASSIARD <plassi_y@epitech.net>");
-   LogPrint(LOG_INFO, "      - Nicolas PITRE <nico@cam.org>");
-}
-
-static int brl_open (BrailleDisplay *brl, char **parameters, const char *device)
+static int brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
 {
    {
      static const DotsTable dots = {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80};
@@ -406,7 +370,7 @@ static int brl_open (BrailleDisplay *brl, char **parameters, const char *device)
    serialSetParity(serialDevice, SERIAL_PARITY_EVEN);
 
   /* set speed */
-   chars_per_sec = BAUDRATE / 10;
+   chars_per_sec = BAUDRATE / 11;
    serialRestartDevice(serialDevice, BAUDRATE);
 
   /* Set model params... */
@@ -415,8 +379,7 @@ static int brl_open (BrailleDisplay *brl, char **parameters, const char *device)
    while (!NbCols)
      {
 	int i = 0;
-	unsigned char AskIdent[] = "\x2SI";
-	WriteToBrlDisplay (brl, sizeof(AskIdent), AskIdent);
+	brl_reset(brl);
 	while (!NbCols)
 	  {
 	     drainBrailleOutput (brl, 100);
@@ -428,10 +391,11 @@ static int brl_open (BrailleDisplay *brl, char **parameters, const char *device)
 
    ReWrite = 1;  /* To write whole display at first time */
    ReWrite_LCD = 1;
+   LogPrint(LOG_INFO, "EuroBraille Display detection successfull, cols=%d", NbCols);
    return 1;
 }
 
-static void brl_close (BrailleDisplay *brl)
+static void brl_destruct (BrailleDisplay *brl)
 {
    if (rawdata)
      {
@@ -453,6 +417,8 @@ static void brl_close (BrailleDisplay *brl)
        serialCloseDevice (serialDevice);
        serialDevice = NULL;
      }
+   NbCols = 0;
+   strcpy(version_ID, "Unknown");
 }
 
 static void brl_writeWindow (BrailleDisplay *brl)
@@ -482,7 +448,7 @@ static void brl_writeWindow (BrailleDisplay *brl)
 	     for (j = 0;
 		  j < i;
 		  *p++ = outputTable[(prevdata[j++] = brl->buffer[j])]);
-	     WriteToBrlDisplay (brl, p - OutBuf, OutBuf);
+	     brl_writePacket (brl, OutBuf, p - OutBuf);
 	     ReWrite = 0;
 	  }
      }
@@ -515,7 +481,7 @@ static void	brl_writeVisual(BrailleDisplay *brl)
       *p++ = 'L';
       for (j = 0; j < i; j++)
 	*p++ = brl->buffer[j];
-      WriteToBrlDisplay(brl, p - OutBuf, OutBuf);
+      brl_writePacket(brl, OutBuf, p - OutBuf);
       ReWrite_LCD = 0;
     }
 }
@@ -898,35 +864,51 @@ static int	key_handle(BrailleDisplay *brl, unsigned char *buf)
 
 #ifdef		BRL_HAVE_PACKET_IO
 
-static ssize_t brl_readPacket(BrailleDisplay *brl, unsigned char *bp, size_t size)
+static ssize_t brl_readPacket(BrailleDisplay *brl, void *packet, size_t size)
 {
-  int           i = 0; /* cpt to build the received packet */
-  char          start = 0;  /* a flag */
-  size_t        offset = 0;
-  unsigned char         par = 0;
-  int           j = 0;
-  int           k = 0; /* cpt to build the returned buffer (bp) */
-  unsigned char c;
-  unsigned char buf[512];
-  
-  while (serialReadChunk(serialDevice, &c, &offset, 1, 1000, 0))
+  unsigned char *bp = packet;
+  static int		i = 0; /* cpt to build the received packet */
+  size_t		offset = 0; /* offset to use serialReadChunk */
+  static char		apacket = 0;  /* =1 when packet starts */
+  static unsigned char	par = 0; /* parity */
+  int			j = 0; /* cpt to parse the read buffer */
+  int			k = 0; /* cpt to build the returned buffer (bp) */
+  unsigned char		c; /* the read char */
+  static unsigned char	buf[512];  /* buffer to store read chars */
+  static unsigned char	end = 0; /* = 1 when end of packet */
+  static unsigned char prefix = 0; /* =1 when escape code detected */
+
+  if ((bp==NULL) || (size<2) || (size>512))
+    return 0;
+  while (serialReadChunk(serialDevice, &c, &offset, 1, 0, 0))
     {
-      if (c == SOH)
-        start = 1;
-      if (c == EOT && buf[i - 1] != DLE)
-        {
-          buf[i++] = EOT;
-          break;
-        }
-      if (start) {
-        buf[i++] = c;
-      }
+      if (!apacket && c == SOH)
+	{
+	  end = 0;
+	  apacket = 1;
+	  i = 0;
+	  buf[i++] = SOH;
+	  par = 0;
+	}
+      else if (apacket)
+	{
+	  if (c == EOT && !prefix)
+	    {
+	      apacket = 0;
+	      buf[i++] = EOT;
+	      end = 1;
+	      break;
+	    }
+	  if (c == DLE && !prefix)
+	    prefix = 1;
+	  else
+	    prefix = 0;
+	  buf[i++] = c;
+	}
       offset = 0;
     }
-  if (i == 0) /* no character, timeout */
-    {
-      return 0;
-    }
+  if (!end) /* either nothing read or not end of packet */
+    return 0;
   for (j = 1, k = 0; j < i - 2 && k < size ; j++) {
     if (buf[j] != DLE || (buf[j - 1] == DLE))
       {
@@ -935,14 +917,17 @@ static ssize_t brl_readPacket(BrailleDisplay *brl, unsigned char *bp, size_t siz
 	k++;
       }
   }
+  end = 0;
   if (par == buf[i - 2])
     {
       sendbyte(ACK);
+      serialDiscardInput(serialDevice);
       return k - 1;
     }
   /* if bad parity */
-  sendbyte(NACK);
-  sendbyte(1); /* error code of a parity error */
+  serialDiscardInput(serialDevice);
+  sendbyte(NAK);
+  sendbyte(PRT_E_PAR); /* error code of a parity error */
   return 0;
 }
 
@@ -981,7 +966,7 @@ static int readbrlkey(BrailleDisplay *brl)
       else
 	switch (c)
 	  {
-	  case NACK:
+	  case NAK:
 	    ErrFlag = 1;
 	    /* no break */
 	  case ACK:
@@ -1002,7 +987,7 @@ static int readbrlkey(BrailleDisplay *brl)
 		parity ^= buf[i];
 	      if (parity != buf[pos - 1])
 		{
-		  sendbyte (NACK);
+		  sendbyte (NAK);
 		  sendbyte (PRT_E_PAR);
 		}
 	      else if (buf[pos - 2] == old_pktnbr)
@@ -1057,10 +1042,8 @@ static int readbrlkey(BrailleDisplay *brl)
 		case 'B': /* PC-BRAILLE mode */
 		  if (JustIdentified == 0)
 		  {
-		    unsigned char AskIdent[] = {2, 'S', 'I'};
-
 		    LogPrint(LOG_INFO, "EuroBraille terminal came in PC mode.");
-		    WriteToBrlDisplay(brl, sizeof(AskIdent), AskIdent);
+		    brl_reset(brl);
 		    JustIdentified = 0;
 		    ReWrite = 1; /* to refresh display */
 		    context = 0;
@@ -1120,7 +1103,7 @@ static int readbrlkey(BrailleDisplay *brl)
 		    model_ID = 5;
 		  else
 		    model_ID = 0;
-		  if (strncmp(version_ID, (char *)(buf + p + 2), 3))
+		  //		  if (strncmp(version_ID, (char *)(buf + p + 2), 3))
 		    {
 		      strncpy(version_ID, (char *)(buf + p + 2), 20);
 		      NbCols = (buf[p + 4] - '0') * 10;
