@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,13 +22,11 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 
-#include "Programs/misc.h"
+#include "misc.h"
 
 typedef enum {
   PARM_DEBUGPACKETS,
@@ -39,7 +37,7 @@ typedef enum {
 #define BRLSTAT ST_AlvaStyle
 #define BRL_HAVE_PACKET_IO
 #define BRL_HAVE_FIRMNESS
-#include "Programs/brl_driver.h"
+#include "brl_driver.h"
 
 typedef struct {
   int (*openPort) (char **parameters, const char *device);
@@ -53,7 +51,7 @@ static const InputOutputOperations *io;
 static unsigned int debugPackets = 0;
 static int outputPayloadLimit;
 
-#include "Programs/io_serial.h"
+#include "io_serial.h"
 static SerialDevice *serialDevice = NULL;
 static int serialCharactersPerSecond;
 
@@ -94,7 +92,7 @@ readSerialBytes (void *buffer, int length) {
 static int
 writeSerialPacket (const void *buffer, int length, unsigned int *delay) {
   int written = serialWriteData(serialDevice, buffer, length);
-  if (delay && (written != -1)) *delay += length * 1000 / serialCharactersPerSecond;
+  if (delay && (written != -1)) *delay += (length * 1000 / serialCharactersPerSecond) + 1;
   return written;
 }
 
@@ -104,17 +102,32 @@ static const InputOutputOperations serialOperations = {
 };
 
 #ifdef ENABLE_USB_SUPPORT
-#include "Programs/io_usb.h"
+#include "io_usb.h"
 
 static UsbChannel *usb = NULL;
 
 static int
 openUsbPort (char **parameters, const char *device) {
   static const UsbChannelDefinition definitions[] = {
-    {0X0F4E, 0X0100, 1, 0, 0, 2, 1, 0}, /* Focus */
-    {0X0F4E, 0X0111, 1, 0, 0, 2, 1, 0}, /* PAC Mate */
-    {0X0F4E, 0X0112, 1, 0, 0, 2, 1, 0}, /* Focus 2 */
-    {0}
+    { /* Focus */
+      .vendor=0X0F4E, .product=0X0100,
+      .configuration=1, .interface=0, .alternative=0,
+      .inputEndpoint=2, .outputEndpoint=1
+    }
+    ,
+    { /* PAC Mate */
+      .vendor=0X0F4E, .product=0X0111,
+      .configuration=1, .interface=0, .alternative=0,
+      .inputEndpoint=2, .outputEndpoint=1
+    }
+    ,
+    { /* Focus 2 */
+      .vendor=0X0F4E, .product=0X0112,
+      .configuration=1, .interface=0, .alternative=0,
+      .inputEndpoint=2, .outputEndpoint=1
+    }
+    ,
+    { .vendor=0 }
   };
 
   if ((usb = usbFindChannel(definitions, (void *)device))) {
@@ -268,10 +281,10 @@ static union {
 } inputBuffer;
 static int inputCount;
 
-static int realKeys;
-static int virtualKeys;
-static int pressedKeys;
-static int activeKeys;
+static unsigned int realKeys;
+static unsigned int virtualKeys;
+static unsigned int pressedKeys;
+static unsigned int activeKeys;
 #define KEY_DOT1            0X00000001
 #define KEY_DOT2            0X00000002
 #define KEY_DOT3            0X00000004
@@ -342,7 +355,7 @@ writePacket (
   }
 
   if (debugPackets)
-    LogBytes("Output Packet", (unsigned char *)&packet, size);
+    LogBytes(LOG_DEBUG, "Output Packet", (unsigned char *)&packet, size);
   return io->writePacket(&packet, size, &brl->writeDelay);
 }
 
@@ -516,7 +529,7 @@ readPacket (BrailleDisplay *brl, Packet *packet) {
     }
 
     if (size <= inputCount) {
-      if (debugPackets) LogBytes("Input Packet", inputBuffer.bytes, size);
+      if (debugPackets) LogBytes(LOG_DEBUG, "Input Packet", inputBuffer.bytes, size);
 
       if (hasPayload) {
         unsigned char checksum = 0;
@@ -542,7 +555,7 @@ readPacket (BrailleDisplay *brl, Packet *packet) {
         } else if ((count == 0) && (inputCount > 0)) {
           if (io->awaitInput(1000)) goto retry;
           if (errno != EAGAIN) count = -1;
-          LogBytes("Aborted Input", inputBuffer.bytes, inputCount);
+          LogBytes(LOG_WARNING, "Aborted Input", inputBuffer.bytes, inputCount);
           inputCount = 0;
         }
         return count;
@@ -560,13 +573,13 @@ readPacket (BrailleDisplay *brl, Packet *packet) {
           if (memchr(packets, inputBuffer.bytes[first], sizeof(packets)))
             break;
         if (first) {
-          LogBytes("Discarded Input", inputBuffer.bytes, first);
+          LogBytes(LOG_WARNING, "Discarded Input", inputBuffer.bytes, first);
           memmove(&inputBuffer.bytes[0], &inputBuffer.bytes[first], count-=first);
         }
       }
 
       if (debugPackets)
-        LogBytes("Input Bytes", &inputBuffer.bytes[inputCount], count);
+        LogBytes(LOG_DEBUG, "Input Bytes", &inputBuffer.bytes[inputCount], count);
       inputCount += count;
     }
   }
@@ -633,25 +646,22 @@ getPacket (BrailleDisplay *brl, Packet *packet) {
   }
 }
 
-static void
-brl_identify (void) {
-  LogPrint(LOG_NOTICE, "Freedom Scientific Driver");
-  LogPrint(LOG_INFO, "   Copyright (C) 2004 by Dave Mielke <dave@mielke.cc>");
-}
-
 static int
-brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
-  validateYesNo(&debugPackets, "debug packets", parameters[PARM_DEBUGPACKETS]);
+brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
+  if (!validateYesNo(&debugPackets, parameters[PARM_DEBUGPACKETS]))
+    LogPrint(LOG_WARNING, "%s: %s", "invalid debug packets setting", parameters[PARM_DEBUGPACKETS]);
 
   if (isSerialDevice(&device)) {
     io = &serialOperations;
+  } else
 
 #ifdef ENABLE_USB_SUPPORT
-  } else if (isUsbDevice(&device)) {
+  if (isUsbDevice(&device)) {
     io = &usbOperations;
+  } else
 #endif /* ENABLE_USB_SUPPORT */
 
-  } else {
+  {
     unsupportedDevice(device);
     return 0;
   }
@@ -743,8 +753,10 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
                   int maximum = textCells / 2;
                   int minimum = -maximum;
                   int value;
-                  if (validateInteger(&value, "status cells specification", word, &minimum, &maximum)) {
+                  if (validateInteger(&value, word, &minimum, &maximum)) {
                     cells = value;
+                  } else {
+                    LogPrint(LOG_WARNING, "%s: %s", "invalid status cells specification", word);
                   }
                 }
 
@@ -811,7 +823,7 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
 }
 
 static void
-brl_close (BrailleDisplay *brl) {
+brl_destruct (BrailleDisplay *brl) {
   while (io->awaitInput(100)) {
     Packet packet;
     int count = getPacket(brl, &packet);
@@ -833,7 +845,7 @@ brl_writeStatus (BrailleDisplay *brl, const unsigned char *status) {
 
 static int
 interpretKeys (void) {
-  int keys = realKeys | virtualKeys;
+  unsigned int keys = realKeys | virtualKeys;
   int press = (keys & pressedKeys) != keys;
   int command = BRL_CMD_NOOP;
   int flags = 0;
@@ -1075,15 +1087,15 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
         int command = BRL_CMD_NOOP;
 
         if (row == model->hotkeysRow) {
-          static int keys[] = {
+          static unsigned int keys[] = {
             KEY_GDF_LEFT,
             KEY_HOT1, KEY_HOT2, KEY_HOT3, KEY_HOT4,
             KEY_HOT5, KEY_HOT6, KEY_HOT7, KEY_HOT8,
             KEY_GDF_RIGHT
           };
-          static const int keyCount = sizeof(keys) / sizeof(keys[0]);
+          static const int keyCount = ARRAY_COUNT(keys);
 
-          int key;
+          unsigned int key;
           button -= (model->totalCells - keyCount) / 2;
 
           if (button < 0) {
@@ -1263,13 +1275,13 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
 }
 
 static ssize_t
-brl_readPacket (BrailleDisplay *brl, unsigned char *buffer, size_t length) {
+brl_readPacket (BrailleDisplay *brl, void *buffer, size_t length) {
   Packet packet;
   int count = readPacket(brl, &packet);
   if (count > 0) {
     if (count > sizeof(packet.header)) count--;
     if (length < count) {
-      LogPrint(LOG_WARNING, "Input packet buffer too small: %d < %d", length, count);
+      LogPrint(LOG_WARNING, "Input packet buffer too small: %d < %d", (int)length, count);
       count = length;
     }
     memcpy(buffer, &packet, count);
@@ -1278,23 +1290,24 @@ brl_readPacket (BrailleDisplay *brl, unsigned char *buffer, size_t length) {
 }
 
 static ssize_t
-brl_writePacket (BrailleDisplay *brl, const unsigned char *buffer, size_t length) {
+brl_writePacket (BrailleDisplay *brl, const void *packet, size_t length) {
+  const unsigned char *bytes = packet;
   int size = 4;
   if (length >= size) {
     int hasPayload = 0;
-    if (buffer[0] & 0X80) {
-      size += buffer[1];
+    if (bytes[0] & 0X80) {
+      size += bytes[1];
       hasPayload = 1;
     }
     if (length >= size) {
       if (length > size)
         LogPrint(LOG_WARNING, "Output packet buffer larger than necessary: %d > %d",
-                 length, size);
-      return writePacket(brl, buffer[0], buffer[1], buffer[2], buffer[3],
-                         (hasPayload? &buffer[4]: NULL));
+                 (int)length, size);
+      return writePacket(brl, bytes[0], bytes[1], bytes[2], bytes[3],
+                         (hasPayload? &bytes[4]: NULL));
     }
   }
-  LogPrint(LOG_WARNING, "Output packet buffer too small: %d < %d", length, size);
+  LogPrint(LOG_WARNING, "Output packet buffer too small: %d < %d", (int)length, size);
   return 0;
 }
 
