@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -17,14 +17,11 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 
-#include "Programs/misc.h"
-#include "Programs/at2.h"
+#include "misc.h"
 
 typedef enum {
   PARM_INPUTMODE
@@ -33,125 +30,179 @@ typedef enum {
 
 #define BRLSTAT ST_AlvaStyle
 #define BRL_HAVE_PACKET_IO
-#include "Programs/brl_driver.h"
+#define BRL_HAVE_SENSITIVITY
+#include "brl_driver.h"
+#include "touch.h"
 #include "braille.h"
-
-/* Communication codes */
-static unsigned char HandyDescribe[] = {0XFF};
-static unsigned char HandyDescription[] = {0XFE};
-static unsigned char HandyBrailleStart[] = {0X01};	/* general header to display braille */
-static unsigned char BookwormBrailleEnd[] = {0X16};	/* bookworm trailer to display braille */
-static unsigned char BookwormStop[] = {0X05, 0X07};	/* bookworm trailer to display braille */
+#include "brldefs-ht.h"
 
 typedef struct {
-  unsigned long int front;
-  int column;
-  int status;
+  uint32_t front;
+  signed char column;
+  signed char status;
 } Keys;
-static Keys currentKeys, pressedKeys, nullKeys;
+static Keys currentKeys, pressedKeys;
+static const Keys nullKeys = {.front=0, .column=-1, .status=-1};
 static unsigned int inputMode;
 
-/* Braille display parameters */
+static const unsigned char BookwormSessionEnd[] = {0X05, 0X07};	/* bookworm trailer to display braille */
+
 typedef int (ByteInterpreter) (BRL_DriverCommandContext context, unsigned char byte, int *command);
 static ByteInterpreter interpretKeyByte;
 static ByteInterpreter interpretBookwormByte;
+
 typedef int (KeysInterpreter) (BRL_DriverCommandContext context, const Keys *keys, int *command);
 static KeysInterpreter interpretModularKeys;
 static KeysInterpreter interpretBrailleWaveKeys;
 static KeysInterpreter interpretBrailleStarKeys;
+
+typedef int (CellWriter) (BrailleDisplay *brl);
+static CellWriter writeStatusAndTextCells;
+static CellWriter writeBookwormCells;
+static CellWriter writeEvolutionCells;
+
 typedef struct {
   const char *name;
-  unsigned char identifier;
-  unsigned char columns;
-  unsigned char statusCells;
-  unsigned char helpPage;
   ByteInterpreter *interpretByte;
   KeysInterpreter *interpretKeys;
-  unsigned char *brailleStartAddress;
-  unsigned char *brailleEndAddress;
-  unsigned char *stopAddress;
-  unsigned char brailleStartLength;
-  unsigned char brailleEndLength;
-  unsigned char stopLength;
-} ModelDescription;
-static const ModelDescription Models[] = {
-  {
-    "Modular 20+4", 0X80,
-    20, 4, 0,
-    interpretKeyByte, interpretModularKeys,
-    HandyBrailleStart,         NULL, NULL,
-    sizeof(HandyBrailleStart), 0,    0
+  CellWriter *writeCells;
+
+  const unsigned char *sessionEndAddress;
+
+  HT_ModelIdentifier identifier:8;
+  unsigned char textCells;
+  unsigned char statusCells;
+  unsigned char helpPage;
+
+  unsigned char sessionEndLength;
+
+  unsigned hasATC:1; /* Active Tactile Control */
+  unsigned isBookworm:1;
+} ModelEntry;
+
+#define HT_BYTE_SEQUENCE(name,bytes) .name##Address = bytes, .name##Length = sizeof(bytes)
+static const ModelEntry modelTable[] = {
+  { .identifier = HT_Model_Modular20,
+    .name = "Modular 20+4",
+    .textCells = 20,
+    .statusCells = 4,
+    .helpPage = 0,
+    .interpretByte = interpretKeyByte,
+    .interpretKeys = interpretModularKeys,
+    .writeCells = writeStatusAndTextCells
   }
   ,
-  {
-    "Modular 40+4", 0X89,
-    40, 4, 0,
-    interpretKeyByte, interpretModularKeys,
-    HandyBrailleStart,         NULL, NULL,
-    sizeof(HandyBrailleStart), 0,    0
+  { .identifier = HT_Model_Modular40,
+    .name = "Modular 40+4",
+    .textCells = 40,
+    .statusCells = 4,
+    .helpPage = 0,
+    .interpretByte = interpretKeyByte,
+    .interpretKeys = interpretModularKeys,
+    .writeCells = writeStatusAndTextCells
   }
   ,
-  {
-    "Modular 80+4", 0X88,
-    80, 4, 0,
-    interpretKeyByte, interpretModularKeys,
-    HandyBrailleStart,         NULL, NULL,
-    sizeof(HandyBrailleStart), 0,    0
+  { .identifier = HT_Model_Modular80,
+    .name = "Modular 80+4",
+    .textCells = 80,
+    .statusCells = 4,
+    .helpPage = 0,
+    .interpretByte = interpretKeyByte,
+    .interpretKeys = interpretModularKeys,
+    .writeCells = writeStatusAndTextCells
   }
   ,
-  {
-    "Braille Wave 40", 0X05,
-    40, 0, 0,
-    interpretKeyByte, interpretBrailleWaveKeys,
-    HandyBrailleStart,         NULL, NULL,
-    sizeof(HandyBrailleStart), 0,    0
+  { .identifier = HT_Model_ModularEvolution64,
+    .name = "Modular Evolution 64",
+    .textCells = 60,
+    .statusCells = 4,
+    .helpPage = 0,
+    .interpretByte = interpretKeyByte,
+    .interpretKeys = interpretBrailleStarKeys,
+    .writeCells = writeEvolutionCells,
+    .hasATC = 1
   }
   ,
-  {
-    "Bookworm", 0X90,
-    8, 0, 1,
-    interpretBookwormByte, NULL,
-    HandyBrailleStart,         BookwormBrailleEnd,         BookwormStop,
-    sizeof(HandyBrailleStart), sizeof(BookwormBrailleEnd), sizeof(BookwormStop)
+  { .identifier = HT_Model_ModularEvolution88,
+    .name = "Modular Evolution 88",
+    .textCells = 80,
+    .statusCells = 8,
+    .helpPage = 0,
+    .interpretByte = interpretKeyByte,
+    .interpretKeys = interpretBrailleStarKeys,
+    .writeCells = writeEvolutionCells,
+    .hasATC = 1
   }
   ,
-  {
-    "Braillino 20", 0X72,
-    20, 0, 2,
-    interpretKeyByte, interpretBrailleStarKeys,
-    HandyBrailleStart,         NULL, NULL,
-    sizeof(HandyBrailleStart), 0,    0 
+  { .identifier = HT_Model_BrailleWave,
+    .name = "Braille Wave 40",
+    .textCells = 40,
+    .statusCells = 0,
+    .helpPage = 0,
+    .interpretByte = interpretKeyByte,
+    .interpretKeys = interpretBrailleWaveKeys,
+    .writeCells = writeStatusAndTextCells
   }
   ,
-  {
-    "Braille Star 40", 0X74,
-    40, 0, 2,
-    interpretKeyByte, interpretBrailleStarKeys,
-    HandyBrailleStart,         NULL, NULL,
-    sizeof(HandyBrailleStart), 0,    0
+  { .identifier = HT_Model_Bookworm,
+    .name = "Bookworm",
+    .isBookworm = 1,
+    .textCells = 8,
+    .statusCells = 0,
+    .helpPage = 1,
+    .interpretByte = interpretBookwormByte,
+    .writeCells = writeBookwormCells,
+    HT_BYTE_SEQUENCE(sessionEnd, BookwormSessionEnd)
   }
   ,
-  {
-    "Braille Star 80", 0X78,
-    80, 0, 3,
-    interpretKeyByte, interpretBrailleStarKeys,
-    HandyBrailleStart,         NULL, NULL,
-    sizeof(HandyBrailleStart), 0,    0
+  { .identifier = HT_Model_Braillino,
+    .name = "Braillino 20",
+    .textCells = 20,
+    .statusCells = 0,
+    .helpPage = 2,
+    .interpretByte = interpretKeyByte,
+    .interpretKeys = interpretBrailleStarKeys,
+    .writeCells = writeStatusAndTextCells
+  }
+  ,
+  { .identifier = HT_Model_BrailleStar40,
+    .name = "Braille Star 40",
+    .textCells = 40,
+    .statusCells = 0,
+    .helpPage = 2,
+    .interpretByte = interpretKeyByte,
+    .interpretKeys = interpretBrailleStarKeys,
+    .writeCells = writeStatusAndTextCells
+  }
+  ,
+  { .identifier = HT_Model_BrailleStar80,
+    .name = "Braille Star 80",
+    .textCells = 80,
+    .statusCells = 0,
+    .helpPage = 3,
+    .interpretByte = interpretKeyByte,
+    .interpretKeys = interpretBrailleStarKeys,
+    .writeCells = writeStatusAndTextCells
+  }
+  ,
+  { .identifier = HT_Model_EasyBraille,
+    .name = "Easy Braille",
+    .textCells = 40,
+    .statusCells = 0,
+    .helpPage = 4,
+    .interpretByte = interpretKeyByte,
+    .interpretKeys = interpretModularKeys,
+    .writeCells = writeStatusAndTextCells
   }
   ,
   { /* end of table */
-    NULL, 0,
-    0, 0, 0,
-    NULL, NULL,
-    NULL, NULL, NULL,
-    0,    0,    0
+    .name = NULL
   }
 };
+#undef HT_BYTE_SEQUENCE
 
 #define BRLROWS		1
-#define MAX_STCELLS	4	/* highest number of status cells */
-
-
+#define MAX_STCELLS	8	/* highest number of status cells */
 
 /* This is the brltty braille mapping standard to Handy's mapping table.
  */
@@ -162,7 +213,7 @@ static unsigned char *rawData = NULL;		/* translated data to send to Braille */
 static unsigned char *prevData = NULL;	/* previously sent raw data */
 static unsigned char rawStatus[MAX_STCELLS];		/* to hold status info */
 static unsigned char prevStatus[MAX_STCELLS];	/* to hold previous status */
-static const ModelDescription *model;		/* points to terminal model config struct */
+static const ModelEntry *model;		/* points to terminal model config struct */
 
 static unsigned char *at2Buffer;
 static int at2Size;
@@ -181,7 +232,7 @@ static const int baud = 19200;
 static int charactersPerSecond;
 
 /* Serial IO */
-#include "Programs/io_serial.h"
+#include "io_serial.h"
 
 static SerialDevice *serialDevice = NULL;			/* file descriptor for Braille display */
 
@@ -215,7 +266,7 @@ readSerialBytes (unsigned char *buffer, int count, int wait) {
 static int
 writeSerialBytes (const unsigned char *buffer, int length, unsigned int *delay) {
   int count = serialWriteData(serialDevice, buffer, length);
-  if (delay && (count != -1)) *delay += length * 1000 / charactersPerSecond;
+  if (delay && (count != -1)) *delay += (length * 1000 / charactersPerSecond) + 1;
   return count;
 }
 
@@ -234,16 +285,36 @@ static const InputOutputOperations serialOperations = {
 
 #ifdef ENABLE_USB_SUPPORT
 /* USB IO */
-#include "Programs/io_usb.h"
+#include "io_usb.h"
 
 static UsbChannel *usb = NULL;
 
 static int
 openUsbPort (char **parameters, const char *device) {
+  const SerialParameters serial = {
+    .baud = baud,
+    .flow = SERIAL_FLOW_NONE,
+    .data = 8,
+    .stop = 1,
+    .parity = SERIAL_PARITY_ODD
+  };
+
   const UsbChannelDefinition definitions[] = {
-    {0X0921, 0X1200, 1, 0, 0, 1, 1, baud, 0, 8, 1, SERIAL_PARITY_ODD}, /* GoHubs chip */
-    {0X0403, 0X6001, 1, 0, 0, 1, 2, baud, 0, 8, 1, SERIAL_PARITY_ODD}, /* FTDI chip */
-    {0}
+    { /* GoHubs chip */
+      .vendor=0X0921, .product=0X1200,
+      .configuration=1, .interface=0, .alternative=0,
+      .inputEndpoint=1, .outputEndpoint=1,
+      .serial = &serial
+    }
+    ,
+    { /* FTDI chip */
+      .vendor=0X0403, .product=0X6001,
+      .configuration=1, .interface=0, .alternative=0,
+      .inputEndpoint=1, .outputEndpoint=2,
+      .serial = &serial
+    }
+    ,
+    { .vendor=0 }
   };
 
   if ((usb = usbFindChannel(definitions, (void *)device))) {
@@ -270,7 +341,7 @@ readUsbBytes (unsigned char *buffer, int length, int wait) {
 
 static int
 writeUsbBytes (const unsigned char *buffer, int length, unsigned int *delay) {
-  if (delay) *delay += length * 1000 / charactersPerSecond;
+  if (delay) *delay += (length * 1000 / charactersPerSecond) + 1;
   return usbWriteEndpoint(usb->device, usb->definition.outputEndpoint, buffer, length, 1000);
 }
 
@@ -290,14 +361,14 @@ static const InputOutputOperations usbOperations = {
 
 #ifdef ENABLE_BLUETOOTH_SUPPORT
 /* Bluetooth IO */
-#include "Programs/io_bluetooth.h"
-#include "Programs/io_misc.h"
+#include "io_bluetooth.h"
+#include "io_misc.h"
 
 static int bluetoothConnection = -1;
 
 static int
 openBluetoothPort (char **parameters, const char *device) {
-  return (bluetoothConnection = openRfcommConnection(device, 1)) != -1;
+  return (bluetoothConnection = btOpenConnection(device, 1, 0)) != -1;
 }
 
 static int
@@ -316,7 +387,7 @@ readBluetoothBytes (unsigned char *buffer, int length, int wait) {
 static int
 writeBluetoothBytes (const unsigned char *buffer, int length, unsigned int *delay) {
   int count = writeData(bluetoothConnection, buffer, length);
-  if (delay) *delay += length * 1000 / charactersPerSecond;
+  if (delay) *delay += (length * 1000 / charactersPerSecond) + 1;
   if (count != length) {
     if (count == -1) {
       LogError("HandyTech Bluetooth write");
@@ -339,10 +410,10 @@ static const InputOutputOperations bluetoothOperations = {
 };
 #endif /* ENABLE_BLUETOOTH_SUPPORT */
 
+#define SYN 0X16
+
 typedef enum {
   BDS_OFF,
-  BDS_RESETTING,
-  BDS_IDENTIFYING,
   BDS_READY,
   BDS_WRITING
 } BrailleDisplayState;
@@ -355,7 +426,7 @@ static unsigned char updateRequired = 0;
 #define KEY_RELEASE 0X80
 #define KEY_ROUTING 0X20
 #define KEY_STATUS  0X70
-#define KEY(code)   (1 << (code))
+#define KEY(code)   (UINT32_C(1) << (code))
 
 /* modular front keys */
 #define KEY_B1              KEY(0X03)
@@ -369,7 +440,7 @@ static unsigned char updateRequired = 0;
 #define KEY_UP              KEY(0X04)
 #define KEY_DOWN            KEY(0X08)
 
-/* modular keypad keys */
+/* keypad keys */
 #define KEY_B12             KEY(0X01)
 #define KEY_ZERO            KEY(0X05)
 #define KEY_B13             KEY(0X09)
@@ -408,12 +479,6 @@ static unsigned char updateRequired = 0;
 #define BWK_ENTER    0X04
 #define BWK_FORWARD  0X08
 
-static void
-brl_identify (void) {
-  LogPrint(LOG_NOTICE, "Handy Tech Driver, version 0.3");
-  LogPrint(LOG_INFO, "  Copyright (C) 2000 by Andreas Gross <andi.gross@gmx.de>");
-}
-
 static void
 setState (BrailleDisplayState state) {
   if (state == currentState) {
@@ -426,19 +491,10 @@ setState (BrailleDisplayState state) {
   // LogPrint(LOG_DEBUG, "State: %d+%d", currentState, retryCount);
 }
 
-static int
-readByte (BrailleDisplay *brl, unsigned char *byte, int wait) {
-  return io->readBytes(byte, sizeof(*byte), wait);
-}
-
 static int
 brl_reset (BrailleDisplay *brl) {
-  return 0;
-}
-
-static int
-writeDescribe (BrailleDisplay *brl) {
-  return io->writeBytes(HandyDescribe, sizeof(HandyDescribe), &brl->writeDelay) != -1;
+  static const unsigned char packet[] = {HT_PKT_Reset};
+  return io->writeBytes(packet, sizeof(packet), &brl->writeDelay) != -1;
 }
 
 static void
@@ -469,36 +525,34 @@ reallocateBuffer (unsigned char **buffer, size_t size) {
 static int
 identifyModel (BrailleDisplay *brl, unsigned char identifier) {
   for (
-    model = Models;
+    model = modelTable;
     model->name && (model->identifier != identifier);
     model++
   );
+
   if (!model->name) {
     LogPrint(LOG_ERR, "Detected unknown HandyTech model with ID %02X.",
              identifier);
-    LogPrint(LOG_WARNING, "Please fix Models[] in HandyTech/braille.c and notify the maintainer.");
     return 0;
   }
+
   LogPrint(LOG_INFO, "Detected %s: %d data %s, %d status %s.",
            model->name,
-           model->columns, (model->columns == 1)? "cell": "cells",
+           model->textCells, (model->textCells == 1)? "cell": "cells",
            model->statusCells, (model->statusCells == 1)? "cell": "cells");
 
   brl->helpPage = model->helpPage;		/* position in the model list */
-  brl->x = model->columns;			/* initialise size of display */
+  brl->x = model->textCells;			/* initialise size of display */
   brl->y = BRLROWS;
 
   if (!reallocateBuffer(&rawData, brl->x*brl->y)) return 0;
   if (!reallocateBuffer(&prevData, brl->x*brl->y)) return 0;
 
-  nullKeys.front = 0;
-  nullKeys.column = -1;
-  nullKeys.status = -1;
   currentKeys = pressedKeys = nullKeys;
   inputMode = 0;
 
   memset(rawStatus, 0, model->statusCells);
-  memset(rawData, 0, model->columns);
+  memset(rawData, 0, model->textCells);
 
   retryCount = 0;
   updateRequired = 0;
@@ -509,8 +563,35 @@ identifyModel (BrailleDisplay *brl, unsigned char identifier) {
 }
 
 static int
-brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
-  AT2_resetState();
+writeExtendedPacket (
+  BrailleDisplay *brl, HT_ExtendedPacketType type,
+  const unsigned char *data, unsigned char size
+) {
+  HT_Packet packet;
+  packet.fields.type = HT_PKT_Extended;
+  packet.fields.data.extended.model = model->identifier;
+  packet.fields.data.extended.length = size + 1; /* type byte is included */
+  packet.fields.data.extended.type = type;
+  memcpy(packet.fields.data.extended.data.bytes, data, size);
+  packet.fields.data.extended.data.bytes[size] = SYN;
+  size += 5; /* EXT, ID, LEN, TYPE, ..., SYN */
+  return io->writeBytes((unsigned char *)&packet, size, &brl->writeDelay) == size;
+}
+
+static int
+setAtcMode (BrailleDisplay *brl, unsigned char value) {
+  const unsigned char data[] = {value};
+  return writeExtendedPacket(brl, HT_EXTPKT_SetAtcMode, data, sizeof(data));
+}
+
+static int
+setAtcSensitivity (BrailleDisplay *brl, unsigned char value) {
+  const unsigned char data[] = {value};
+  return writeExtendedPacket(brl, HT_EXTPKT_SetAtcSensitivity, data, sizeof(data));
+}
+
+static int
+brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
   at2Buffer = NULL;
   at2Size = 0;
   at2Count = 0;
@@ -522,36 +603,47 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
 
   if (isSerialDevice(&device)) {
     io = &serialOperations;
+  } else
 
 #ifdef ENABLE_USB_SUPPORT
-  } else if (isUsbDevice(&device)) {
+  if (isUsbDevice(&device)) {
     io = &usbOperations;
+  } else
 #endif /* ENABLE_USB_SUPPORT */
 
 #ifdef ENABLE_BLUETOOTH_SUPPORT
-  } else if (isBluetoothDevice(&device)) {
+  if (isBluetoothDevice(&device)) {
     io = &bluetoothOperations;
+  } else
 #endif /* ENABLE_BLUETOOTH_SUPPORT */
 
-  } else {
+  {
     unsupportedDevice(device);
     return 0;
   }
 
   rawData = prevData = NULL;		/* clear pointers */
-  charactersPerSecond = baud / 10;
+  charactersPerSecond = baud / 11;
 
   if (io->openPort(parameters, device)) {
     int tries = 0;
-    while (writeDescribe(brl)) {
+    while (brl_reset(brl)) {
       while (io->awaitInput(100)) {
-        unsigned char response[sizeof(HandyDescription) + 1];
-        if (io->readBytes(response, sizeof(response), 0) == sizeof(response)) {
-          if (memcmp(response, HandyDescription, sizeof(HandyDescription)) == 0) {
-            if (identifyModel(brl, response[sizeof(HandyDescription)])) {
+        HT_Packet response;
+        if (brl_readPacket(brl, &response, sizeof(response)) > 0) {
+          if (response.fields.type == HT_PKT_OK) {
+            if (identifyModel(brl, response.fields.data.ok.model)) {
+              if (model->hasATC) {
+                setAtcMode(brl, 1);
+                setAtcSensitivity(brl, 50);
+
+                touchAnalyzeCells(brl, NULL);
+                brl->touchEnabled = 1;
+              }
+
               if (*parameters[PARM_INPUTMODE])
-                validateYesNo(&inputMode, "input mode",
-                              parameters[PARM_INPUTMODE]);
+                if (!validateYesNo(&inputMode, parameters[PARM_INPUTMODE]))
+                  LogPrint(LOG_WARNING, "%s: %s", "invalid input setting", parameters[PARM_INPUTMODE]);
 
               return 1;
             }
@@ -570,9 +662,9 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
 }
 
 static void
-brl_close (BrailleDisplay *brl) {
-  if (model->stopLength) {
-    io->writeBytes(model->stopAddress, model->stopLength, NULL);
+brl_destruct (BrailleDisplay *brl) {
+  if (model->sessionEndLength) {
+    io->writeBytes(model->sessionEndAddress, model->sessionEndLength, NULL);
   }
   io->closePort();
 
@@ -585,53 +677,73 @@ brl_close (BrailleDisplay *brl) {
 }
 
 static int
-updateBrailleCells (BrailleDisplay *brl) {
-  if (updateRequired && (currentState == BDS_READY)) {
-    unsigned char buffer[model->brailleStartLength + model->statusCells + model->columns + model->brailleEndLength];
-    int count = 0;
+writeCells (BrailleDisplay *brl) {
+  if (!model->writeCells(brl)) return 0;
+  setState(BDS_WRITING);
+  return 1;
+}
 
-    if (model->brailleStartLength) {
-      memcpy(buffer+count, model->brailleStartAddress, model->brailleStartLength);
-      count += model->brailleStartLength;
-    }
+static int
+writeStatusAndTextCells (BrailleDisplay *brl) {
+  unsigned char buffer[1 + model->statusCells + model->textCells];
 
-    memcpy(buffer+count, rawStatus, model->statusCells);
-    count += model->statusCells;
+  buffer[0] = 0X01;
+  memcpy(buffer+1, rawStatus, model->statusCells);
+  memcpy(buffer+model->statusCells+1, rawData, model->textCells);
 
-    memcpy(buffer+count, rawData, model->columns);
-    count += model->columns;
+  return io->writeBytes(buffer, sizeof(buffer), &brl->writeDelay) != -1;
+}
 
-    if (model->brailleEndLength) {
-      memcpy(buffer+count, model->brailleEndAddress, model->brailleEndLength);
-      count += model->brailleEndLength;
-    }
+static int
+writeBookwormCells (BrailleDisplay *brl) {
+  unsigned char buffer[1 + model->statusCells + model->textCells + 1];
 
-    // LogBytes("Write", buffer, count);
-    if (io->writeBytes(buffer, count, &brl->writeDelay) == -1) {
-      setState(BDS_OFF);
-      return 0;
-    }
-    setState(BDS_WRITING);
-    updateRequired = 0;
+  buffer[0] = 0X01;
+  memcpy(buffer+1, rawData, model->textCells);
+  buffer[sizeof(buffer)-1] = SYN;
+  return io->writeBytes(buffer, sizeof(buffer), &brl->writeDelay) != -1;
+}
+
+static int
+writeEvolutionCells (BrailleDisplay *brl) {
+  unsigned char buffer[model->textCells + model->statusCells];
+
+  memcpy(buffer, rawData, model->textCells);
+  memcpy(buffer+model->textCells, rawStatus, model->statusCells);
+
+  return writeExtendedPacket(brl, HT_EXTPKT_Braille, buffer, sizeof(buffer));
+}
+
+static int
+updateCells (BrailleDisplay *brl) {
+  if (!updateRequired) return 1;
+  if (currentState != BDS_READY) return 1;
+
+  if (!writeCells(brl)) {
+    setState(BDS_OFF);
+    return 0;
   }
+
+  updateRequired = 0;
   return 1;
 }
 
 static void
 brl_writeWindow (BrailleDisplay *brl) {
-  if (memcmp(brl->buffer, prevData, model->columns) != 0) {
+  if (memcmp(brl->buffer, prevData, model->textCells) != 0) {
     int i;
-    for (i=0; i<model->columns; ++i) {
+    for (i=0; i<model->textCells; ++i) {
       rawData[i] = outputTable[(prevData[i] = brl->buffer[i])];
     }
     updateRequired = 1;
   }
-  updateBrailleCells(brl);
+  updateCells(brl);
 }
 
 static void
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *st) {
-  if (memcmp(st, prevStatus, model->statusCells) != 0) {	/* only if it changed */
+  if (model->statusCells &&
+      (memcmp(st, prevStatus, model->statusCells) != 0)) {
     int i;
     for (i=0; i<model->statusCells; ++i) {
       rawStatus[i] = outputTable[(prevStatus[i] = st[i])];
@@ -649,7 +761,7 @@ interpretKeyByte (BRL_DriverCommandContext context, unsigned char byte, int *com
   currentKeys.status = -1;
 
   if ((byte >= KEY_ROUTING) &&
-      (byte < (KEY_ROUTING + model->columns))) {
+      (byte < (KEY_ROUTING + model->textCells))) {
     *command = BRL_CMD_NOOP;
     if (!release) {
       currentKeys.column = byte - KEY_ROUTING;
@@ -673,7 +785,7 @@ interpretKeyByte (BRL_DriverCommandContext context, unsigned char byte, int *com
   }
 
   if (byte < 0X20) {
-    unsigned long int key = KEY(byte);
+    uint32_t key = KEY(byte);
     *command = BRL_CMD_NOOP;
     if (release) {
       currentKeys.front &= ~key;
@@ -905,9 +1017,9 @@ interpretModularKeys (BRL_DriverCommandContext context, const Keys *keys, int *c
     }
 
     if (inputMode) {
-      const unsigned long int dots = KEY_B1 | KEY_B2 | KEY_B3 | KEY_B4 | KEY_B5 | KEY_B6 | KEY_B7 | KEY_B8;
+      const uint32_t dots = KEY_B1 | KEY_B2 | KEY_B3 | KEY_B4 | KEY_B5 | KEY_B6 | KEY_B7 | KEY_B8;
       if (keys->front & dots) {
-        unsigned long int modifiers = keys->front & ~dots;
+        uint32_t modifiers = keys->front & ~dots;
         *command = BRL_BLK_PASSDOTS;
 
         if (keys->front & KEY_B1) *command |= BRL_DOT7;
@@ -1141,32 +1253,54 @@ interpretBrailleStarKeys (BRL_DriverCommandContext context, const Keys *keys, in
         *command = BRL_CMD_FWINRT;
         return 1;
       case (ROCKER_LEFT_MIDDLE | ROCKER_RIGHT_MIDDLE):
+      case (ROCKER_LEFT_MIDDLE | KEY_B3):
+      case (KEY_B6 | ROCKER_RIGHT_MIDDLE):
         *command = BRL_CMD_HOME;
         return 1;
       case (ROCKER_RIGHT_MIDDLE | ROCKER_LEFT_TOP):
+      case (ROCKER_RIGHT_MIDDLE | KEY_B5):
+      case (KEY_B3 | ROCKER_LEFT_TOP):
         *command = BRL_CMD_TOP_LEFT;
         return 1;
       case (ROCKER_RIGHT_MIDDLE | ROCKER_LEFT_BOTTOM):
+      case (ROCKER_RIGHT_MIDDLE | KEY_B7):
+      case (KEY_B3 | ROCKER_LEFT_BOTTOM):
         *command = BRL_CMD_BOT_LEFT;
         return 1;
       case (ROCKER_LEFT_MIDDLE | ROCKER_RIGHT_TOP):
+      case (ROCKER_LEFT_MIDDLE | KEY_B4):
+      case (KEY_B6 | ROCKER_RIGHT_TOP):
         *command = BRL_CMD_TOP;
         return 1;
       case (ROCKER_LEFT_MIDDLE | ROCKER_RIGHT_BOTTOM):
+      case (ROCKER_LEFT_MIDDLE | KEY_B2):
+      case (KEY_B6 | ROCKER_RIGHT_BOTTOM):
         *command = BRL_CMD_BOT;
         return 1;
       case (ROCKER_LEFT_TOP | ROCKER_RIGHT_TOP):
+      case (ROCKER_LEFT_TOP | KEY_B4):
+      case (KEY_B5 | ROCKER_RIGHT_TOP):
         *command = BRL_CMD_PRDIFLN;
         return 1;
       case (ROCKER_LEFT_TOP | ROCKER_RIGHT_BOTTOM):
+      case (ROCKER_LEFT_TOP | KEY_B2):
+      case (KEY_B5 | ROCKER_RIGHT_BOTTOM):
         *command = BRL_CMD_NXDIFLN;
         return 1;
       case (ROCKER_LEFT_BOTTOM | ROCKER_RIGHT_TOP):
+      case (ROCKER_LEFT_BOTTOM | KEY_B4):
+      case (KEY_B7 | ROCKER_RIGHT_TOP):
         *command = BRL_CMD_ATTRUP;
         return 1;
       case (ROCKER_LEFT_BOTTOM | ROCKER_RIGHT_BOTTOM):
+      case (ROCKER_LEFT_BOTTOM | KEY_B2):
+      case (KEY_B7 | ROCKER_RIGHT_BOTTOM):
         *command = BRL_CMD_ATTRDN;
         return 1;
+      case (KEY_B2 | KEY_B4 | KEY_B5 | KEY_SPACE_LEFT):
+        setState(BDS_OFF);
+        *command = EOF;
+        return 1;
     }
   }
   if (!(keys->front & ~(KEY_B1 | KEY_B2 | KEY_B3 | KEY_B4 |
@@ -1295,107 +1429,173 @@ interpretBookwormByte (BRL_DriverCommandContext context, unsigned char byte, int
 
 static int
 brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
-  int timedOut = 1;
+  int noInput = 1;
 
   if (at2Count) {
     unsigned char code = at2Buffer[0];
     memcpy(at2Buffer, at2Buffer+1, --at2Count);
-    return BRL_BLK_PASSAT2 + code;
+    return BRL_BLK_PASSAT + code;
   }
 
   while (1) {
-    unsigned char byte;
-    {
-      int count = readByte(brl, &byte, 0);
-      if (count == -1) return BRL_CMD_RESTARTBRL;
-      if (count == 0) break;
-    }
-    timedOut = 0;
-    /* LogPrint(LOG_DEBUG, "Read: %02X", byte); */
+    HT_Packet packet;
+    int size = brl_readPacket(brl, &packet, sizeof(packet));
 
-    if (byte == 0X06) {
-      if (currentState != BDS_OFF) {
-        if (io->awaitInput(10)) {
-          setState(BDS_OFF);
-          continue;
+    if (size == -1) return BRL_CMD_RESTARTBRL;
+    if (size == 0) break;
+    noInput = 0;
+
+    /* a kludge to handle the Bookworm going offline */
+    if (model->isBookworm) {
+      if (packet.fields.type == 0X06) {
+        if (currentState != BDS_OFF) {
+          /* if we get another byte right away then the device
+           * has gone offline and is echoing its display
+           */
+          if (io->awaitInput(10)) {
+            setState(BDS_OFF);
+            continue;
+          }
+
+          /* if an input error occurred then restart the driver */
+          if (errno != EAGAIN) return BRL_CMD_RESTARTBRL;
+
+          /* no additional input so fall through and interpret the packet as keys */
         }
-        if (errno != EAGAIN) return BRL_CMD_RESTARTBRL;
       }
     }
 
-    switch (byte) {
-      case 0XFE:
-        setState(BDS_IDENTIFYING);
-        continue;
+    switch (packet.fields.type) {
+      case HT_PKT_OK:
+        if (packet.fields.data.ok.model == model->identifier) {
+          setState(BDS_READY);
+          updateRequired = 1;
+          currentKeys = pressedKeys = nullKeys;
+          continue;
+        }
+        break;
+
       default:
         switch (currentState) {
           case BDS_OFF:
             continue;
-          case BDS_RESETTING:
-            break;
-          case BDS_IDENTIFYING:
-            if (byte == model->identifier) {
-              setState(BDS_READY);
-              updateRequired = 1;
-              currentKeys = pressedKeys = nullKeys;
-              continue;
-            }
-            break;
+
           case BDS_WRITING:
-            switch (byte) {
-              case 0X7D:
+            switch (packet.fields.type) {
+              case HT_PKT_NAK:
                 updateRequired = 1;
-              case 0X7E:
+              case HT_PKT_ACK:
+                if (model->hasATC) touchAnalyzeCells(brl, prevData);
                 setState(BDS_READY);
                 continue;
+
+              case HT_PKT_Extended: {
+                unsigned char length UNUSED = packet.fields.data.extended.length - 1;
+                const unsigned char *bytes = &packet.fields.data.extended.data.bytes[0];
+
+                switch (packet.fields.data.extended.type) {
+                  case HT_EXTPKT_Confirmation:
+                    switch (bytes[0]) {
+                      case HT_PKT_NAK:
+                        updateRequired = 1;
+                      case HT_PKT_ACK:
+                        if (model->hasATC) touchAnalyzeCells(brl, prevData);
+                        setState(BDS_READY);
+                        continue;
+
+                      default:
+                        break;
+                    }
+                    break;
+
+                  default:
+                    break;
+                }
+                break;
+              }
+
               default:
                 break;
             }
-          case BDS_READY:
-            switch (byte) {
-              case 0X79: {
-                unsigned char header[2];
-                io->readBytes(header, sizeof(header), 1);
-                if (header[0] == model->identifier) {
-                  unsigned char length = header[1];
-                  unsigned char data[length+1];
-                  io->readBytes(data, sizeof(data), 1);
-                  if (data[length] == 0X16) {
-                    const unsigned char *bytes = data + 1;
-                    length--;
 
-                    switch (data[0]) {
-                      case 0X09: {
-                        if (length) {
-                          unsigned char code = *bytes++;
-                          if (--length) {
-                            int newCount = at2Count + length;
-                            if (newCount > at2Size) {
-                              int newSize = (newCount | 0XF) + 1;
-                              at2Buffer = reallocWrapper(at2Buffer, newSize);
-                              at2Size = newSize;
-                            }
-                            memcpy(at2Buffer+at2Count, bytes, length);
-                            at2Count = newCount;
-                          }
-                          return BRL_BLK_PASSAT2 + code;
-                        }
-                        break;
-                      }
+          case BDS_READY:
+            switch (packet.fields.type) {
+              case HT_PKT_Extended: {
+                unsigned char length = packet.fields.data.extended.length - 1;
+                const unsigned char *bytes = &packet.fields.data.extended.data.bytes[0];
+
+                switch (packet.fields.data.extended.type) {
+                  case HT_EXTPKT_Key: {
+                    int command;
+                    if (model->interpretByte(context, bytes[0], &command)) {
+                      updateCells(brl);
+                      return command;
                     }
-                  } else {
-                    LogBytes("Malformed keycode packet", data, sizeof(data));
+                    break;
                   }
-                } else {
-                  LogError("Keycode packet ID mismatch");
+
+                  case HT_EXTPKT_Scancode: {
+                    if (length) {
+                      unsigned char code = bytes++[0];
+                      if (--length) {
+                        int newCount = at2Count + length;
+                        if (newCount > at2Size) {
+                          int newSize = (newCount | 0XF) + 1;
+                          at2Buffer = reallocWrapper(at2Buffer, newSize);
+                          at2Size = newSize;
+                        }
+                        memcpy(at2Buffer+at2Count, bytes, length);
+                        at2Count = newCount;
+                      }
+                      return BRL_BLK_PASSAT + code;
+                    }
+                    break;
+                  }
+
+                  case HT_EXTPKT_AtcInfo: {
+                    unsigned int cellCount = model->textCells + model->statusCells;
+                    unsigned char pressureValues[cellCount];
+                    const unsigned char *pressure;
+
+                    if (bytes[0]) {
+                      int cellIndex = bytes[0] - 1;
+                      int dataIndex;
+
+                      memset(pressureValues, 0, cellCount);
+                      for (dataIndex=1; dataIndex<length; dataIndex++) {
+                        unsigned char byte = bytes[dataIndex];
+                        unsigned char nibble;
+
+                        nibble = byte & 0XF0;
+                        pressureValues[cellIndex++] = nibble | (nibble >> 4);
+
+                        nibble = byte & 0X0F;
+                        pressureValues[cellIndex++] = nibble | (nibble << 4);
+                      }
+
+                      pressure = &pressureValues[0];
+                    } else {
+                      pressure = NULL;
+                    }
+
+                    {
+                      int command = touchAnalyzePressure(brl, pressure);
+                      if (command != EOF) return command;
+                    }
+
+                    continue;
+                  }
+
+                  default:
+                    break;
                 }
-                continue;
+                break;
               }
 
               default: {
                 int command;
-                if (model->interpretByte(context, byte, &command)) {
-                  updateBrailleCells(brl);
+                if (model->interpretByte(context, packet.fields.type, &command)) {
+                  updateCells(brl);
                   return command;
                 }
                 break;
@@ -1406,61 +1606,116 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
         break;
     }
 
-    LogPrint(LOG_WARNING, "Unexpected byte: %02X (state %d)", byte, currentState);
+    LogPrint(LOG_WARNING, "Unexpected Packet: %02X (state %d)", packet.fields.type, currentState);
   }
 
-  if (timedOut) {
+  if (noInput) {
     switch (currentState) {
       case BDS_OFF:
         break;
-      case BDS_RESETTING:
-        if (millisecondsSince(&stateTime) > 3000) {
-          if (retryCount > 3) {
-            setState(BDS_OFF);
-          } else if (writeDescribe(brl)) {
-            setState(BDS_RESETTING);
-          } else {
-            setState(BDS_OFF);
-          }
-        }
-        break;
-      case BDS_IDENTIFYING:
-        if (millisecondsSince(&stateTime) > 1000) {
-          if (writeDescribe(brl)) {
-            setState(BDS_RESETTING);
-          } else {
-            setState(BDS_OFF);
-          }
-        }
-        break;
+
       case BDS_READY:
         break;
+
       case BDS_WRITING:
         if (millisecondsSince(&stateTime) > 1000) {
-          if (retryCount > 3) {
-            if (writeDescribe(brl)) {
-              setState(BDS_RESETTING);
-            } else {
-              setState(BDS_OFF);
-            }
-          } else {
-            updateRequired = 1;
-          }
+          if (retryCount > 3) return BRL_CMD_RESTARTBRL;
+          if (!writeCells(brl)) return BRL_CMD_RESTARTBRL;
         }
         break;
     }
   }
-  updateBrailleCells(brl);
+  updateCells(brl);
 
   return EOF;
 }
 
 static ssize_t
-brl_readPacket (BrailleDisplay *brl, unsigned char *bytes, size_t count) {
-  return io->readBytes(bytes, count, 0);
+brl_readPacket (BrailleDisplay *brl, void *buffer, size_t size) {
+  static const int logInputPackets = 0;
+  unsigned char *packet = buffer;
+  int offset = 0;
+  int length = 0;
+
+  while (1) {
+    unsigned char byte;
+
+    {
+      int started = offset > 0;
+      int count = io->readBytes(&byte, 1, started);
+
+      if (count != 1) {
+        if (!count && started) LogBytes(LOG_WARNING, "Partial Packet", packet, offset);
+        return count;
+      }
+    }
+
+    if (offset == 0) {
+      switch (byte) {
+        default:
+          length = 1;
+          break;
+
+        case HT_PKT_OK:
+          length = 2;
+          break;
+
+        case HT_PKT_Extended:
+          length = 4;
+          break;
+      }
+    } else {
+      switch (packet[0]) {
+        case HT_PKT_Extended:
+          if (offset == 2) length += byte;
+          break;
+      }
+    }
+
+    if (offset < size) {
+      packet[offset] = byte;
+    } else {
+      if (offset == size) LogBytes(LOG_WARNING, "Truncated Packet", packet, offset);
+      LogBytes(LOG_WARNING, "Discarded Byte", &byte, 1);
+    }
+
+    if (++offset == length) {
+      if (offset <= size) {
+        int ok = 0;
+
+        switch (packet[0]) {
+          case HT_PKT_Extended:
+            if (packet[length-1] == SYN) ok = 1;
+            break;
+
+          default:
+            ok = 1;
+            break;
+        }
+
+        if (ok) {
+          if (logInputPackets) LogBytes(LOG_DEBUG, "Input Packet", packet, offset);
+          return length;
+        }
+
+        LogBytes(LOG_WARNING, "Malformed Packet", packet, offset);
+      }
+
+      offset = 0;
+      length = 0;
+    }
+  }
 }
 
 static ssize_t
-brl_writePacket (BrailleDisplay *brl, const unsigned char *data, size_t length) {
-  return io->writeBytes(data, length, &brl->writeDelay);
+brl_writePacket (BrailleDisplay *brl, const void *packet, size_t length) {
+  return io->writeBytes(packet, length, &brl->writeDelay);
+}
+
+static void
+brl_sensitivity (BrailleDisplay *brl, BrailleSensitivity setting) {
+  if (model->hasATC) {
+    unsigned char value = 0XFF - (setting * 0XF0 / BS_MAXIMUM);
+    setAtcSensitivity(brl, value);
+  }
 }
