@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,8 +22,6 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -31,13 +29,13 @@
 #include <fcntl.h>
 #include <sys/stat.h>
 
-#include "Programs/misc.h"
+#include "misc.h"
 
 #define BRLSTAT ST_Generic
-#include "Programs/brl_driver.h"
-#include "Programs/tbl.h"
+#include "brl_driver.h"
+#include "tbl.h"
 #include "braille.h"
-#include "Programs/io_serial.h"
+#include "io_serial.h"
 
 static SerialDevice *serialDevice = NULL;
 
@@ -321,12 +319,6 @@ static TranslationTable outputTable = {
 };
 static TranslationTable inputTable;
 
-static void
-brl_identify (void) {
-   LogPrint(LOG_NOTICE, "LogText Driver");
-   LogPrint(LOG_INFO, "   Copyright (C) 2001 by Dave Mielke <dave@mielke.cc>");
-}
-
 #ifndef __MINGW32__
 static int
 makeFifo (const char *path, mode_t mode) {
@@ -357,7 +349,7 @@ makeDownloadFifo (void) {
 }
 
 static int
-brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
+brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
    if (memchr(outputTable, 0XFF, sizeof(outputTable))) {
       reverseTranslationTable(outputTable, inputTable);
       outputTable[inputTable[0XFF]] = 0X1A;
@@ -385,7 +377,7 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
 }
 
 static void
-brl_close (BrailleDisplay *brl) {
+brl_destruct (BrailleDisplay *brl) {
    serialCloseDevice(serialDevice);
    serialDevice = NULL;
 }
@@ -429,10 +421,10 @@ sendData (unsigned char line, unsigned char column, unsigned char count) {
    *target++ = (line == cursorRow)? cursorColumn+1: 0;
    *target++ = column + 1;
    *target++ = count;
-   LogBytes("Output dots", source, count);
+   LogBytes(LOG_DEBUG, "Output dots", source, count);
    while (count--) *target++ = outputTable[*source++];
    count = target - data;
-   LogBytes("LogText write", data, count);
+   LogBytes(LOG_DEBUG, "LogText write", data, count);
    if (checkData(data, count)) {
       if (sendBytes(data, count)) {
          return 1;
