@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -35,13 +35,11 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 
-#include "Programs/misc.h"
+#include "misc.h"
 
 typedef enum {
    PARM_CONFIGFILE,
@@ -53,7 +51,8 @@ typedef enum {
 
 #define BRLSTAT ST_Generic
 #define BRLCONST
-#include "Programs/brl_driver.h"
+#define BRL_HAVE_FIRMNESS
+#include "brl_driver.h"
 #include "braille.h"
 
 #ifdef ENABLE_PM_CONFIGURATION_FILE
@@ -71,7 +70,7 @@ static void
 loadConfigurationFile (const char *path) {
   LogPrint(LOG_DEBUG, "Loading Papenmeier configuration file: %s", path);
   if ((configurationFile = fopen(path, "r")) != NULL) {
-    parse();
+    parseConfigurationFile();
     fclose(configurationFile);
     configurationFile = NULL;
   } else {
@@ -116,7 +115,7 @@ compareCommands (const void *item1, const void *item2) {
 
 static void
 sortCommands (void) {
-  qsort(terminal->commands, terminal->commandCount, sizeof(*terminal->commands), compareCommands);
+  qsort(terminal->commandDefinitions, terminal->commandCount, sizeof(*terminal->commandDefinitions), compareCommands);
 }
 
 static int
@@ -128,7 +127,7 @@ findCommand (int *command, int key, int modifiers) {
   ref.modifiers = modifiers;
   while (first <= last) {
     int current = (first + last) / 2;
-    CommandDefinition *cmd = &terminal->commands[current];
+    CommandDefinition *cmd = &terminal->commandDefinitions[current];
     int relation = compareCommands(cmd, &ref);
 
     if (!relation) {
@@ -173,6 +172,10 @@ handleCommand (BrailleDisplay *brl, int cmd, int repeat) {
   if (cmd == BRL_CMD_INPUT) {
     /* translate toggle -> ON/OFF */
     cmd |= inputMode? BRL_FLG_TOGGLE_OFF: BRL_FLG_TOGGLE_ON;
+  } else if ((cmd & BRL_MSK_BLK) == BRL_BLK_GOTOLINE) {
+    int arg = cmd & BRL_MSK_ARG;
+    cmd &= ~BRL_MSK_ARG;
+    cmd |= rescaleInteger(arg, terminal->statusCount-1, BRL_MSK_ARG);
   }
 
   if (!IS_DELAYED_COMMAND(repeat)) {
@@ -194,19 +197,19 @@ handleCommand (BrailleDisplay *brl, int cmd, int repeat) {
         break;
 
       case BRL_CMD_SWSIM_LC:
-        return changeModifiers(MOD_EASY_SLR|MOD_EASY_SLF, MOD_EASY_SLC);
+        return changeModifiers(MOD_BAR_SLR|MOD_BAR_SLF, MOD_BAR_SLC);
       case BRL_CMD_SWSIM_LR:
-        return changeModifiers(MOD_EASY_SLF, MOD_EASY_SLR);
+        return changeModifiers(MOD_BAR_SLF, MOD_BAR_SLR);
       case BRL_CMD_SWSIM_LF:
-        return changeModifiers(MOD_EASY_SLR, MOD_EASY_SLF);
+        return changeModifiers(MOD_BAR_SLR, MOD_BAR_SLF);
       case BRL_CMD_SWSIM_RC:
-        return changeModifiers(MOD_EASY_SRR|MOD_EASY_SRF, MOD_EASY_SRC);
+        return changeModifiers(MOD_BAR_SRR|MOD_BAR_SRF, MOD_BAR_SRC);
       case BRL_CMD_SWSIM_RR:
-        return changeModifiers(MOD_EASY_SRF, MOD_EASY_SRR);
+        return changeModifiers(MOD_BAR_SRF, MOD_BAR_SRR);
       case BRL_CMD_SWSIM_RF:
-        return changeModifiers(MOD_EASY_SRR, MOD_EASY_SRF);
+        return changeModifiers(MOD_BAR_SRR, MOD_BAR_SRF);
       case BRL_CMD_SWSIM_BC:
-        return changeModifiers(MOD_EASY_SLR|MOD_EASY_SLF|MOD_EASY_SRR|MOD_EASY_SRF, MOD_EASY_SLC|MOD_EASY_SRC);
+        return changeModifiers(MOD_BAR_SLR|MOD_BAR_SLF|MOD_BAR_SRR|MOD_BAR_SRF, MOD_BAR_SLC|MOD_BAR_SRC);
       case BRL_CMD_SWSIM_BQ: {
         static const char *const states[] = {"center", "rear", "front", "both"};
         const char *left = states[currentModifiers & 0X3];
@@ -248,7 +251,7 @@ handleModifier (BrailleDisplay *brl, int bit, int press) {
           command |= *dot;
       if (debugKeys)
         LogPrint(LOG_DEBUG, "cmd: [%02X]->%04X", modifiers, command); 
-    } else if (findCommand(&command, NOKEY, modifiers)) {
+    } else if (findCommand(&command, KEY_NONE, modifiers)) {
       if (debugKeys)
         LogPrint(LOG_DEBUG, "cmd: [%04X]->%04X",
                  modifiers, command); 
@@ -265,7 +268,7 @@ handleKey (BrailleDisplay *brl, int code, int press, int offset) {
 
   /* look for modfier keys */
   for (i=0; i<terminal->modifierCount; i++)
-    if (terminal->modifiers[i] == code)
+    if (terminal->modifierKeys[i] == code)
       return handleModifier(brl, 1<<i, press);
 
   /* must be a "normal key" - search for cmd on key press */
@@ -297,8 +300,8 @@ typedef struct {
   void (*closePort) (void);
   void (*flushPort) (BrailleDisplay *brl);
   int (*awaitInput) (int milliseconds);
-  int (*readBytes) (void *buffer, size_t *offset, size_t length, int timeout);
-  int (*writeBytes) (const void *buffer, int length);
+  int (*readBytes) (unsigned char *buffer, size_t *offset, size_t length, int timeout);
+  int (*writeBytes) (const unsigned char *buffer, int length);
 } InputOutputOperations;
 
 static const InputOutputOperations *io;
@@ -307,7 +310,7 @@ static int charactersPerSecond;
 
 /*--- Serial Operations ---*/
 
-#include "Programs/io_serial.h"
+#include "io_serial.h"
 static SerialDevice *serialDevice = NULL;
 static const int serialBauds[] = {19200, 38400, 0};
 
@@ -348,12 +351,12 @@ awaitSerialInput (int milliseconds) {
 }
 
 static int
-readSerialBytes (void *buffer, size_t *offset, size_t length, int timeout) {
+readSerialBytes (unsigned char *buffer, size_t *offset, size_t length, int timeout) {
   return serialReadChunk(serialDevice, buffer, offset, length, 0, timeout);
 }
 
 static int
-writeSerialBytes (const void *buffer, int length) {
+writeSerialBytes (const unsigned char *buffer, int length) {
   return serialWriteData(serialDevice, buffer, length);
 }
 
@@ -366,15 +369,28 @@ static const InputOutputOperations serialOperations = {
 /*--- USB Operations ---*/
 
 #ifdef ENABLE_USB_SUPPORT
-#include "Programs/io_usb.h"
+#include "io_usb.h"
 static UsbChannel *usb = NULL;
-static const int usbBauds[] = {57600, 0};
+static const int usbBauds[] = {115200, 57600, 0};
 
 static int
 openUsbPort (char **parameters, const char *device) {
+  const SerialParameters serial = {
+    .baud = *baud,
+    .flow = SERIAL_FLOW_NONE,
+    .data = 8,
+    .stop = 1,
+    .parity = SERIAL_PARITY_NONE
+  };
+
   const UsbChannelDefinition definitions[] = {
-    {0X0403, 0Xf208, 1, 0, 0, 1, 2, *baud, 0, 8, 1, SERIAL_PARITY_NONE},
-    {0}
+    { .vendor=0X0403, .product=0Xf208,
+      .configuration=1, .interface=0, .alternative=0,
+      .inputEndpoint=1, .outputEndpoint=2,
+      .serial = &serial
+    }
+    ,
+    { .vendor=0 }
   };
 
   if ((usb = usbFindChannel(definitions, (void *)device))) {
@@ -407,7 +423,7 @@ awaitUsbInput (int milliseconds) {
 }
 
 static int
-readUsbBytes (void *buffer, size_t *offset, size_t length, int timeout) {
+readUsbBytes (unsigned char *buffer, size_t *offset, size_t length, int timeout) {
   int count = usbReapInput(usb->device, usb->definition.inputEndpoint, buffer+*offset, length, 
                            (*offset? timeout: 0), timeout);
   if (count == -1) return 0;
@@ -416,7 +432,7 @@ readUsbBytes (void *buffer, size_t *offset, size_t length, int timeout) {
 }
 
 static int
-writeUsbBytes (const void *buffer, int length) {
+writeUsbBytes (const unsigned char *buffer, int length) {
   return usbWriteEndpoint(usb->device, usb->definition.outputEndpoint, buffer, length, 1000);
 }
 
@@ -427,6 +443,64 @@ static const InputOutputOperations usbOperations = {
 };
 #endif /* ENABLE_USB_SUPPORT */
 
+/*--- Bluetooth Operations ---*/
+#ifdef ENABLE_BLUETOOTH_SUPPORT
+#include "io_bluetooth.h"
+#include "io_misc.h"
+
+static int bluetoothConnection = -1;
+static const int bluetoothBauds[] = {115200, 0};
+
+static int
+openBluetoothPort (char **parameters, const char *device) {
+  return (bluetoothConnection = btOpenConnection(device, 1, 0)) != -1;
+}
+
+static int
+prepareBluetoothPort (void) {
+  return 1;
+}
+
+static void
+closeBluetoothPort (void) {
+  close(bluetoothConnection);
+  bluetoothConnection = -1;
+}
+
+static void
+flushBluetoothPort (BrailleDisplay *brl) {
+}
+
+static int
+awaitBluetoothInput (int milliseconds) {
+  return awaitInput(bluetoothConnection, milliseconds);
+}
+
+static int
+readBluetoothBytes (unsigned char *buffer, size_t *offset, size_t length, int timeout) {
+  return readChunk(bluetoothConnection, buffer, offset, length, 0, timeout);
+}
+
+static int
+writeBluetoothBytes (const unsigned char *buffer, int length) {
+  int count = writeData(bluetoothConnection, buffer, length);
+  if (count != length) {
+    if (count == -1) {
+      LogError("Papenmeier Bluetooth write");
+    } else {
+      LogPrint(LOG_WARNING, "Trunccated bluetooth write: %d < %d", count, length);
+    }
+  }
+  return count;
+}
+
+static const InputOutputOperations bluetoothOperations = {
+  bluetoothBauds, 0, 3,
+  openBluetoothPort, prepareBluetoothPort, closeBluetoothPort, flushBluetoothPort,
+  awaitBluetoothInput, readBluetoothBytes, writeBluetoothBytes
+};
+#endif /* ENABLE_BLUETOOTH_SUPPORT */
+
 /*--- Protocol Operation Utilities ---*/
 
 typedef struct {
@@ -436,6 +510,7 @@ typedef struct {
   void (*writeText) (BrailleDisplay *brl, int start, int count);
   void (*writeStatus) (BrailleDisplay *brl, int start, int count);
   void (*flushCells) (BrailleDisplay *brl);
+  void (*setFirmness) (BrailleDisplay *brl, BrailleFirmness setting);
 } ProtocolOperations;
 
 static const ProtocolOperations *protocol;
@@ -445,9 +520,9 @@ static unsigned char currentText[BRLCOLSMAX];
 
 static int
 writeBytes (BrailleDisplay *brl, const unsigned char *bytes, int count) {
-  if (debugWrites) LogBytes("Write", bytes, count);
+  if (debugWrites) LogBytes(LOG_DEBUG, "Write", bytes, count);
   if (io->writeBytes(bytes, count) != -1) {
-    brl->writeDelay += count * 1000 / charactersPerSecond;
+    brl->writeDelay += (count * 1000 / charactersPerSecond) + 1;
     return 1;
   } else {
     LogError("Write");
@@ -461,14 +536,14 @@ interpretIdentity (BrailleDisplay *brl, unsigned char id, int major, int minor)
   LogPrint(LOG_INFO, "Papenmeier ID: %d  Version: %d.%02d", 
            id, major, minor);
   for (tn=0; tn<pmTerminalCount; tn++) {
-    if (pmTerminals[tn].identifier == id) {
+    if (pmTerminals[tn].modelIdentifier == id) {
       terminal = &pmTerminals[tn];
       LogPrint(LOG_INFO, "%s  Size: %dx%d  HelpFile: %s", 
-               terminal->name,
-               terminal->columns, terminal->rows,
+               terminal->modelName,
+               terminal->textColumns, terminal->textRows,
                terminal->helpFile);
-      brl->x = terminal->columns;
-      brl->y = terminal->rows;
+      brl->x = terminal->textColumns;
+      brl->y = terminal->textRows;
 
       /* TODO: ?? HACK */
       BRLSYMBOL.helpFile = terminal->helpFile;
@@ -552,7 +627,7 @@ readBytes1 (BrailleDisplay *brl, unsigned char *buffer, size_t offset, size_t co
   if (io->readBytes(buffer, &offset, count, 1000)) {
     if (!(flags & RBF_ETX)) return 1;
     if (*(buffer+offset-1) == cETX) return 1;
-    LogBytes("Corrupt Packet", buffer, offset);
+    LogBytes(LOG_WARNING, "Corrupt Packet", buffer, offset);
   }
   if ((offset > 0) && (flags & RBF_RESET)) resetTerminal1(brl);
   return 0;
@@ -605,7 +680,7 @@ interpretIdentity1 (BrailleDisplay *brl, const unsigned char *identity) {
   rcvStatusFirst = RCV_KEYROUTE;
   rcvStatusLast  = rcvStatusFirst + 3 * (terminal->statusCount - 1);
   rcvCursorFirst = rcvStatusLast + 3;
-  rcvCursorLast  = rcvCursorFirst + 3 * (terminal->columns - 1);
+  rcvCursorLast  = rcvCursorFirst + 3 * (terminal->textColumns - 1);
   LogPrint(LOG_DEBUG, "Routing Keys: status=%03X-%03X cursor=%03X-%03X",
            rcvStatusFirst, rcvStatusLast,
            rcvCursorFirst, rcvCursorLast);
@@ -614,9 +689,9 @@ interpretIdentity1 (BrailleDisplay *brl, const unsigned char *identity) {
   rcvFrontFirst = RCV_KEYFUNC + 3;
   rcvFrontLast  = rcvFrontFirst + 3 * (terminal->frontKeys - 1);
   rcvBarFirst = rcvFrontLast + 3;
-  rcvBarLast  = rcvBarFirst + 3 * ((terminal->hasEasyBar? 8: 0) - 1);
+  rcvBarLast  = rcvBarFirst + 3 * ((terminal->hasBar? 8: 0) - 1);
   rcvSwitchFirst = rcvBarLast + 3;
-  rcvSwitchLast  = rcvSwitchFirst + 3 * ((terminal->hasEasyBar? 8: 0) - 1);
+  rcvSwitchLast  = rcvSwitchFirst + 3 * ((terminal->hasBar? 8: 0) - 1);
   LogPrint(LOG_DEBUG, "Function Keys: front=%03X-%03X bar=%03X-%03X switches=%03X-%03X",
            rcvFrontFirst, rcvFrontLast,
            rcvBarFirst, rcvBarLast,
@@ -640,23 +715,23 @@ handleKey1 (BrailleDisplay *brl, int code, int press, int time) {
   if (rcvFrontFirst <= code && 
       code <= rcvFrontLast) { /* front key */
     num = 1 + (code - rcvFrontFirst) / 3;
-    return handleKey(brl, OFFS_FRONT+num, press, 0);
+    return handleKey(brl, KEYS_FRONT+num, press, 0);
   }
 
   if (rcvStatusFirst <= code && 
       code <= rcvStatusLast) { /* status key */
     num = 1 + (code - rcvStatusFirst) / 3;
-    return handleKey(brl, OFFS_STAT+num, press, 0);
+    return handleKey(brl, KEYS_STATUS+num, press, 0);
   }
 
   if (rcvBarFirst <= code && 
-      code <= rcvBarLast) { /* easy bar */
+      code <= rcvBarLast) { /* easy access bar */
     {
       static const int modifiers[] = {
-        MOD_EASY_SLR, MOD_EASY_SLF,
-        MOD_EASY_KLR, MOD_EASY_KLF,
-        MOD_EASY_KRR, MOD_EASY_KRF,
-        MOD_EASY_SRR, MOD_EASY_SRF,
+        MOD_BAR_SLR, MOD_BAR_SLF,
+        MOD_BAR_KLR, MOD_BAR_KLF,
+        MOD_BAR_KRR, MOD_BAR_KRF,
+        MOD_BAR_SRR, MOD_BAR_SRF,
         0
       };
       const int *modifier = modifiers;
@@ -680,19 +755,19 @@ handleKey1 (BrailleDisplay *brl, int code, int press, int time) {
     }
 
     num = 1 + (code - rcvBarFirst) / 3;
-    return handleKey(brl, OFFS_EASY+num, press, 0);
+    return handleKey(brl, KEYS_BAR+num, press, 0);
   }
 
   if (rcvSwitchFirst <= code && 
-      code <= rcvSwitchLast) { /* easy bar */
+      code <= rcvSwitchLast) { /* easy access bar */
     num = 1 + (code - rcvSwitchFirst) / 3;
-    return handleKey(brl, OFFS_SWITCH+num, press, 0);
+    return handleKey(brl, KEYS_SWITCH+num, press, 0);
   }
 
   if (rcvCursorFirst <= code && 
       code <= rcvCursorLast) { /* Routing Keys */ 
     num = (code - rcvCursorFirst) / 3;
-    return handleKey(brl, ROUTINGKEY1, press, num);
+    return handleKey(brl, KEY_ROUTING1, press, num);
   }
 
   LogPrint(LOG_WARNING, "Unexpected key: %04X", code);
@@ -710,7 +785,7 @@ disableOutputTranslation1 (BrailleDisplay *brl, unsigned char xmtOffset, int cou
 static void
 initializeTable1 (BrailleDisplay *brl) {
   disableOutputTranslation1(brl, xmtStatusOffset, terminal->statusCount);
-  disableOutputTranslation1(brl, xmtTextOffset, terminal->columns);
+  disableOutputTranslation1(brl, xmtTextOffset, terminal->textColumns);
 }
 
 static void
@@ -735,7 +810,7 @@ initializeTerminal1 (BrailleDisplay *brl) {
   writeStatus1(brl, 0, terminal->statusCount);
   drainBrailleOutput(brl, 0);
 
-  writeText1(brl, 0, terminal->columns);
+  writeText1(brl, 0, terminal->textColumns);
   drainBrailleOutput(brl, 0);
 }
 
@@ -754,7 +829,7 @@ readCommand1 (BrailleDisplay *brl, BRL_DriverCommandContext context) {
     while (1) {
       READ(0, 1, 0);
       if (buf[0] == cSTX) break;
-      LogBytes("Discarded Byte", buf, 1);
+      LogBytes(LOG_WARNING, "Discarded Byte", buf, 1);
     }
 
     READ(1, 1, 0);
@@ -772,7 +847,7 @@ readCommand1 (BrailleDisplay *brl, BRL_DriverCommandContext context) {
       case cIdIdentify: {
         const int length = 10;
         READ(2, length-2, RBF_ETX);
-        if (debugReads) LogBytes("Identity Packet", buf, length);
+        if (debugReads) LogBytes(LOG_DEBUG, "Identity Packet", buf, length);
         if (interpretIdentity1(brl, buf)) brl->resizeRequired = 1;
         approximateDelay(200);
         restartTerminal1(brl);
@@ -791,7 +866,7 @@ readCommand1 (BrailleDisplay *brl, BRL_DriverCommandContext context) {
           return EOF;
         }
         READ(6, length-6, RBF_ETX);			/* Data */
-        if (debugReads) LogBytes("Input Packet", buf, length);
+        if (debugReads) LogBytes(LOG_DEBUG, "Input Packet", buf, length);
 
         {
           int command = handleKey1(brl, ((buf[2] << 8) | buf[3]),
@@ -820,7 +895,7 @@ readCommand1 (BrailleDisplay *brl, BRL_DriverCommandContext context) {
         message = "data framing error";
       logError:
         READ(2, 1, RBF_ETX);
-        if (debugReads) LogBytes("Error Packet", buf, 3);
+        if (debugReads) LogBytes(LOG_DEBUG, "Error Packet", buf, 3);
         LogPrint(LOG_WARNING, "Output packet error: %02X: %s", buf[1], message);
         restartTerminal1(brl);
         break;
@@ -837,7 +912,8 @@ releaseResources1 (void) {
 static const ProtocolOperations protocolOperations1 = {
   initializeTerminal1, releaseResources1,
   readCommand1,
-  writeText1, writeStatus1, flushCells1
+  writeText1, writeStatus1, flushCells1,
+  NULL
 };
 
 static int
@@ -901,6 +977,7 @@ typedef struct {
 static InputMapping2 *inputMap2 = NULL;
 static int inputBytes2;
 static int inputBits2;
+static int inputKeySize2;
 
 static unsigned char *inputState2 = NULL;
 static int refreshRequired2;
@@ -914,7 +991,7 @@ readPacket2 (BrailleDisplay *brl, Packet2 *packet) {
 
   while (1) {
     if (!io->readBytes(buffer, &offset, 1, 1000)) {
-      LogBytes("Partial Packet", buffer, offset);
+      LogBytes(LOG_WARNING, "Partial Packet", buffer, offset);
       return 0;
     }
 
@@ -926,24 +1003,24 @@ readPacket2 (BrailleDisplay *brl, Packet2 *packet) {
       switch (byte) {
         case cSTX:
           if (offset > 1) {
-            LogBytes("Incomplete Packet", buffer, offset);
+            LogBytes(LOG_WARNING, "Incomplete Packet", buffer, offset);
             offset = 1;
           }
           continue;
 
         case cETX:
           if ((offset >= 5) && (offset == size)) {
-            if (debugReads) LogBytes("Input Packet", buffer, offset);
+            if (debugReads) LogBytes(LOG_DEBUG, "Input Packet", buffer, offset);
             return 1;
           }
-          LogBytes("Short Packet", buffer, offset);
+          LogBytes(LOG_WARNING, "Short Packet", buffer, offset);
           offset = 0;
           continue;
 
         default:
           switch (offset) {
             case 1:
-              LogBytes("Discarded Byte", buffer, offset);
+              LogBytes(LOG_WARNING, "Discarded Byte", buffer, offset);
               offset = 0;
               continue;
     
@@ -971,7 +1048,7 @@ readPacket2 (BrailleDisplay *brl, Packet2 *packet) {
               if (type != 0X30) break;
 
               if (offset == size) {
-                LogBytes("Long Packet", buffer, offset);
+                LogBytes(LOG_WARNING, "Long Packet", buffer, offset);
                 offset = 0;
                 continue;
               }
@@ -996,7 +1073,7 @@ readPacket2 (BrailleDisplay *brl, Packet2 *packet) {
       }
     }
 
-    LogBytes("Corrupt Packet", buffer, offset);
+    LogBytes(LOG_WARNING, "Corrupt Packet", buffer, offset);
     offset = 0;
   }
 }
@@ -1044,8 +1121,12 @@ flushCells2 (BrailleDisplay *brl) {
     unsigned char buffer[0XFF];
     unsigned int size = 0;
 
+    /* The status cells. */
+    memcpy(&buffer[size], currentStatus, terminal->statusCount);
+    size += terminal->statusCount;
+
     /* Two dummy cells for each key on the left side. */
-    {
+    if (terminal->protocolRevision < 2) {
       int count = terminal->leftKeys;
       while (count-- > 0) {
         buffer[size++] = 0;
@@ -1054,15 +1135,11 @@ flushCells2 (BrailleDisplay *brl) {
     }
 
     /* The text cells. */
-    memcpy(&buffer[size], currentText, terminal->columns);
-    size += terminal->columns;
-
-    /* The status cells. */
-    memcpy(&buffer[size], currentStatus, terminal->statusCount);
-    size += terminal->statusCount;
+    memcpy(&buffer[size], currentText, terminal->textColumns);
+    size += terminal->textColumns;
 
     /* Two dummy cells for each key on the right side. */
-    {
+    if (terminal->protocolRevision < 2) {
       int count = terminal->rightKeys;
       while (count-- > 0) {
         buffer[size++] = 0;
@@ -1080,30 +1157,36 @@ initializeTerminal2 (BrailleDisplay *brl) {
   memset(inputState2, 0, inputBytes2);
   refreshRequired2 = 1;
 
-  {
+  /* Don't send the init packet by default as that was done at the factory
+   * and shouldn't need to be done again. We'll keep the code, though,
+   * just in case it's ever needed. Perhaps there should be a driver
+   * parameter to control it.
+   */
+  if (0) {
     unsigned char data[13];
     unsigned char size = 0;
 
-    data[size++] = terminal->identifier;
+    data[size++] = terminal->modelIdentifier; /* device identification code */
 
-    /* Set serial baud (bcd encoded) to default (57,600). */
+    /* serial baud (bcd-encoded, six digits, one per nibble) */
+    /* set to zero for default (57,600) */
     data[size++] = 0;
     data[size++] = 0;
     data[size++] = 0;
 
-    data[size++] = 0; /* vertical modules */
-    data[size++] = terminal->leftKeys;
-    data[size++] = terminal->columns + terminal->statusCount;
-    data[size++] = terminal->rightKeys;
+    data[size++] = terminal->statusCount; /* number of vertical braille cells */
+    data[size++] = terminal->leftKeys; /* number of left keys and switches */
+    data[size++] = terminal->textColumns; /* number of horizontal braille cells */
+    data[size++] = terminal->rightKeys; /* number of right keys and switches */
 
-    data[size++] = 2; /* routing keys per cell */
+    data[size++] = 2; /* number of routing keys per braille cell */
     data[size++] = 0; /* size of LCD */
 
-    data[size++] = 1; /* keys mixed into braille data stream */
-    data[size++] = 0; /* EAB mixed into braille data stream */
+    data[size++] = 1; /* keys and switches mixed into braille data stream */
+    data[size++] = 0; /* easy access bar mixed into braille data stream */
     data[size++] = 1; /* routing keys mixed into braille data stream */
 
-    LogBytes("Init Packet", data, size);
+    LogBytes(LOG_DEBUG, "Init Packet", data, size);
     writePacket2(brl, 1, size, data);
   }
 }
@@ -1138,7 +1221,7 @@ readCommand2 (BrailleDisplay *brl, BRL_DriverCommandContext context) {
 
               while (bit) {
                 if (!(new & bit) && (old & bit)) {
-                  if (mapping->code != NOKEY) {
+                  if (mapping->code != KEY_NONE) {
                     int cmd = handleKey(brl, mapping->code, 0, mapping->offset);
                     if (!release) {
                       command = cmd;
@@ -1169,7 +1252,7 @@ readCommand2 (BrailleDisplay *brl, BRL_DriverCommandContext context) {
 
             while (bit) {
               if ((new & bit) && !(old & bit)) {
-                if (mapping->code != NOKEY) {
+                if (mapping->code != KEY_NONE) {
                   command = handleKey(brl, mapping->code, 1, mapping->offset);
                 }
 
@@ -1184,6 +1267,29 @@ readCommand2 (BrailleDisplay *brl, BRL_DriverCommandContext context) {
 
         return command;
       }
+
+      case 0X0C: {
+        int command = BRL_BLK_PASSDOTS;
+        unsigned char modifiers = packet.data.bytes[0];
+        unsigned char dots = packet.data.bytes[1];
+
+        if (dots) {
+          if (dots & 0X01) command |= BRL_DOT1;
+          if (dots & 0X02) command |= BRL_DOT2;
+          if (dots & 0X04) command |= BRL_DOT3;
+          if (dots & 0X08) command |= BRL_DOT4;
+          if (dots & 0X10) command |= BRL_DOT5;
+          if (dots & 0X20) command |= BRL_DOT6;
+          if (dots & 0X40) command |= BRL_DOT7;
+          if (dots & 0X80) command |= BRL_DOT8;
+          if (modifiers & 0X02) command |= BRL_FLG_CHAR_UPPER;
+        } else if (!(modifiers & 0X02)) {
+          return BRL_CMD_NOOP;
+        }
+        if (modifiers & 0X04) command |= BRL_FLG_CHAR_CONTROL;
+        if (modifiers & 0X01) command |= BRL_FLG_CHAR_META;
+        return command;
+      }
     }
   }
 
@@ -1204,83 +1310,114 @@ releaseResources2 (void) {
   }
 }
 
+static void
+setFirmness2 (BrailleDisplay *brl, BrailleFirmness setting) {
+  unsigned char data[] = {(setting * 98 / BF_MAXIMUM) + 2, 0X99};
+  writePacket2(brl, 6, sizeof(data), data);
+}
+
 static const ProtocolOperations protocolOperations2 = {
   initializeTerminal2, releaseResources2,
   readCommand2,
-  writeCells2, writeCells2, flushCells2
+  writeCells2, writeCells2, flushCells2,
+  setFirmness2
 };
 
+typedef struct {
+  unsigned char byte;
+  unsigned char bit;
+  unsigned char size;
+} InputModule2;
+
 static void
-addInputMapping2 (int byte, int bit, int code, int offset) {
-  InputMapping2 *mapping = &inputMap2[(byte * 8) + bit];
-  mapping->code = code;
-  mapping->offset = offset;
+addInputMapping2 (const InputModule2 *module, unsigned char bit, int code, int offset) {
+  if (terminal->protocolRevision < 2) {
+    bit += module->bit;
+  } else {
+    bit += 8 - module->bit - module->size;
+  }
+
+  {
+    InputMapping2 *mapping = &inputMap2[(module->byte * 8) + bit];
+    mapping->code = code;
+    mapping->offset = offset;
+  }
+}
+
+static int
+nextInputModule2 (InputModule2 *module, unsigned char size) {
+  if (!module->bit) {
+    if (!module->byte) return 0;
+    module->byte -= 1;
+    module->bit = 8;
+  }
+  module->bit -= module->size = size;
+  return 1;
 }
 
 static void
-nextInputModule2 (int *byte, int *bit) {
-  if (!*bit) *bit = 8, *byte -= 1;
-  *bit -= 2;
-}
-
-static void
-mapKey2 (int count, int *byte, int *bit, int rear, int front) {
+mapInputKey2 (int count, InputModule2 *module, int rear, int front) {
   while (count--) {
-    nextInputModule2(byte, bit);
-    nextInputModule2(byte, bit);
-    addInputMapping2(*byte, *bit+1, front, 0);
-    addInputMapping2(*byte, *bit, rear, 0);
+    nextInputModule2(module, inputKeySize2);
+    addInputMapping2(module, 0, rear, 0);
+    addInputMapping2(module, 1, front, 0);
   }
 }
 
 static void
 mapInputModules2 (void) {
-  int byte = inputBytes2;
-  int bit = 0;
+  InputModule2 module;
+  module.byte = inputBytes2;
+  module.bit = 0;
 
   {
     int i;
     for (i=0; i<inputBits2; ++i) {
       InputMapping2 *mapping = &inputMap2[i];
-      mapping->code = NOKEY;
+      mapping->code = KEY_NONE;
       mapping->offset = 0;
     }
   }
 
-  mapKey2(terminal->rightKeys, &byte, &bit,
-          OFFS_SWITCH+KEY_RIGHT_REAR,
-          OFFS_SWITCH+KEY_RIGHT_FRONT);
+  mapInputKey2(terminal->rightKeys, &module,
+               KEYS_SWITCH+KEY_RIGHT_REAR,
+               KEYS_SWITCH+KEY_RIGHT_FRONT);
+
+  {
+    unsigned char column = terminal->textColumns;
+    while (column) {
+      nextInputModule2(&module, 1);
+      addInputMapping2(&module, 0, KEY_ROUTING2, --column);
+
+      nextInputModule2(&module, 1);
+      addInputMapping2(&module, 0, KEY_ROUTING1, column);
+    }
+  }
+
+  mapInputKey2(terminal->leftKeys, &module,
+               KEYS_SWITCH+KEY_LEFT_REAR,
+               KEYS_SWITCH+KEY_LEFT_FRONT);
 
   {
     unsigned char cell = terminal->statusCount;
     while (cell) {
-      nextInputModule2(&byte, &bit);
-      addInputMapping2(byte, bit, OFFS_STAT+cell--, 0);
+      nextInputModule2(&module, 1);
+      addInputMapping2(&module, 0, KEY_STATUS2, cell-1);
+
+      nextInputModule2(&module, 1);
+      addInputMapping2(&module, 0, KEYS_STATUS+cell--, 0);
     }
   }
 
-  {
-    unsigned char column = terminal->columns;
-    while (column) {
-      nextInputModule2(&byte, &bit);
-      addInputMapping2(byte, bit, ROUTINGKEY1, column-1);
-      addInputMapping2(byte, bit+1, ROUTINGKEY2, --column);
-    }
-  }
-
-  mapKey2(terminal->leftKeys, &byte, &bit,
-          OFFS_SWITCH+KEY_LEFT_REAR,
-          OFFS_SWITCH+KEY_LEFT_FRONT);
-
-  byte--;
-  addInputMapping2(byte, 7, OFFS_EASY+EASY_L2, 0);
-  addInputMapping2(byte, 6, OFFS_EASY+EASY_R2, 0);
-  addInputMapping2(byte, 5, OFFS_EASY+EASY_L1, 0);
-  addInputMapping2(byte, 4, OFFS_EASY+EASY_R1, 0);
-  addInputMapping2(byte, 3, OFFS_EASY+EASY_D2, 0);
-  addInputMapping2(byte, 2, OFFS_EASY+EASY_D1, 0);
-  addInputMapping2(byte, 1, OFFS_EASY+EASY_U1, 0);
-  addInputMapping2(byte, 0, OFFS_EASY+EASY_U2, 0);
+  nextInputModule2(&module, 8);
+  addInputMapping2(&module, 0, KEYS_BAR+BAR_U2, 0);
+  addInputMapping2(&module, 1, KEYS_BAR+BAR_U1, 0);
+  addInputMapping2(&module, 2, KEYS_BAR+BAR_D1, 0);
+  addInputMapping2(&module, 3, KEYS_BAR+BAR_D2, 0);
+  addInputMapping2(&module, 4, KEYS_BAR+BAR_R1, 0);
+  addInputMapping2(&module, 5, KEYS_BAR+BAR_L1, 0);
+  addInputMapping2(&module, 6, KEYS_BAR+BAR_R2, 0);
+  addInputMapping2(&module, 7, KEYS_BAR+BAR_L2, 0);
 }
 
 static int
@@ -1300,9 +1437,13 @@ identifyTerminal2 (BrailleDisplay *brl) {
               makeOutputTable(dots, outputTable);
             }
 
+            inputKeySize2 = (terminal->protocolRevision < 2)? 4: 8;
             {
-              int modules = terminal->leftKeys + terminal->rightKeys;
-              inputBytes2 = modules + 1 + ((((modules * 4) + ((terminal->columns + terminal->statusCount) * 2)) + 7) / 8);
+              int keyCount = terminal->leftKeys + terminal->rightKeys;
+              inputBytes2 = keyCount + 1 +
+                            ((((keyCount * inputKeySize2) +
+                               ((terminal->textColumns + terminal->statusCount) * 2)
+                              ) + 7) / 8);
             }
             inputBits2 = inputBytes2 * 8;
 
@@ -1329,14 +1470,6 @@ identifyTerminal2 (BrailleDisplay *brl) {
 
 /*--- Driver Operations ---*/
 
-static void
-brl_identify (void) {
-  LogPrint(LOG_NOTICE, "Papenmeier Driver (compiled on %s at %s)", __DATE__, __TIME__);
-  LogPrint(LOG_INFO, "   Copyright (C) 1998-2001 by The BRLTTY Team.");
-  LogPrint(LOG_INFO, "                 August Hrandl <august.hoerandl@gmx.at>");
-  LogPrint(LOG_INFO, "                 Heimo Schn <heimo.schoen@gmx.at>");
-}
-
 static int
 identifyTerminal (BrailleDisplay *brl) {
   if (io->protocol1 && identifyTerminal1(brl)) return 1;
@@ -1344,11 +1477,28 @@ identifyTerminal (BrailleDisplay *brl) {
   return 0;
 }
 
+static void
+resetTerminalTable (void) {
+  if (pmTerminalsAllocated) {
+#ifdef ENABLE_PM_CONFIGURATION_FILE
+    deallocateTerminalTable();
+#endif /* ENABLE_PM_CONFIGURATION_FILE */
+    pmTerminals = pmTerminalTable;
+    pmTerminalCount = ARRAY_COUNT(pmTerminalTable);
+    pmTerminalsAllocated = 0;
+  }
+}
+
 static int
-brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
-  validateYesNo(&debugKeys, "debug keys flag", parameters[PARM_DEBUGKEYS]);
-  validateYesNo(&debugReads, "debug reads flag", parameters[PARM_DEBUGREADS]);
-  validateYesNo(&debugWrites, "debug writes flag", parameters[PARM_DEBUGWRITES]);
+brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
+  if (!validateYesNo(&debugKeys, parameters[PARM_DEBUGKEYS]))
+    LogPrint(LOG_WARNING, "%s: %s", "invalid debug keys setting", parameters[PARM_DEBUGKEYS]);
+
+  if (!validateYesNo(&debugReads, parameters[PARM_DEBUGREADS]))
+    LogPrint(LOG_WARNING, "%s: %s", "invalid debug reads setting", parameters[PARM_DEBUGREADS]);
+
+  if (!validateYesNo(&debugWrites, parameters[PARM_DEBUGWRITES]))
+    LogPrint(LOG_WARNING, "%s: %s", "invalid debug writes setting", parameters[PARM_DEBUGWRITES]);
 
 #ifdef ENABLE_PM_CONFIGURATION_FILE
   {
@@ -1371,15 +1521,23 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
 
   if (isSerialDevice(&device)) {
     io = &serialOperations;
+  } else
 
 #ifdef ENABLE_USB_SUPPORT
-  } else if (isUsbDevice(&device)) {
+  if (isUsbDevice(&device)) {
     io = &usbOperations;
+  } else
 #endif /* ENABLE_USB_SUPPORT */
 
-  } else {
+#ifdef ENABLE_BLUETOOTH_SUPPORT
+  if (isBluetoothDevice(&device)) {
+    io = &bluetoothOperations;
+  } else
+#endif /* ENABLE_BLUETOOTH_SUPPORT */
+
+  {
     unsupportedDevice(device);
-    return 0;
+    goto failed;
   }
 
   baud = io->bauds;
@@ -1389,7 +1547,7 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
 
     if (io->openPort(parameters, device)) {
       if (identifyTerminal(brl)) {
-        memset(currentText, outputTable[0], terminal->columns);
+        memset(currentText, outputTable[0], terminal->textColumns);
         memset(currentStatus, outputTable[0], terminal->statusCount);
         resetState();
         protocol->initializeTerminal(brl);
@@ -1401,13 +1559,16 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
     ++baud;
   }
 
+failed:
+  resetTerminalTable();
   return 0;
 }
 
 static void
-brl_close (BrailleDisplay *brl) {
+brl_destruct (BrailleDisplay *brl) {
   io->closePort();
   protocol->releaseResources();
+  resetTerminalTable();
 }
 
 static void
@@ -1436,8 +1597,8 @@ updateCells (BrailleDisplay *brl, int size, const unsigned char *data, unsigned
 static void
 brl_writeWindow (BrailleDisplay *brl) {
   int i;
-  for (i=0; i<terminal->columns; i++) brl->buffer[i] = outputTable[brl->buffer[i]];
-  updateCells(brl, terminal->columns, brl->buffer, currentText, protocol->writeText);
+  for (i=0; i<terminal->textColumns; i++) brl->buffer[i] = outputTable[brl->buffer[i]];
+  updateCells(brl, terminal->textColumns, brl->buffer, currentText, protocol->writeText);
   protocol->flushCells(brl);
 }
 
@@ -1482,3 +1643,8 @@ static int
 brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
   return protocol->readCommand(brl, context);
 }
+
+static void
+brl_firmness (BrailleDisplay *brl, BrailleFirmness setting) {
+  if (protocol->setFirmness) protocol->setFirmness(brl, setting);
+}
