@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -14,34 +14,31 @@
  *
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
-#define VS_VERSION "BRLTTY driver for VisioBraille, version 0.2, 2002"
-#define VS_COPYRIGHT "Copyright Sebastien HINDERER <Sebastien.Hinderer@libertysurf.fr"
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "Programs/misc.h"
-#include "Programs/scr.h"
-#include "Programs/message.h"
+#include "misc.h"
+#include "scr.h"
+#include "message.h"
 
 typedef enum {
   PARM_DISPSIZE=0,
-  PARM_PROMVER=1
+  PARM_PROMVER=1,
+  PARM_BAUD=2
 } DriverParameter;
-#define BRLPARMS "displaysize", "promVersion"
+#define BRLPARMS "displaysize", "promversion", "baud"
 
 #define BRL_HAVE_PACKET_IO
 #define BRL_HAVE_KEY_CODES
-#include "Programs/brl_driver.h"
+#include "brl_driver.h"
 #include "braille.h"
 #include "brldefs-vs.h"
-#include "Programs/io_serial.h"
+#include "io_serial.h"
 
-#define MAXPKTLEN 512
+#define MAXPACKETSIZE 512
 
 static SerialDevice *serialDevice;
 #ifdef SendIdReq
@@ -65,10 +62,11 @@ static int printcode = 0;
 /* Function : brl_writePacket */ 
 /* Sends a packet of size bytes, stored at address p to the braille terminal */
 /* Returns 0 if everything is right, -1 if an error occured while sending */
-static ssize_t brl_writePacket(BrailleDisplay *brl, const unsigned char *p, size_t size)
+static ssize_t brl_writePacket(BrailleDisplay *brl, const void *packet, size_t size)
 {
+  const unsigned char *p = packet;
   int lgtho = 1;
-  static unsigned char obuf[MAXPKTLEN] = { 02 };
+  static unsigned char obuf[MAXPACKETSIZE] = { 02 };
   const unsigned char *x;
   unsigned char *y = obuf+1;
   unsigned char chksum=0;
@@ -106,7 +104,7 @@ static ssize_t brl_writePacket(BrailleDisplay *brl, const unsigned char *p, size
 /* The size of the packet is returned */
 /* If a packet is too long, it is discarded and a message sent to the syslog */
 /* "+" packets are silently discarded, since they are only disturbing us */
-static ssize_t brl_readPacket(BrailleDisplay *brl, unsigned char *p, size_t size) 
+static ssize_t brl_readPacket(BrailleDisplay *brl, void *p, size_t size) 
 {
   size_t offset = 0;
   static unsigned char ack = 04;
@@ -114,9 +112,9 @@ static ssize_t brl_readPacket(BrailleDisplay *brl, unsigned char *p, size_t size
   static int apacket = 0;
   static unsigned char prefix, checksum;
   unsigned char ch;
-  static unsigned char buf[MAXPKTLEN]; 
+  static unsigned char buf[MAXPACKETSIZE]; 
   static unsigned char *q;
-  if ((p==NULL) || (size<2) || (size>MAXPKTLEN)) return 0; 
+  if ((p==NULL) || (size<2) || (size>MAXPACKETSIZE)) return 0; 
   while (serialReadChunk(serialDevice,&ch,&offset,1,0,1000)) {
     if (ch==0x02) {
       apacket = 1;
@@ -166,22 +164,14 @@ static int brl_reset(BrailleDisplay *brl)
   return 1;
 }
 
-/* Function : brl_identify */
-/* Prints information about the driver in the system log and on stderr */
-static void brl_identify()
-{
-  LogPrint(LOG_NOTICE, VS_VERSION);
-  LogPrint(LOG_INFO,   "   "VS_COPYRIGHT);
-}
-
-/* Function : brl_open */
+/* Function : brl_construct */
 /* Opens and configures the serial port properly */
-/* if brl->x <= 0 when brl_open is called, then brl->x is initialized */
+/* if brl->x <= 0 when brl_construct is called, then brl->x is initialized */
 /* either with the BRAILLEDISPLAYSIZE constant, defined in braille.h */
 /* or with the size got through identification request if it succeeds */
-/* Else, brl->x is left unmodified by brl_open, so that */
+/* Else, brl->x is left unmodified by brl_construct, so that */
 /* the braille display can be resized without reloading the driver */ 
-static int brl_open(BrailleDisplay *brl, char **parameters, const char *device)
+static int brl_construct(BrailleDisplay *brl, char **parameters, const char *device)
 {
 #ifdef SendIdReq
   unsigned char ch = '?';
@@ -189,13 +179,22 @@ static int brl_open(BrailleDisplay *brl, char **parameters, const char *device)
 #endif /* SendIdReq */
   int ds = BRAILLEDISPLAYSIZE;
   int promVersion = 4;
+  int ttyBaud = 57600;
   if (*parameters[PARM_DISPSIZE]) {
     int dsmin=20, dsmax=40;
-    validateInteger(&ds, "Size of braille display",parameters[PARM_DISPSIZE],&dsmin,&dsmax);
+    if (!validateInteger(&ds, parameters[PARM_DISPSIZE], &dsmin, &dsmax))
+      LogPrint(LOG_WARNING, "%s: %s", "invalid braille display size", parameters[PARM_DISPSIZE]);
   }
   if (*parameters[PARM_PROMVER]) {
     int pvmin=3, pvmax=6;
-    validateInteger(&promVersion, "PROM version",parameters[PARM_PROMVER],&pvmin,&pvmax);
+    if (!validateInteger(&promVersion, parameters[PARM_PROMVER], &pvmin, &pvmax))
+      LogPrint(LOG_WARNING, "%s: %s", "invalid PROM version", parameters[PARM_PROMVER]);
+  }
+  if (*parameters[PARM_BAUD]) {
+    int baud;
+    if (serialValidateBaud(&baud, "TTY baud", parameters[PARM_BAUD], NULL)) {
+      ttyBaud = baud;
+    }
   }
 
   if (!isSerialDevice(&device)) {
@@ -205,7 +204,7 @@ static int brl_open(BrailleDisplay *brl, char **parameters, const char *device)
   if (!(serialDevice = serialOpenDevice(device))) return 0;
   serialSetParity(serialDevice, SERIAL_PARITY_ODD);
   if (promVersion<4) serialSetFlowControl(serialDevice, SERIAL_FLOW_INPUT_CTS);
-  serialRestartDevice(serialDevice,57600); 
+  serialRestartDevice(serialDevice,ttyBaud); 
 #ifdef SendIdReq
   {
     brl_writePacket(brl,(unsigned char *) &ch,1); 
@@ -243,9 +242,9 @@ static int brl_open(BrailleDisplay *brl, char **parameters, const char *device)
   return 1;
 }
 
-/* Function : brl_close */
+/* Function : brl_destruct */
 /* Closes the braille device and deallocates dynamic structures */
-static void brl_close(BrailleDisplay *brl)
+static void brl_destruct(BrailleDisplay *brl)
 {
   if (serialDevice) {
     serialCloseDevice(serialDevice);
@@ -641,25 +640,16 @@ int brl_keyToCommand(BrailleDisplay *brl, BRL_DriverCommandContext context, int
 /* for BRL_ROUTING, the code is the ofset to route, starting from 0 */
 static int brl_readKey(BrailleDisplay *brl)
 {
-  unsigned char ch, ibuf[MAXPKTLEN-1];
+  unsigned char ch, packet[MAXPACKETSIZE];
   static int routing = 0;
-  ssize_t lgthi;
-  readNextPacket:
-  lgthi = brl_readPacket(brl,ibuf,MAXPKTLEN-1);
-  if (lgthi==0) return EOF;
-  if (ibuf[0]=='%') {
-    ibuf[lgthi] = '\0';
-    insertString((char *)&ibuf[1]);
-    return EOF;
-  }  
-  if ((ibuf[0]!=0x3c) && (ibuf[0]!=0x3d) && (ibuf[0]!=0x23)) {
-    char buf[100];
-    if (ibuf[0]==0x2b) return EOF;
-    sprintf(buf,"Unknown packet 0x%x",ibuf[0]);
-    message(buf,MSG_WAITKEY | MSG_NODELAY);
+  ssize_t packetSize;
+  packetSize = brl_readPacket(brl,packet,sizeof(packet));
+  if (packetSize==0) return EOF;
+  if ((packet[0]!=0x3c) && (packet[0]!=0x3d) && (packet[0]!=0x23)) {
+    LogBytes(LOG_WARNING, "[vs] Discarding unsupported packet", packet, packetSize);
     return EOF;
   }
-  ch = ibuf[1];
+  ch = packet[1];
   if (printcode) {
     char buf [100];
     sprintf(buf,"Keycode: 0x%x",ch);
@@ -669,13 +659,13 @@ static int brl_readKey(BrailleDisplay *brl)
   }
   if (routing) {
     routing=0;
-    if (ch>=0xc0)  return (ibuf[1]-0xc0) | BRL_VSMSK_ROUTING;
+    if (ch>=0xc0)  return (packet[1]-0xc0) | BRL_VSMSK_ROUTING;
     return EOF;
   }
   if ((ch>=0xc0) && (ch<=0xdf)) return (ch-0xc0) | BRL_VSMSK_FUNCTIONKEY;
   if (ch==0x91) {
     routing = 1;
-    goto readNextPacket;
+    return BRL_CMD_NOOP;
   } 
   if ((ch>=0x20) && (ch<=0x9e)) {
     switch (ch) {
