@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -14,20 +14,19 @@
  *
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
-#define XW_VERSION "BRLTTY driver for X, version 0.1, 2004"
-#define XW_COPYRIGHT "Copyright Samuel Thibault <samuel.thibault@ens-lyon.org>"
 
 #include "prologue.h"
 
 #include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
 #include <locale.h>
 
-#include "Programs/misc.h"
+#include "misc.h"
+#include "charset.h"
 
-#if defined(HAVE_PKG_XAW)
+#if defined(WINDOWS)
+#define USE_WINDOWS
+#elif defined(HAVE_PKG_XAW)
 #define USE_XAW
 #define USE_XT
 #include <X11/Intrinsic.h>
@@ -87,8 +86,6 @@
 #include <Xm/ToggleB.h>
 #include <Xm/RowColumn.h>
 #include <Xm/MenuShell.h>
-#elif defined(WINDOWS)
-#define USE_WINDOWS
 #else /* HAVE_PKG_ */
 #error GUI toolkit either unspecified or unsupported
 #endif /* HAVE_PKG_ */
@@ -193,14 +190,14 @@ typedef void *XtPointer;
 typedef enum {
   PARM_TKPARMS,
   PARM_LINES,
-  PARM_COLS,
+  PARM_COLUMNS,
   PARM_MODEL,
   PARM_INPUT
 } DriverParameter;
-#define BRLPARMS "tkparms", "lines", "cols", "model", "input"
+#define BRLPARMS "tkparms", "lines", "columns", "model", "input"
 
 #define BRL_HAVE_VISUAL_DISPLAY
-#include "Programs/brl_driver.h"
+#include "brl_driver.h"
 #include "braille.h"
 
 #define MAXLINES 3
@@ -215,9 +212,9 @@ static char **xtArgv = xtDefArgv;
 static int regenerate;
 static void generateToplevel(void);
 static void destroyToplevel(void);
-#ifdef USE_XAW
+#if defined(USE_XAW) || defined(USE_WINDOWS)
 static unsigned char displayedWindow[WHOLESIZE];
-#endif /* USE_XAW */
+#endif /* USE_XAW || USE_WINDOWS */
 static unsigned char displayedVisual[WHOLESIZE];
 
 #define BUTWIDTH 48
@@ -225,6 +222,9 @@ static unsigned char displayedVisual[WHOLESIZE];
 
 static Widget toplevel,hbox,display[WHOLESIZE];
 static MenuWidget menu;
+#if defined(USE_XAW) || defined(USE_WINDOWS)
+static Widget displayb[WHOLESIZE];
+#endif /* USE_XAW || USE_WINDOWS */
 #ifdef USE_XAW
 static Pixmap check;
 #endif /* USE_XAW */
@@ -232,15 +232,17 @@ static int lastcursor = -1;
 #ifdef USE_XT
 static Widget vbox,keybox;
 static Pixel displayForeground,displayBackground;
-#ifdef USE_XAW
-static Widget displayb[WHOLESIZE];
-static XFontSet fontset;
-#endif /* USE_XAW */
 static XtAppContext app_con;
 #ifdef USE_XM
 static XmString display_cs;
 #endif /* USE_XAW */
 #endif /* USE_XT */
+#ifdef USE_XAW
+static XFontSet fontset;
+#elif defined(USE_WINDOWS)
+static HFONT font;
+static int totlines;
+#endif /* USE_WINDOWS */
 
 #ifdef USE_WINDOWS
 static int modelWidth,modelHeight;
@@ -266,7 +268,23 @@ static void keypress(Widget w, XEvent *event, String *params, Cardinal *num_para
   keysym = XtGetActionKeysym(event, &modifiers);
   modifiers |= my_modifiers;
   LogPrint(LOG_DEBUG,"keypress(%#lx), modif(%#x)", keysym, modifiers);
-  if (keysym<0x100) keypressed = keysym | BRL_BLK_PASSCHAR;
+
+  /* latin1 */
+  if (keysym < 0x100) keysym |= 0x1000000;
+
+  if ((keysym & 0x1f000000) == 0x1000000) {
+    /* unicode */
+    if ((keysym & 0xffff00) == BRL_UC_ROW)
+      keypressed = BRL_BLK_PASSDOTS | (keysym & 0xff);
+    else {
+      int c = convertWcharToChar(keysym & 0xffffff);
+      if (c == EOF) {
+	LogPrint(LOG_DEBUG, "non translatable unicode U+%lx", keysym & 0xffffff);
+	return;
+      }
+      keypressed = BRL_BLK_PASSCHAR | c;
+    }
+  }
   else switch(keysym) {
     case XK_Shift_L:
     case XK_Shift_R:   modifier = ShiftMask;   goto modif;
@@ -359,14 +377,16 @@ static void keypress(Widget w, XEvent *event, String *params, Cardinal *num_para
     case XK_KP_7:         keypressed = BRL_BLK_PASSCHAR | '7'; break;
     case XK_KP_8:         keypressed = BRL_BLK_PASSCHAR | '8'; break;
     case XK_KP_9:         keypressed = BRL_BLK_PASSCHAR | '9'; break;
-    default: return;
+    default: LogPrint(LOG_DEBUG,"unsupported keysym %lx",keysym); return;
   }
   if (modifiers & ControlMask)
     keypressed |= BRL_FLG_CHAR_CONTROL;
   if (modifiers & Mod1Mask)
     keypressed |= BRL_FLG_CHAR_META;
-  if (modifiers & (ShiftMask|LockMask))
+  if (modifiers & ShiftMask)
     keypressed |= BRL_FLG_CHAR_SHIFT;
+  if (modifiers & LockMask)
+    keypressed |= BRL_FLG_CHAR_UPPER;
   if (event->type == KeyPress)
     keypressed |= BRL_FLG_REPEAT_DELAY | BRL_FLG_REPEAT_INITIAL;
   else
@@ -410,7 +430,7 @@ static inline Widget crKeyBut(char *name, long keycode, int repeat,
     NULL);
   XtAddCallback(button, Ncallback, KeyPressCB, (XtPointer) keycode);
 #elif defined(USE_WINDOWS)
-  button = CreateWindow(WC_BUTTON, name, WS_CHILD | WS_VISIBLE, horizDistance, lines*CHRY+1+vertDistance, BUTWIDTH, BUTHEIGHT, toplevel, NULL, NULL, NULL);
+  button = CreateWindow(WC_BUTTON, name, WS_CHILD | WS_VISIBLE, horizDistance, totlines*CHRY+1+vertDistance, BUTWIDTH, BUTHEIGHT, toplevel, NULL, NULL, NULL);
   SetWindowLong(button, GWL_USERDATA, (long) keycode);
 #else /* USE_ */
 #error Toolkit button creation unspecified
@@ -434,16 +454,33 @@ struct model {
 static struct model *keyModel;
 
 static struct button buttons_simple[] = {
-  { "^",      BRL_CMD_LNUP,   1, 1, 0 },
-  { "<",      BRL_CMD_FWINLT, 1, 0, 1 },
-  { "Home",   BRL_CMD_HOME,   0, 1, 1 },
-  { ">",      BRL_CMD_FWINRT, 1, 2, 1 },
-  { "v",      BRL_CMD_LNDN,   1, 1, 2 },
-  { "alt-a",  BRL_FLG_CHAR_META    | BRL_BLK_PASSCHAR | 'a', 0, 0, 3 },
-  { "ctrl-a", BRL_FLG_CHAR_CONTROL | BRL_BLK_PASSCHAR | 'a', 0, 1, 3 },
-  { "a",      BRL_BLK_PASSCHAR                        | 'a', 0, 2, 3 },
-  { "A",      BRL_BLK_PASSCHAR                        | 'A', 0, 3, 3 },
-  { NULL,     0,              0, 0, 0},
+  { "Dot1",   BRL_BLK_PASSDOTS  | BRL_DOT1  , 0, 0, 0 },
+  { "Dot2",   BRL_BLK_PASSDOTS  | BRL_DOT2  , 0, 0, 1 },
+  { "Dot3",   BRL_BLK_PASSDOTS  | BRL_DOT3  , 0, 0, 2 },
+  { "Dot4",   BRL_BLK_PASSDOTS  | BRL_DOT4  , 0, 1, 0 },
+  { "Dot5",   BRL_BLK_PASSDOTS  | BRL_DOT5  , 0, 1, 1 },
+  { "Dot6",   BRL_BLK_PASSDOTS  | BRL_DOT6  , 0, 1, 2 },
+  { "Dot7",   BRL_BLK_PASSDOTS  | BRL_DOT7  , 0, 0, 3 },
+  { "Dot8",   BRL_BLK_PASSDOTS  | BRL_DOT8  , 0, 1, 3 },
+  { "`",      BRL_CMD_TOP_LEFT, 0, 3, 0 },
+  { "^",      BRL_CMD_LNUP,   1, 4, 0 },
+  { "<",      BRL_CMD_FWINLT, 1, 3, 1 },
+  { "Home",   BRL_CMD_HOME,   0, 4, 1 },
+  { ">",      BRL_CMD_FWINRT, 1, 5, 1 },
+  { "<=",     BRL_CMD_FWINLTSKIP, 0, 3, 2 },
+  { "v",      BRL_CMD_LNDN,   1, 4, 2 },
+  { "=>",     BRL_CMD_FWINRTSKIP, 0, 5, 2 },
+  { "alt-c",  BRL_FLG_CHAR_META    | BRL_BLK_PASSCHAR | 'c', 0, 3, 3 },
+  { "ctrl-c", BRL_FLG_CHAR_CONTROL | BRL_BLK_PASSCHAR | 'c', 0, 4, 3 },
+  { "a",      BRL_BLK_PASSCHAR                        | 'a', 0, 5, 3 },
+  { "A",      BRL_BLK_PASSCHAR                        | 'A', 0, 6, 3 },
+  { "Alt-F1", BRL_FLG_CHAR_META | BRL_KEY_FUNCTION | BRL_BLK_PASSKEY , 0, 7, 3 },
+  { "Frez",   BRL_CMD_FREEZE,   0, 6, 0 },
+  { "Help",   BRL_CMD_HELP,     0, 7, 0 },
+  { "Pref",   BRL_CMD_PREFMENU, 0, 6, 1 },
+  { "PL",     BRL_CMD_PREFLOAD, 0, 6, 2 },
+  { "PS",     BRL_CMD_PREFSAVE, 0, 7, 2 },
+  { NULL,     0,                0, 0, 0},
 };
 
 static struct button buttons_vs[] = {
@@ -492,7 +529,7 @@ static struct model models[] = {
 
 static void setModel(Widget w, XtPointer closure, XtPointer data)
 {
-  int newModel = (int) closure;
+  intptr_t newModel = (intptr_t) closure;
   if (newModel == XtNumber(models))
     keyModel = NULL;
   else
@@ -513,6 +550,7 @@ struct radioInt {
 
 static struct radioInt colsRadio [] = {
   { "80", 80 },
+  { "60", 60 },
   { "40", 40 },
   { "20", 20 },
   { "8",  8  },
@@ -526,14 +564,14 @@ static struct radioInt linesRadio [] = {
 
 static void setWidth(Widget w, XtPointer closure, XtPointer data)
 {
-  int newCols = (int) closure;
+  intptr_t newCols = (intptr_t) closure;
   cols = newCols;
   regenerate = 1;
 }
 
 static void setHeight(Widget w, XtPointer closure, XtPointer data)
 {
-  int newLines = (int) closure;
+  intptr_t newLines = (intptr_t) closure;
   lines = newLines;
   regenerate = 1;
 }
@@ -556,7 +594,7 @@ static actionfun_t actionfun[] = {
 #define SET_ACTION(cb, set) \
   (cb)[0].callback = (XtCallbackProc) actionfun[set]
 #define SET_VALUE(cb, value) \
-  (cb)[0].closure = (caddr_t) (value)
+  (cb)[0].closure = (void*)(intptr_t) (value)
 #elif defined(USE_WINDOWS)
 #define SET_ACTION(cb, set) \
   (cb) = (set) << 8
@@ -587,6 +625,22 @@ static LRESULT CALLBACK wndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lPar
   }
   return DefWindowProc(hwnd, uMsg, wParam, lParam);
 }
+
+#define BRAILLE_USB 82
+int CALLBACK fontEnumProc(ENUMLOGFONTEX *lpelfe, NEWTEXTMETRICEX *lpntme, DWORD FontType, LPARAM lParam) {
+	int shift = 8*sizeof(lpntme->ntmFontSig.fsUsb[0]);
+	if (!(lpntme->ntmFontSig.fsUsb[BRAILLE_USB / shift] &
+		(1 << (BRAILLE_USB % shift))))
+		return 1;
+	font = CreateFont(CHRY-6, CHRX-4, 0, 0, 0, FALSE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, lpelfe->elfFullName);
+	if (!font) {
+		LogWindowsError("Couldn't load font");
+		LogPrint(LOG_ERR,"font %s", lpelfe->elfFullName);
+		return 1;
+	}
+	LogPrint(LOG_INFO, "Using braille font `%s\'\n",lpelfe->elfFullName);
+	return 0;
+}
 #endif /* USE_WINDOWS */
 
 static int brl_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext context)
@@ -600,7 +654,7 @@ static int brl_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext context
 
     while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
       if (msg.message == WM_QUIT)
-	return BRL_CMD_RESTARTBRL;
+	return BRL_CMD_SHUTDOWN;
       else {
 	TranslateMessage(&msg);
 	DispatchMessage(&msg);
@@ -626,8 +680,6 @@ static int brl_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext context
 }
 
 #ifdef USE_XT
-static XrmOptionDescRec optionDescList[] = { };
-
 static char *fallback_resources[] = {
   "*display.background: lightgreen",
 #ifdef USE_XAW
@@ -641,12 +693,6 @@ static char *fallback_resources[] = {
 };
 #endif /* USE_XT */
 
-static void brl_identify()
-{
-  LogPrint(LOG_NOTICE, XW_VERSION);
-  LogPrint(LOG_INFO,   "   "XW_COPYRIGHT);
-}
-
 #ifdef USE_XM
 static void popup(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
@@ -699,15 +745,31 @@ static void generateToplevel(void)
 
   /* toplevel */
   toplevel = XtVaOpenApplication(&app_con, "Brltty",
-    optionDescList, XtNumber(optionDescList),
+    NULL, 0,
     &argc, argv, fallback_resources,
     sessionShellWidgetClass,
     XtNallowShellResize, True,
+    XtNinput, input ? True : False,
     NULL);
 
+  free(argv);
+
   XtAppAddActions(app_con,actions,XtNumber(actions));
 
 #elif defined(USE_WINDOWS)
+  {
+    HWND root = GetDesktopWindow();
+    HDC hdc = GetDC(root);
+    EnumFontFamiliesEx(hdc, NULL, (void*) fontEnumProc, 0, 0);
+    ReleaseDC(root, hdc);
+    if (!font) {
+      LogPrint(LOG_ERR,"Error while loading braille font");
+      totlines = lines;
+    } else {
+      totlines = 2*lines;
+    }
+  }
+
   {
     WNDCLASS wndclass = {
       .style = 0,
@@ -737,7 +799,7 @@ static void generateToplevel(void)
     if (!(toplevel = CreateWindowEx(WS_EX_TOPMOST | WS_EX_TOOLWINDOW,
 	    "BRLTTYWClass", "BRLTTY",
 	    WS_POPUP, GetSystemMetrics(SM_CXSCREEN)-modelWidth-RIGHTMARGIN, 0,
-	    modelWidth, lines*CHRY+modelHeight, NULL, NULL, NULL, NULL))) {
+	    modelWidth, totlines*CHRY+modelHeight, NULL, NULL, NULL, NULL))) {
       LogWindowsError("CreateWindow");
       exit(1);
     }
@@ -804,7 +866,7 @@ static void generateToplevel(void)
 
 #ifdef USE_WINDOWS
   hbox = CreateWindow(WC_STATIC, "", WS_CHILD | WS_VISIBLE, 0, 0,
-       modelWidth, lines*CHRY+modelHeight, toplevel, NULL, NULL, NULL);
+       modelWidth, totlines*CHRY+modelHeight, toplevel, NULL, NULL, NULL);
 #endif /* USE_WINDOWS */
 
   for (x=0;x<cols;x++) {
@@ -865,6 +927,11 @@ static void generateToplevel(void)
 #elif defined(USE_WINDOWS)
       display[y*cols+x] = CreateWindow(WC_BUTTON, " ", WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_PUSHLIKE, x*CHRX, y*CHRY, CHRX, CHRY, toplevel, NULL, NULL, NULL);
       SetWindowLong(display[y*cols+x], GWL_USERDATA, (long) (BRL_BLK_ROUTE | ((y*cols+x)&BRL_MSK_ARG)));
+      if (font) {
+        displayb[y*cols+x] = CreateWindowW(WC_BUTTONW, L" ", WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_PUSHLIKE, x*CHRX, (lines+y)*CHRY, CHRX, CHRY, toplevel, NULL, NULL, NULL);
+        SetWindowLong(displayb[y*cols+x], GWL_USERDATA, (long) (BRL_BLK_ROUTE | ((y*cols+x)&BRL_MSK_ARG)));
+	SendMessage(displayb[y*cols+x], WM_SETFONT, (WPARAM) font, TRUE);
+      }
 #else /* USE_ */
 #error Toolkit display unspecified
 #endif /* USE_ */
@@ -958,37 +1025,46 @@ static void generateToplevel(void)
 #else /* USE_ */
 #error Toolkit toplevel realization unspecified
 #endif /* USE_ */
-#ifdef USE_XAW
+#if defined(USE_XAW) || defined(USE_WINDOWS)
   memset(displayedWindow,0,sizeof(displayedWindow));
-#endif /* USE_XAW */
+#endif /* USE_XAW || USE_WINDOWS */
   memset(displayedVisual,0,sizeof(displayedVisual));
   lastcursor = -1;
 }
 
-static int brl_open(BrailleDisplay *brl, char **parameters, const char *device)
+static int brl_construct(BrailleDisplay *brl, char **parameters, const char *device)
 {
   lines=1;
   if (*parameters[PARM_LINES]) {
     static const int minimum = 1;
     static const int maximum = MAXLINES;
     int value;
-    if (validateInteger(&value, "lines", parameters[PARM_LINES], &minimum, &maximum))
+    if (validateInteger(&value, parameters[PARM_LINES], &minimum, &maximum)) {
       lines=value;
+    } else {
+      LogPrint(LOG_WARNING, "%s: %s", "invalid line count", parameters[PARM_LINES]);
+    }
   }
 
   cols=40;
-  if (*parameters[PARM_COLS]) {
+  if (*parameters[PARM_COLUMNS]) {
     static const int minimum = 1;
     static const int maximum = MAXCOLS;
     int value;
-    if (validateInteger(&value, "cols", parameters[PARM_COLS], &minimum, &maximum))
+    if (validateInteger(&value, parameters[PARM_COLUMNS], &minimum, &maximum)) {
       cols=value;
+    } else {
+      LogPrint(LOG_WARNING, "%s: %s", "invalid column count", parameters[PARM_COLUMNS]);
+    }
   }
 
   if (*parameters[PARM_INPUT]) {
     unsigned int value;
-    if (validateFlag(&value, "input", parameters[PARM_INPUT], "on", "off"))
+    if (validateFlag(&value, parameters[PARM_INPUT], "on", "off")) {
       input = value;
+    } else {
+      LogPrint(LOG_WARNING, "%s: %s", "invalid input setting", parameters[PARM_INPUT]);
+    }
   }
 
   if (*parameters[PARM_TKPARMS]) {
@@ -1035,21 +1111,29 @@ static void destroyToplevel(void)
   DestroyMenu(menu);
   if (!DestroyWindow(toplevel))
     LogWindowsError("DestroyWindow");
+  if (font) {
+    DeleteObject(font);
+    font = NULL;
+  }
 #else /* USE_ */
 #error Toolkit toplevel destruction unspecified
 #endif /* USE_ */
 }
 
-static void brl_close(BrailleDisplay *brl)
+static void brl_destruct(BrailleDisplay *brl)
 {
   destroyToplevel();
 }
 
 static void brl_writeWindow(BrailleDisplay *brl)
 {
-#ifdef USE_XAW
+#if defined(USE_XAW) || defined(USE_WINDOWS)
   int i;
+#ifdef USE_XAW
   unsigned char data[4];
+#elif defined(USE_WINDOWS)
+  wchar_t data[2] = {0};
+#endif
   char c;
 
   if (!displayb[0] || !memcmp(brl->buffer,displayedWindow,brl->y*brl->x)) return;
@@ -1057,7 +1141,7 @@ static void brl_writeWindow(BrailleDisplay *brl)
 
   for (i=0;i<brl->y*brl->x;i++) {
     c = brl->buffer[i];
-    brl->buffer[i] =
+    c =
        (!!(c&BRL_DOT1))<<0
       |(!!(c&BRL_DOT2))<<1
       |(!!(c&BRL_DOT3))<<2
@@ -1066,16 +1150,22 @@ static void brl_writeWindow(BrailleDisplay *brl)
       |(!!(c&BRL_DOT6))<<5
       |(!!(c&BRL_DOT7))<<6
       |(!!(c&BRL_DOT8))<<7;
+#ifdef USE_XAW
     data[0]=0xe0|((0x28>>4)&0x0f);
-    data[1]=0x80|((0x28<<2)&0x3f)|(brl->buffer[i]>>6);
-    data[2]=0x80                 |(brl->buffer[i]&0x3f);
+    data[1]=0x80|((0x28<<2)&0x3f)|(c>>6);
+    data[2]=0x80                 |(c&0x3f);
     data[3]=0;
 
     XtVaSetValues(displayb[i],
       XtNlabel, data,
       NULL);
+#elif defined(USE_WINDOWS)
+    data[0] = BRL_UC_ROW | c;
+    data[1] = 0;
+    SetWindowTextW(displayb[i],data);
+#endif /* USE_WINDOWS */
   }
-#endif /* USE_XAW */
+#endif /* USE_XAW || USE_WINDOWS */
 }
 
 static void brl_writeVisual(BrailleDisplay *brl)
