@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -17,47 +17,47 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
 
 #include "misc.h"
-#include "sysmisc.h"
+#include "async.h"
 #include "message.h"
+#include "drivers.h"
 #include "brl.h"
 #include "tbl.h"
 #include "brl.auto.h"
 #include "cmd.h"
 
 #define BRLSYMBOL noBraille
-#define BRLNAME NoBraille
-#define BRLCODE no
-#define BRLCOMMENT ""
+#define DRIVER_NAME NoBraille
+#define DRIVER_CODE no
+#define DRIVER_COMMENT "no braille support"
+#define DRIVER_VERSION ""
+#define DRIVER_DEVELOPERS ""
 #define BRLHELP "/dev/null"
 #include "brl_driver.h"
-static void
-brl_identify (void) {
-  LogPrint(LOG_NOTICE, "No braille support.");
-}
+
 static int
-brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
-  brl->x = 80;
-  brl->y = 1;
+brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
   return 1;
 }
+
 static void
-brl_close (BrailleDisplay *brl) {
+brl_destruct (BrailleDisplay *brl) {
 }
+
 static int
 brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
   return EOF;
 }
+
 static void
 brl_writeWindow (BrailleDisplay *brl) {
 }
+
 static void
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *status) {
 }
@@ -79,31 +79,49 @@ TranslationTable attributesTable = {
 
 void *contractionTable = NULL;
 
+int
+haveBrailleDriver (const char *code) {
+  return haveDriver(code, BRAILLE_DRIVER_CODES, driverTable);
+}
+
+const char *
+getDefaultBrailleDriver (void) {
+  return getDefaultDriver(driverTable);
+}
+
 const BrailleDriver *
 loadBrailleDriver (const char *code, void **driverObject, const char *driverDirectory) {
   return loadDriver(code, driverObject,
                     driverDirectory, driverTable,
-                    "braille", 'b', "brl_driver",
-                    &noBraille, noBraille.code);
+                    "braille", 'b', "brl",
+                    &noBraille, &noBraille.definition);
 }
 
 void
-identifyBrailleDrivers (void) {
+identifyBrailleDriver (const BrailleDriver *driver, int full) {
+  identifyDriver("Braille", &driver->definition, full);
+}
+
+void
+identifyBrailleDrivers (int full) {
   const DriverEntry *entry = driverTable;
   while (entry->address) {
     const BrailleDriver *driver = entry++->address;
-    driver->identify();
+    identifyBrailleDriver(driver, full);
   }
 }
 
 void
 initializeBrailleDisplay (BrailleDisplay *brl) {
-   brl->x = brl->y = 0;
-   brl->helpPage = 0;
-   brl->buffer = NULL;
-   brl->writeDelay = 0;
-   brl->bufferResized = NULL;
-   brl->dataDirectory = NULL;
+  brl->x = 80;
+  brl->y = 1;
+  brl->helpPage = 0;
+  brl->buffer = NULL;
+  brl->writeDelay = 0;
+  brl->bufferResized = NULL;
+  brl->dataDirectory = NULL;
+  brl->touchEnabled = 0;
+  brl->highlightWindow = 0;
 }
 
 unsigned int
@@ -111,7 +129,7 @@ drainBrailleOutput (BrailleDisplay *brl, int minimumDelay) {
   int duration = brl->writeDelay + 1;
   if (duration < minimumDelay) duration = minimumDelay;
   brl->writeDelay = 0;
-  approximateDelay(duration);
+  asyncWait(duration);
   return duration;
 }
 
@@ -171,38 +189,52 @@ setStatusText (BrailleDisplay *brl, const char *text) {
 }
 
 static void
-brailleBufferResized (BrailleDisplay *brl) {
+brailleBufferResized (BrailleDisplay *brl, int infoLevel) {
   memset(brl->buffer, 0, brl->x*brl->y);
-  LogPrint(LOG_INFO, "Braille Display Dimensions: %d %s, %d %s",
+  LogPrint(infoLevel, "Braille Display Dimensions: %d %s, %d %s",
            brl->y, (brl->y == 1)? "row": "rows",
            brl->x, (brl->x == 1)? "column": "columns");
-  if (brl->bufferResized) brl->bufferResized(brl->y, brl->x);
+  if (brl->bufferResized) brl->bufferResized(infoLevel, brl->y, brl->x);
 }
 
 static int
-resizeBrailleBuffer (BrailleDisplay *brl) {
+resizeBrailleBuffer (BrailleDisplay *brl, int infoLevel) {
   if (brl->resizeRequired) {
     brl->resizeRequired = 0;
+
     if (brl->isCoreBuffer) {
-      int size = brl->x * brl->y;
-      unsigned char *buffer = realloc(brl->buffer, size);
-      if (!buffer) {
-        LogError("braille buffer allocation");
-        return 0;
+      static void *currentAddress = NULL;
+      static size_t currentSize = 0;
+      size_t newSize = brl->x * brl->y;
+
+      if (newSize > currentSize) {
+        void *newAddress = malloc(newSize);
+
+        if (!newAddress) {
+          LogError("malloc");
+          return 0;
+        }
+
+        if (currentAddress) free(currentAddress);
+        currentAddress = newAddress;
+        currentSize = newSize;
       }
-      brl->buffer = buffer;
+
+      brl->buffer = currentAddress;
     }
-    brailleBufferResized(brl);
+
+    brailleBufferResized(brl, infoLevel);
   }
+
   return 1;
 }
 
 int
-allocateBrailleBuffer (BrailleDisplay *brl) {
+ensureBrailleBuffer (BrailleDisplay *brl, int infoLevel) {
   if ((brl->isCoreBuffer = brl->resizeRequired = brl->buffer == NULL)) {
-    if (!resizeBrailleBuffer(brl)) return 0;
+    if (!resizeBrailleBuffer(brl, infoLevel)) return 0;
   } else {
-    brailleBufferResized(brl);
+    brailleBufferResized(brl, infoLevel);
   }
   return 1;
 }
@@ -210,7 +242,7 @@ allocateBrailleBuffer (BrailleDisplay *brl) {
 int
 readBrailleCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
   int command = braille->readCommand(brl, context);
-  resizeBrailleBuffer(brl);
+  resizeBrailleBuffer(brl, LOG_INFO);
   return command;
 }
 
@@ -354,3 +386,9 @@ setBrailleFirmness (BrailleDisplay *brl, int setting) {
   LogPrint(LOG_DEBUG, "setting braille firmness: %d", setting);
   braille->firmness(brl, setting);
 }
+
+void
+setBrailleSensitivity (BrailleDisplay *brl, int setting) {
+  LogPrint(LOG_DEBUG, "setting braille sensitivity: %d", setting);
+  braille->sensitivity(brl, setting);
+}
