@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -24,8 +24,6 @@
 extern "C" {
 #endif /* __cplusplus */
 
-#include <inttypes.h>
-
 /* help.h - describes the helpfile format
  * $Id: help.h,v 1.3 1996/09/24 01:04:26 nn201 Exp $
  */
@@ -40,20 +38,12 @@ extern "C" {
 typedef struct {
   unsigned char pages;
   unsigned char unused;
-}
-#ifdef HAVE_ATTRIBUTE_PACKED
-  __attribute__((packed))
-#endif /* HAVE_ATTRIBUTE_PACKED */
-  HelpFileHeader;
+} PACKED HelpFileHeader;
 
 typedef struct {
   uint16_t height;
   uint16_t width;
-}
-#ifdef HAVE_ATTRIBUTE_PACKED
-  __attribute__((packed))
-#endif /* HAVE_ATTRIBUTE_PACKED */
-  HelpPageEntry;
+} PACKED HelpPageEntry;
 
 #ifdef __cplusplus
 }
