@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -23,7 +23,10 @@ extern "C" {
 #endif /* __cplusplus */
 
 extern int isBluetoothDevice (const char **path);
-extern int openRfcommConnection (const char *address, unsigned char channel);
+
+extern void btForgetConnectErrors (void);
+
+extern int btOpenConnection (const char *address, unsigned char channel, int force);
 
 #ifdef __cplusplus
 }
