@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,18 +15,15 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
 #include <ctype.h>
 #include <errno.h>
 
+#include "program.h"
 #include "options.h"
 #include "misc.h"
 #include "system.h"
@@ -35,12 +32,6 @@
 #include <getopt.h>
 #endif /* HAVE_GETOPT_H */
 
-const char *programPath;
-const char *programName;
-
-typedef char **StringSetting;
-typedef int *FlagSetting;
-
 typedef struct {
   const OptionEntry *optionTable;
   unsigned int optionCount;
@@ -48,6 +39,16 @@ typedef struct {
   int errorCount;
 } OptionProcessingInformation;
 
+static int
+wordMeansTrue (const char *word) {
+  return strcasecmp(word, FLAG_TRUE_WORD) == 0;
+}
+
+static int
+wordMeansFalse (const char *word) {
+  return strcasecmp(word, FLAG_FALSE_WORD) == 0;
+}
+
 static void
 extendSetting (char **setting, const char *value, int prepend) {
   if (value && *value) {
@@ -72,33 +73,34 @@ ensureSetting (
   const OptionEntry *option,
   const char *value
 ) {
-  if (value) {
-    unsigned char *ensured = &info->ensuredSettings[option->letter];
-    if (!*ensured) {
-      *ensured = 1;
+  unsigned char *ensured = &info->ensuredSettings[option->letter];
 
-      if (option->argument) {
-        StringSetting setting = option->setting;
+  if (!*ensured) {
+    *ensured = 1;
+
+    if (option->argument) {
+      if (option->setting.string) {
         if (option->flags & OPT_Extend) {
-          extendSetting(setting, value, 1);
+          extendSetting(option->setting.string, value, 1);
         } else {
-          *setting = strdupWrapper(value);
+          *option->setting.string = strdupWrapper(value);
         }
-      } else {
-        FlagSetting setting = option->setting;
-        if (strcasecmp(value, "on") == 0) {
-          *setting = 1;
-        } else if (strcasecmp(value, "off") == 0) {
-          *setting = 0;
+      }
+    } else {
+      if (option->setting.flag) {
+        if (wordMeansTrue(value)) {
+          *option->setting.flag = 1;
+        } else if (wordMeansFalse(value)) {
+          *option->setting.flag = 0;
         } else if (!(option->flags & OPT_Extend)) {
-          LogPrint(LOG_ERR, "Invalid flag setting: %s", value);
+          LogPrint(LOG_ERR, "%s: %s", gettext("invalid flag setting"), value);
           info->errorCount++;
         } else {
           int count;
           if (isInteger(&count, value) && (count >= 0)) {
-            *setting = count;
+            *option->setting.flag = count;
           } else {
-            LogPrint(LOG_ERR, "Invalid counter setting: %s", value);
+            LogPrint(LOG_ERR, "%s: %s", gettext("invalid counter setting"), value);
             info->errorCount++;
           }
         }
@@ -116,28 +118,26 @@ printHelp (
   int all
 ) {
   char line[lineWidth+1];
+  unsigned int wordWidth = 0;
   unsigned int argumentWidth = 0;
   int optionIndex;
 
-#ifdef HAVE_GETOPT_LONG
-  unsigned int wordWidth = 0;
-#endif /* HAVE_GETOPT_LONG */
-
   for (optionIndex=0; optionIndex<info->optionCount; ++optionIndex) {
     const OptionEntry *option = &info->optionTable[optionIndex];
-
-#ifdef HAVE_GETOPT_LONG
     if (option->word) wordWidth = MAX(wordWidth, strlen(option->word));
-#endif /* HAVE_GETOPT_LONG */
-
     if (option->argument) argumentWidth = MAX(argumentWidth, strlen(option->argument));
   }
 
-  fprintf(outputStream, "Usage: %s", programName);
-  if (info->optionCount)
-    fprintf(outputStream, " [option ...]");
-  if (argumentsSummary && *argumentsSummary)
+  fputs(gettext("Usage"), outputStream);
+  fprintf(outputStream, ": %s", programName);
+  if (info->optionCount) {
+    fputs(" [", outputStream);
+    fputs(gettext("option"), outputStream);
+    fputs(" ...]", outputStream);
+  }
+  if (argumentsSummary && *argumentsSummary) {
     fprintf(outputStream, " %s", argumentsSummary);
+  }
   fprintf(outputStream, "\n");
 
   for (optionIndex=0; optionIndex<info->optionCount; ++optionIndex) {
@@ -161,7 +161,6 @@ printHelp (
     }
     line[lineLength++] = ' ';
 
-#ifdef HAVE_GETOPT_LONG
     {
       unsigned int end = lineLength + 2 + wordWidth + 1;
       if (option->word) {
@@ -175,35 +174,54 @@ printHelp (
       while (lineLength < end) line[lineLength++] = ' ';
     }
     line[lineLength++] = ' ';
-#endif /* HAVE_GETOPT_LONG */
 
     line[lineLength++] = ' ';
     {
       unsigned int headerWidth = lineLength;
       unsigned int descriptionWidth = lineWidth - headerWidth;
-      const char *description = option->description;
-      unsigned int charsLeft = strlen(description);
+      const char *description = option->description? gettext(option->description): "";
+      char buffer[0X100];
 
-      while (1) {
-        unsigned int charCount = charsLeft;
-        if (charCount > descriptionWidth) {
-          charCount = descriptionWidth;
-          while (description[charCount] != ' ') --charCount;
-          while (description[charCount] == ' ') --charCount;
-          ++charCount;
+      if (option->strings) {
+        int index = 0;
+        int count = 4;
+        const char *strings[count];
+
+        while (option->strings[index]) {
+          strings[index] = option->strings[index];
+          ++index;
         }
-        memcpy(line+lineLength, description, charCount);
-        lineLength += charCount;
 
-        line[lineLength] = 0;
-        fprintf(outputStream, "%s\n", line);
+        while (index < count) strings[index++] = NULL;
+        snprintf(buffer, sizeof(buffer),
+                 description, strings[0], strings[1], strings[2], strings[3]);
+        description = buffer;
+      }
 
-        while (description[charCount] == ' ') ++charCount;
-        if (!(charsLeft -= charCount)) break;
-        description += charCount;
+      {
+        unsigned int charsLeft = strlen(description);
 
-        lineLength = 0;
-        while (lineLength < headerWidth) line[lineLength++] = ' ';
+        while (1) {
+          unsigned int charCount = charsLeft;
+          if (charCount > descriptionWidth) {
+            charCount = descriptionWidth;
+            while (description[charCount] != ' ') --charCount;
+            while (description[charCount] == ' ') --charCount;
+            ++charCount;
+          }
+          memcpy(line+lineLength, description, charCount);
+          lineLength += charCount;
+
+          line[lineLength] = 0;
+          fprintf(outputStream, "%s\n", line);
+
+          while (description[charCount] == ' ') ++charCount;
+          if (!(charsLeft -= charCount)) break;
+          description += charCount;
+
+          lineLength = 0;
+          while (lineLength < headerWidth) line[lineLength++] = ' ';
+        }
       }
     }
   }
@@ -216,44 +234,53 @@ processCommandLine (
   char ***argumentVector,
   const char *argumentsSummary
 ) {
-  int opt_help = 0;
-  int opt_helpAll = 0;
+  int lastOptInd = -1;
+  int index;
 
+  const char resetPrefix = '+';
+  const char *reset = NULL;
+  int resetLetter;
+
+  const char dosPrefix = '/';
+  int dosSyntax = 0;
+
+  int optHelp = 0;
+  int optHelpAll = 0;
   const OptionEntry *optionEntries[0X100];
   char shortOptions[1 + (info->optionCount * 2) + 1];
-  int index;
 
 #ifdef HAVE_GETOPT_LONG
   struct option longOptions[(info->optionCount * 2) + 1];
-  int flagLetter;
 
   {
     struct option *opt = longOptions;
     for (index=0; index<info->optionCount; ++index) {
       const OptionEntry *entry = &info->optionTable[index];
 
-      opt->name = entry->word;
-      opt->has_arg = entry->argument? required_argument: no_argument;
-      opt->flag = NULL;
-      opt->val = entry->letter;
-      ++opt;
-
-      if (!entry->argument && entry->setting) {
-        static const char *prefix = "no-";
-        int length = strlen(prefix);
-
-        if (strncasecmp(prefix, entry->word, length) == 0) {
-          opt->name = strdupWrapper(entry->word + length);
-        } else {
-          char *name = mallocWrapper(length + strlen(entry->word) + 1);
-          sprintf(name, "%s%s", prefix, entry->word);
-          opt->name = name;
-        }
-
-        opt->has_arg = no_argument;
-        opt->flag = &flagLetter;
+      if (entry->word) {
+        opt->name = entry->word;
+        opt->has_arg = entry->argument? required_argument: no_argument;
+        opt->flag = NULL;
         opt->val = entry->letter;
         ++opt;
+
+        if (!entry->argument && entry->setting.flag) {
+          static const char *prefix = "no-";
+          int length = strlen(prefix);
+
+          if (strncasecmp(prefix, entry->word, length) == 0) {
+            opt->name = strdupWrapper(entry->word + length);
+          } else {
+            char *name = mallocWrapper(length + strlen(entry->word) + 1);
+            sprintf(name, "%s%s", prefix, entry->word);
+            opt->name = name;
+          }
+
+          opt->has_arg = no_argument;
+          opt->flag = &resetLetter;
+          opt->val = entry->letter;
+          ++opt;
+        }
       }
     }
 
@@ -274,29 +301,124 @@ processCommandLine (
       *opt++ = entry->letter;
       if (entry->argument) *opt++ = ':';
 
-      if (entry->setting) {
-        if (entry->argument) {
-          StringSetting setting = entry->setting;
-          *setting = NULL;
-        } else {
-          FlagSetting setting = entry->setting;
-          *setting = 0;
-        }
+      if (entry->argument) {
+        if (entry->setting.string) *entry->setting.string = NULL;
+      } else {
+        if (entry->setting.flag) *entry->setting.flag = 0;
       }
     }
 
     *opt = 0;
   }
 
+  if (*argumentCount > 1)
+    if (*(*argumentVector)[1] == dosPrefix)
+      dosSyntax = 1;
+
   opterr = 0;
+  optind = 1;
+
   while (1) {
     int option;
+    char prefix = '-';
+
+    if (optind == *argumentCount) {
+      option = -1;
+    } else {
+      char *argument = (*argumentVector)[optind];
+
+      if (dosSyntax) {
+        prefix = dosPrefix;
+        optind++;
+
+        if (*argument != dosPrefix) {
+          option = -1;
+        } else {
+          char *name = argument + 1;
+          size_t nameLength = strcspn(name, ":");
+          char *value = (nameLength == strlen(name))? NULL: (name + nameLength + 1);
+          const OptionEntry *entry;
+
+          if (nameLength == 1) {
+            entry = optionEntries[option = *name];
+          } else {
+            int count = info->optionCount;
+            entry = info->optionTable;
+            option = -1;
+
+            while (count--) {
+              if (entry->word) {
+                size_t wordLength = strlen(entry->word);
+
+                if ((wordLength == nameLength) &&
+                    (strncasecmp(entry->word, name, wordLength) == 0)) {
+                  option = entry->letter;
+                  break;
+                }
+              }
+
+              entry++;
+            }
+
+            if (option < 0) {
+              option = 0;
+              entry = NULL;
+            }
+          }
+
+          optopt = option;
+          optarg = NULL;
+
+          if (!entry) {
+            option = '?';
+          } else if (entry->argument) {
+            if (!(optarg = value)) option = ':';
+          } else if (value) {
+            if (!entry->setting.flag) goto dosBadFlagValue;
+
+            if (!wordMeansTrue(value)) {
+              if (wordMeansFalse(value)) {
+                resetLetter = option;
+                option = 0;
+              } else {
+              dosBadFlagValue:
+                option = '?';
+              }
+            }
+          }
+        }
+      } else if (reset) {
+        prefix = resetPrefix;
+
+        if (!(option = *reset++)) {
+          reset = NULL;
+          optind++;
+          continue;
+        }
+
+        {
+          const OptionEntry *entry = optionEntries[option];
+          if (entry && !entry->argument && entry->setting.flag) {
+            resetLetter = option;
+            option = 0;
+          } else {
+            optopt = option;
+            option = '?';
+          }
+        }
+      } else {
+        if (optind != lastOptInd) {
+          lastOptInd = optind;
+          if ((reset = (*argument == resetPrefix)? argument+1: NULL)) continue;
+        }
 
 #ifdef HAVE_GETOPT_LONG
-    option = getopt_long(*argumentCount, *argumentVector, shortOptions, longOptions, NULL);
+        option = getopt_long(*argumentCount, *argumentVector, shortOptions, longOptions, NULL);
 #else /* HAVE_GETOPT_LONG */
-    option = getopt(*argumentCount, *argumentVector, shortOptions);
+        option = getopt(*argumentCount, *argumentVector, shortOptions);
 #endif /* HAVE_GETOPT_LONG */
+      }
+    }
     if (option == -1) break;
 
     /* continue on error as much as possible, as often we are typing blind
@@ -307,18 +429,25 @@ processCommandLine (
         const OptionEntry *entry = optionEntries[option];
 
         if (entry->argument) {
-          StringSetting setting = entry->setting;
-          if (entry->flags & OPT_Extend) {
-            extendSetting(setting, optarg, 0);
-          } else {
-            *setting = optarg;
+          if (!*optarg) {
+            info->ensuredSettings[option] = 0;
+            break;
+          }
+
+          if (entry->setting.string) {
+            if (entry->flags & OPT_Extend) {
+              extendSetting(entry->setting.string, optarg, 0);
+            } else {
+              *entry->setting.string = optarg;
+            }
           }
         } else {
-          FlagSetting setting = entry->setting;
-          if (entry->flags & OPT_Extend) {
-            ++*setting;
-          } else {
-            *setting = 1;
+          if (entry->setting.flag) {
+            if (entry->flags & OPT_Extend) {
+              ++*entry->setting.flag;
+            } else {
+              *entry->setting.flag = 1;
+            }
           }
         }
 
@@ -326,37 +455,34 @@ processCommandLine (
         break;
       }
 
-#ifdef HAVE_GETOPT_LONG
       case 0: {
-        const OptionEntry *entry = optionEntries[flagLetter];
-        FlagSetting setting = entry->setting;
-        *setting = 0;
-        info->ensuredSettings[flagLetter] = 1;
+        const OptionEntry *entry = optionEntries[resetLetter];
+        *entry->setting.flag = 0;
+        info->ensuredSettings[resetLetter] = 1;
         break;
       }
-#endif /* HAVE_GETOPT_LONG */
 
       case '?':
-        LogPrint(LOG_ERR, "Unknown option: -%c", optopt);
+        LogPrint(LOG_ERR, "%s: %c%c", gettext("unknown option"), prefix, optopt);
         info->errorCount++;
         break;
 
       case ':': /* An invalid option has been specified. */
-        LogPrint(LOG_ERR, "Missing operand: -%c", optopt);
+        LogPrint(LOG_ERR, "%s: %c%c", gettext("missing operand"), prefix, optopt);
         info->errorCount++;
         break;
 
       case 'H':                /* help */
-        opt_helpAll = 1;
+        optHelpAll = 1;
       case 'h':                /* help */
-        opt_help = 1;
+        optHelp = 1;
         break;
     }
   }
   *argumentVector += optind, *argumentCount -= optind;
 
-  if (opt_help) {
-    printHelp(info, stdout, 79, argumentsSummary, opt_helpAll);
+  if (optHelp) {
+    printHelp(info, stdout, 79, argumentsSummary, optHelpAll);
     exit(0);
   }
 
@@ -395,7 +521,7 @@ processBootParameters (
     for (optionIndex=0; optionIndex<info->optionCount; ++optionIndex) {
       const OptionEntry *option = &info->optionTable[optionIndex];
       if ((option->bootParameter) && (option->bootParameter <= count)) {
-        char *parameter = parameters[option->bootParameter-1];
+        const char *parameter = parameters[option->bootParameter-1];
         if (*parameter) ensureSetting(info, option, parameter);
       }
     }
@@ -415,12 +541,12 @@ processEnvironmentVariables (
   int optionIndex;
   for (optionIndex=0; optionIndex<info->optionCount; ++optionIndex) {
     const OptionEntry *option = &info->optionTable[optionIndex];
-    if (option->flags & OPT_Environ) {
-      unsigned char name[prefixLength + 1 + strlen(option->word) + 1];
+    if ((option->flags & OPT_Environ) && option->word) {
+      char name[prefixLength + 1 + strlen(option->word) + 1];
       sprintf(name, "%s_%s", prefix, option->word);
 
       {
-        unsigned char *character = name;
+        char *character = name;
         while (*character) {
           if (*character == '-') {
             *character = '_';
@@ -431,7 +557,10 @@ processEnvironmentVariables (
         }
       }
 
-      ensureSetting(info, option, getenv(name));
+      {
+        char *setting = getenv(name);
+        if (setting && *setting) ensureSetting(info, option, setting);
+      }
     }
   }
 }
@@ -444,8 +573,11 @@ setDefaultOptions (
   int optionIndex;
   for (optionIndex=0; optionIndex<info->optionCount; ++optionIndex) {
     const OptionEntry *option = &info->optionTable[optionIndex];
-    if (!(option->flags & OPT_Config) != !config) continue;
-    ensureSetting(info, option, option->defaultSetting);
+    if (!(option->flags & OPT_Config) == !config) {
+      const char *setting = option->defaultSetting;
+      if (!setting) setting = option->argument? "": FLAG_FALSE_WORD;
+      ensureSetting(info, option, setting);
+    }
   }
 }
 
@@ -473,22 +605,22 @@ processConfigurationLine (
     int optionIndex;
     for (optionIndex=0; optionIndex<conf->info->optionCount; ++optionIndex) {
       const OptionEntry *option = &conf->info->optionTable[optionIndex];
-      if (option->flags & OPT_Config) {
+      if ((option->flags & OPT_Config) && option->word) {
         if (strcasecmp(directive, option->word) == 0) {
           const char *operand = strtok(NULL, delimiters);
 
           if (!operand) {
-            LogPrint(LOG_ERR, "Operand not supplied for configuration directive: %s", line);
+            LogPrint(LOG_ERR, "%s: %s", gettext("operand not supplied for configuration directive"), line);
             conf->info->errorCount++;
           } else if (strtok(NULL, delimiters)) {
             while (strtok(NULL, delimiters));
-            LogPrint(LOG_ERR, "Too many operands for configuration directive: %s", line);
+            LogPrint(LOG_ERR, "%s: %s", gettext("too many operands for configuration directive"), line);
             conf->info->errorCount++;
           } else {
             char **setting = &conf->settings[optionIndex];
 
             if (*setting && !(option->argument && (option->flags & OPT_Extend))) {
-              LogPrint(LOG_ERR, "Configuration directive specified more than once: %s", line);
+              LogPrint(LOG_ERR, "%s: %s", gettext("configuration directive specified more than once"), line);
               conf->info->errorCount++;
               free(*setting);
               *setting = NULL;
@@ -505,7 +637,7 @@ processConfigurationLine (
         }
       }
     }
-    LogPrint(LOG_ERR, "Unknown configuration directive: %s", line);
+    LogPrint(LOG_ERR, "%s: %s", gettext("unknown configuration directive"), line);
     conf->info->errorCount++;
   }
   return 1;
@@ -517,7 +649,7 @@ processConfigurationFile (
   const char *path,
   int optional
 ) {
-  FILE *file = fopen(path, "r");
+  FILE *file = openDataFile(path, "r");
   if (file != NULL) { /* The configuration file has been successfully opened. */
     int processed;
 
@@ -533,20 +665,22 @@ processConfigurationFile (
 
       for (index=0; index<info->optionCount; ++index) {
         char *setting = conf.settings[index];
-        ensureSetting(info, &info->optionTable[index], setting);
-        free(setting);
+        if (setting) {
+          ensureSetting(info, &info->optionTable[index], setting);
+          free(setting);
+        }
       }
       free(conf.settings);
     }
 
     fclose(file);
     if (processed) return 1;
-    LogPrint(LOG_ERR, "File '%s' processing error.", path);
+    LogPrint(LOG_ERR, gettext("file '%s' processing error."), path);
     info->errorCount++;
   } else {
     int ok = optional && (errno == ENOENT);
-    LogPrint((ok? LOG_DEBUG: LOG_ERR),
-             "Cannot open configuration file: %s: %s",
+    LogPrint((ok? LOG_DEBUG: LOG_ERR), "%s: %s: %s",
+             gettext("cannot open configuration file"),
              path, strerror(errno));
     if (ok) return 1;
     info->errorCount++;
@@ -555,81 +689,38 @@ processConfigurationFile (
 }
 
 int
-processOptions (
-  const OptionEntry *optionTable,
-  unsigned int optionCount,
-  const char *applicationName,
-  int *argumentCount,
-  char ***argumentVector,
-  int *doBootParameters,
-  int *doEnvironmentVariables,
-  char **configurationFile,
-  const char *argumentsSummary
-) {
-  OptionProcessingInformation info;
-  int index;
+processOptions (const OptionsDescriptor *descriptor, int *argumentCount, char ***argumentVector) {
+  OptionProcessingInformation info = {
+    .optionTable = descriptor->optionTable,
+    .optionCount = descriptor->optionCount,
+    .errorCount = 0
+  };
 
-  info.optionTable = optionTable;
-  info.optionCount = optionCount;
-  for (index=0; index<0X100; ++index) info.ensuredSettings[index] = 0;
-  info.errorCount = 0;
-
-  programPath = **argumentVector;
-  programName = strrchr(programPath, '/');
-  programName = programName? programName+1: programPath;
-
-  processCommandLine(&info, argumentCount, argumentVector, argumentsSummary);
   {
-    int configurationFileSpecified = configurationFile && *configurationFile;
+    int index;
+    for (index=0; index<0X100; ++index) info.ensuredSettings[index] = 0;
+  }
 
-    if (doBootParameters && *doBootParameters)
-      processBootParameters(&info, applicationName);
+  prepareProgram(*argumentCount, *argumentVector);
+  processCommandLine(&info, argumentCount, argumentVector, descriptor->argumentsSummary);
 
-    if (doEnvironmentVariables && *doEnvironmentVariables)
-      processEnvironmentVariables(&info, applicationName);
+  {
+    int configurationFileSpecified = descriptor->configurationFile && *descriptor->configurationFile;
+
+    if (descriptor->doBootParameters && *descriptor->doBootParameters)
+      processBootParameters(&info, descriptor->applicationName);
+
+    if (descriptor->doEnvironmentVariables && *descriptor->doEnvironmentVariables)
+      processEnvironmentVariables(&info, descriptor->applicationName);
 
     setDefaultOptions(&info, 0);
-    if (configurationFile && *configurationFile)
-      processConfigurationFile(&info, *configurationFile, !configurationFileSpecified);
+    if (descriptor->configurationFile && *descriptor->configurationFile) {
+      char *configurationFile = *descriptor->configurationFile;
+      fixInstallPath(&configurationFile);
+      processConfigurationFile(&info, configurationFile, !configurationFileSpecified);
+    }
     setDefaultOptions(&info, 1);
   }
 
-  return info.errorCount == 0;
-}
-
-short
-integerArgument (
-  const char *argument,
-  short minimum,
-  short maximum,
-  const char *name
-) {
-  char *end;
-  long int value = strtol(argument, &end, 0);
-  if ((end == argument) || *end) {
-    fprintf(stderr, "%s: Invalid %s: %s\n", programName, name, argument);
-  } else if (value < minimum) {
-    fprintf(stderr, "%s: %s is less than %d: %ld\n", programName, name, minimum, value);
-  } else if (value > maximum) {
-    fprintf(stderr, "%s: %s is greater than %d: %ld\n", programName, name, maximum, value);
-  } else {
-    return value;
-  }
-  exit(2);
-}
-
-unsigned int
-wordArgument (
-  const char *argument,
-  const char *const *choices,
-  const char *name
-) {
-  size_t length = strlen(argument);
-  const char *const *choice = choices;
-  while (*choice) {
-    if (strncasecmp(argument, *choice, length) == 0) return choice - choices;
-    ++choice;
-  }
-  fprintf(stderr, "%s: Invalid %s: %s\n", programName, name, argument);
-  exit(2);
+  return info.errorCount;
 }
