@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -17,9 +17,6 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
-
 #include "misc.h"
 #include "queue.h"
 
@@ -116,19 +113,59 @@ deleteElement (Element *element) {
   discardElement(element);
 }
 
+typedef struct {
+  Queue *queue;
+  void *item;
+} FindReferenceElementData;
+
+static int
+findReferenceElement (void *item, void *data) {
+  FindReferenceElementData *fre = data;
+  return fre->queue->compare(fre->item, item, fre->queue->data);
+}
+
+static void
+enqueueElement (Element *element) {
+  Queue *queue = element->queue;
+
+  if (queue->head) {
+    Element *reference;
+    int newHead = 0;
+
+    if (queue->compare) {
+      FindReferenceElementData fre;
+      fre.queue = queue;
+      fre.item = element->item;
+
+      if (!(reference = processQueue(queue, findReferenceElement, &fre))) {
+        reference = queue->head;
+      } else if (reference == queue->head) {
+        newHead = 1;
+      }
+    } else {
+      reference = queue->head;
+    }
+
+    linkAdditionalElement(reference, element);
+    if (newHead) queue->head = element;
+  } else {
+    linkFirstElement(element);
+  }
+}
+
 Element *
 enqueueItem (Queue *queue, void *item) {
   Element *element = newElement(queue, item);
-  if (element) {
-    if (queue->head) {
-      linkAdditionalElement(queue->head, element);
-    } else {
-      linkFirstElement(element);
-    }
-  }
+  if (element) enqueueElement(element);
   return element;
 }
 
+void
+requeueElement (Element *element) {
+  unlinkElement(element);
+  enqueueElement(element);
+}
+
 void *
 dequeueItem (Queue *queue) {
   void *item;
@@ -142,6 +179,16 @@ dequeueItem (Queue *queue) {
   return item;
 }
 
+Queue *
+getElementQueue (Element *element) {
+  return element->queue;
+}
+
+void *
+getElementItem (Element *element) {
+  return element->item;
+}
+
 Queue *
 newQueue (ItemDeallocator deallocate, ItemComparator compare) {
   Queue *queue;
@@ -167,6 +214,11 @@ deallocateQueue (Queue *queue) {
   free(queue);
 }
 
+Element *
+getQueueHead (Queue *queue) {
+  return queue->head;
+}
+
 int
 getQueueSize (Queue *queue) {
   return queue->size;
