@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,17 +22,19 @@
 extern "C" {
 #endif /* __cplusplus */
 
-extern int startCursorRouting (int column, int row, int screen);
-extern volatile pid_t routingProcess;
-extern volatile int routingStatus;
-
 typedef enum {
-  ROUTE_OK,
+  ROUTE_NONE,
+  ROUTE_DONE,
   ROUTE_WRONG_COLUMN,
   ROUTE_WRONG_ROW,
   ROUTE_ERROR
 } RoutingStatus;
 
+extern int startCursorRouting (int column, int row, int screen);
+
+extern volatile pid_t routingProcess;
+extern volatile RoutingStatus routingStatus;
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
