@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -24,18 +24,20 @@ extern "C" {
 
 typedef struct {
   void (*describe) (ScreenDescription *);
-  int (*read) (ScreenBox, unsigned char *, ScreenMode);
-  int (*insert) (ScreenKey);
-  int (*route) (int, int, int);
-  int (*point) (int, int);
-  int (*pointer) (int *, int *);
-  int (*selectvt) (int);
-  int (*switchvt) (int);
-  int (*currentvt) (void);
-  int (*execute) (int);
+  int (*read) (ScreenBox box, unsigned char *buffer, ScreenCharacterProperty property);
+  int (*insertKey) (ScreenKey key);
+  int (*routeCursor) (int column, int row, int screen);
+  int (*highlightRegion) (int left, int right, int top, int bottom);
+  int (*unhighlightRegion) (void);
+  int (*getPointer) (int *column, int *row);
+  int (*selectVirtualTerminal) (int vt);
+  int (*switchVirtualTerminal) (int vt);
+  int (*currentVirtualTerminal) (void);
+  int (*executeCommand) (int command);
 } BaseScreen;
 
 extern void initializeBaseScreen (BaseScreen *);
+extern void describeBaseScreen (BaseScreen *, ScreenDescription *);
 
 #ifdef __cplusplus
 }
