@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _SCR_HELP_H
-#define _SCR_HELP_H
+#ifndef BRLTTY_INCLUDED_SCR_HELP
+#define BRLTTY_INCLUDED_SCR_HELP
 
 #ifdef __cplusplus
 extern "C" {
@@ -27,8 +27,8 @@ extern "C" {
 
 typedef struct {
   BaseScreen base;
-  int (*open) (const char *);
-  void (*close) (void);		   /* called once to close the help screen */
+  int (*construct) (const char *);
+  void (*destruct) (void);		   /* called once to close the help screen */
   void (*setPageNumber) (short);
   short (*getPageNumber) (void);
   short (*getPageCount) (void);
@@ -40,4 +40,4 @@ extern void initializeHelpScreen (HelpScreen *);
 }
 #endif /* __cplusplus */
 
-#endif /* _SCR_HELP_H */
+#endif /* BRLTTY_INCLUDED_SCR_HELP */
