@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -17,8 +17,6 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 
 #include "misc.h"
@@ -26,35 +24,29 @@
 #include "scr_main.h"
 
 static int
-prepare_MainScreen (char **parameters) {
+processParameters_MainScreen (char **parameters) {
   return 1;
 }
 
 static int
-open_MainScreen (void) {
-  return 1;
-}
-
-static int
-setup_MainScreen (void) {
+construct_MainScreen (void) {
   return 1;
 }
 
 static void
-close_MainScreen (void) {
+destruct_MainScreen (void) {
 }
 
 static int
-uservt_MainScreen (int number) {
+userVirtualTerminal_MainScreen (int number) {
   return 0 + number;
 }
 
 void
 initializeMainScreen (MainScreen *main) {
   initializeBaseScreen(&main->base);
-  main->prepare = prepare_MainScreen;
-  main->open = open_MainScreen;
-  main->setup = setup_MainScreen;
-  main->close = close_MainScreen;
-  main->uservt = uservt_MainScreen;
+  main->processParameters = processParameters_MainScreen;
+  main->construct = construct_MainScreen;
+  main->destruct = destruct_MainScreen;
+  main->userVirtualTerminal = userVirtualTerminal_MainScreen;
 }
