@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,29 +15,35 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _SCR_MAIN_H
-#define _SCR_MAIN_H
+#ifndef BRLTTY_INCLUDED_SCR_MAIN
+#define BRLTTY_INCLUDED_SCR_MAIN
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
 #include "scr_base.h"
+#include "driver.h"
 
 typedef struct {
   BaseScreen base;
-  const char *const * (*parameters) (void);
-  int (*prepare) (char **parameters);
-  int (*open) (void);
-  int (*setup) (void);
-  void (*close) (void);
+  int (*processParameters) (char **parameters);
+  int (*construct) (void);
+  void (*destruct) (void);
+  int (*userVirtualTerminal) (int number);
 } MainScreen;
 
 extern void initializeMainScreen (MainScreen *);
-extern void initializeLiveScreen (MainScreen *);
+
+struct ScreenDriverStruct {
+  DRIVER_DEFINITION_DECLARATION;
+  const char *const *parameters;
+
+  void (*initialize) (MainScreen *main);		/* initialize speech device */
+};
 
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
-#endif /* _SCR_MAIN_H */
+#endif /* BRLTTY_INCLUDED_SCR_MAIN */
