@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -20,13 +20,12 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
 #include <errno.h>
 
+#include "program.h"
 #include "options.h"
 #include "spk.h"
 #include "misc.h"
@@ -38,30 +37,52 @@ static char *opt_speechVolume;
 static char *opt_libraryDirectory;
 static char *opt_dataDirectory;
 
-BEGIN_OPTION_TABLE
-  {"pcm-device", "device", 'p', 0, 0,
-   &opt_pcmDevice, NULL,
-   "Device specifier for soundcard digital audio."},
+BEGIN_OPTION_TABLE(programOptions)
+  { .letter = 'D',
+    .word = "data-directory",
+    .flags = OPT_Hidden,
+    .argument = "directory",
+    .setting.string = &opt_dataDirectory,
+    .defaultSetting = DATA_DIRECTORY,
+    .description = "Path to directory for configuration files."
+  },
 
-  {"rate", "speed", 'r', 0, 0,
-   &opt_speechRate, NULL,
-   "Floating-point speech rate multiplier."},
+  { .letter = 'L',
+    .word = "library-directory",
+    .flags = OPT_Hidden,
+    .argument = "directory",
+    .setting.string = &opt_libraryDirectory,
+    .defaultSetting = LIBRARY_DIRECTORY,
+    .description = "Path to directory for loading drivers."
+  },
 
-  {"text-string", "string", 't', 0, 0,
-   &opt_textString, NULL,
-   "Text to be spoken."},
+  { .letter = 't',
+    .word = "text-string",
+    .argument = "string",
+    .setting.string = &opt_textString,
+    .description = "Text to be spoken."
+  },
 
-  {"volume", "loudness", 'v', 0, 0,
-   &opt_speechVolume, NULL,
-   "Floating-point speech volume multiplier."},
+  { .letter = 'v',
+    .word = "volume",
+    .argument = "loudness",
+    .setting.string = &opt_speechVolume,
+    .description = "Floating-point speech volume multiplier."
+  },
 
-  {"data-directory", "directory", 'D', 0, 0,
-   &opt_dataDirectory, DATA_DIRECTORY,
-   "Path to directory for configuration files."},
+  { .letter = 'r',
+    .word = "rate",
+    .argument = "speed",
+    .setting.string = &opt_speechRate,
+    .description = "Floating-point speech rate multiplier."
+  },
 
-  {"library-directory", "directory", 'L', 0, 0,
-   &opt_libraryDirectory, LIBRARY_DIRECTORY,
-   "Path to directory for loading drivers."},
+  { .letter = 'd',
+    .word = "device",
+    .argument = "device",
+    .setting.string = &opt_pcmDevice,
+    .description = "Digital audio soundcard device specifier."
+  },
 END_OPTION_TABLE
 
 static int
@@ -78,21 +99,42 @@ main (int argc, char *argv[]) {
   float speechRate;
   float speechVolume;
 
-  processOptions(optionTable, optionCount,
-                 "spktest", &argc, &argv,
-                 NULL, NULL, NULL,
-                 "[driver [parameter=value ...]]");
-
-  if (opt_speechRate && *opt_speechRate) {
-    speechRate = floatArgument(opt_speechRate, 0.1, 10.0, "multiplier");
-  } else {
-    speechRate = 1.0;
+  {
+    static const OptionsDescriptor descriptor = {
+      OPTION_TABLE(programOptions),
+      .applicationName = "spktest",
+      .argumentsSummary = "[driver [parameter=value ...]]"
+    };
+    processOptions(&descriptor, &argc, &argv);
   }
 
+  {
+    char **const paths[] = {
+      &opt_libraryDirectory,
+      &opt_dataDirectory,
+      NULL
+    };
+    fixInstallPaths(paths);
+  }
+
+  speechRate = 1.0;
+  if (opt_speechRate && *opt_speechRate) {
+    static const float minimum = 0.1;
+    static const float maximum = 10.0;
+    if (!validateFloat(&speechRate, opt_speechRate, &minimum, &maximum)) {
+      LogPrint(LOG_ERR, "%s: %s", "invalid rate multiplier", opt_speechRate);
+      exit(2);
+    }
+  }
+
+  speechVolume = 1.0;
   if (opt_speechVolume && *opt_speechVolume) {
-    speechVolume = floatArgument(opt_speechVolume, 0.0, 200.0, "multiplier");
-  } else {
-    speechVolume = 1.0;
+    static const float minimum = 0.0;
+    static const float maximum = 2.0;
+    if (!validateFloat(&speechVolume, opt_speechVolume, &minimum, &maximum)) {
+      LogPrint(LOG_ERR, "%s: %s", "invalid volume multiplier", opt_speechVolume);
+      exit(2);
+    }
   }
 
   if (argc) {
@@ -147,8 +189,8 @@ main (int argc, char *argv[]) {
     }
 
     if (chdir(opt_dataDirectory) != -1) {
-      speech->identify();		/* start-up messages */
-      if (speech->open(parameterSettings)) {
+      identifySpeechDriver(speech, 0);		/* start-up messages */
+      if (speech->construct(parameterSettings)) {
         if (speech->rate) speech->rate(speechRate);
         if (speech->volume) speech->volume(speechVolume);
 
@@ -157,7 +199,7 @@ main (int argc, char *argv[]) {
         } else {
           processLines(stdin, sayLine, NULL);
         }
-        speech->close();		/* finish with the display */
+        speech->destruct();		/* finish with the display */
         status = 0;
       } else {
         LogPrint(LOG_ERR, "can't initialize speech driver.");
