@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -17,14 +17,12 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
 #include <errno.h>
 
-#ifdef HAVE_FUNC_SHL_LOAD
+#ifdef HAVE_SHL_LOAD
 #  include <dl.h>
-#endif /* HAVE_FUNC_SHL_LOAD */
+#endif /* HAVE_SHL_LOAD */
 
 #ifdef HAVE_HPUX_AUDIO
 #  include <Alib.h>
@@ -39,36 +37,42 @@
 
 #include "sys_boot_none.h"
 
+#include "sys_exec_unix.h"
+
+#include "sys_mount_none.h"
+
+#ifdef ENABLE_SHARED_OBJECTS
 void *
 loadSharedObject (const char *path) {
-#ifdef HAVE_FUNC_SHL_LOAD
+#ifdef HAVE_SHL_LOAD
   shl_t object = shl_load(path, BIND_IMMEDIATE|BIND_VERBOSE|DYNAMIC_PATH, 0L);
   if (object) return object;
   LogPrint(LOG_ERR, "Shared library '%s' not loaded: %s",
            path, strerror(errno));
-#endif /* HAVE_HPUX_AUDIO */
+#endif /* HAVE_SHL_LOAD */
   return NULL;
 }
 
 void 
 unloadSharedObject (void *object) {
-#ifdef HAVE_FUNC_SHL_LOAD
+#ifdef HAVE_SHL_LOAD
   if (shl_unload(object) == -1)
     LogPrint(LOG_ERR, "Shared library unload error: %s",
              strerror(errno));
-#endif /* HAVE_HPUX_AUDIO */
+#endif /* HAVE_SHL_LOAD */
 }
 
 int 
 findSharedSymbol (void *object, const char *symbol, const void **address) {
-#ifdef HAVE_FUNC_SHL_LOAD
+#ifdef HAVE_SHL_LOAD
   shl_t handle = object;
   if (shl_findsym(&handle, symbol, TYPE_UNDEFINED, address) != -1) return 1;
   LogPrint(LOG_ERR, "Shared symbol '%s' not found: %s",
            symbol, strerror(errno));
-#endif /* HAVE_HPUX_AUDIO */
+#endif /* HAVE_SHL_LOAD */
   return 0;
 }
+#endif /* ENABLE_SHARED_OBJECTS */
 
 #include "sys_beep_none.h"
 
