@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -254,7 +254,7 @@ openMidiDevice (int errorLevel, const char *device) {
             int port;
             int deviceOk;
 
-            if (device) {
+            if (*device) {
               deviceOk = parseMidiDevice(midi, errorLevel, device, &client, &port);
             } else {
               deviceOk = findMidiDevice(midi, errorLevel, &client, &port);
