@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,6 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
+#include <sys/soundcard.h>
+
 #include "io_misc.h"
 
 struct MidiDeviceStruct {
@@ -47,7 +49,7 @@ MidiDevice *
 openMidiDevice (int errorLevel, const char *device) {
   MidiDevice *midi;
   if ((midi = malloc(sizeof(*midi)))) {
-    if (!device) device = MIDI_OSS_DEVICE_PATH;
+    if (!*device) device = MIDI_OSS_DEVICE_PATH;
     if ((midi->fileDescriptor = open(device, O_WRONLY)) != -1) {
       {
         int count;
