@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -17,9 +17,6 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
-
 #include "misc.h"
 #include "system.h"
 
@@ -27,7 +24,13 @@
 
 #include "sys_boot_none.h"
 
+#include "sys_exec_none.h"
+
+#include "sys_mount_none.h"
+
+#ifdef ENABLE_SHARED_OBJECTS
 #include "sys_shlib_none.h"
+#endif /* ENABLE_SHARED_OBJECTS */
 
 #include "sys_beep_none.h"
 
