@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -21,8 +21,6 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 
 #include "options.h"
@@ -30,7 +28,7 @@
 #include "brl.h"
 #include "tbl.h"
 
-BEGIN_OPTION_TABLE
+BEGIN_OPTION_TABLE(programOptions)
 END_OPTION_TABLE
 
 int
@@ -39,10 +37,14 @@ main (int argc, char *argv[]) {
   char *path;
   TranslationTable table;
 
-  processOptions(optionTable, optionCount,
-                 "tbl2hex", &argc, &argv,
-                 NULL, NULL, NULL,
-                 "translation-table");
+  {
+    static const OptionsDescriptor descriptor = {
+      OPTION_TABLE(programOptions),
+      .applicationName = "tbl2hex",
+      .argumentsSummary = "translation-table"
+    };
+    processOptions(&descriptor, &argc, &argv);
+  }
 
   if (argc == 0) {
     LogPrint(LOG_ERR, "missing translation table.");
