@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -20,8 +20,6 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
@@ -49,46 +47,61 @@ char *opt_midiDevice;
 static char *opt_midiInstrument;
 #endif /* ENABLE_MIDI_SUPPORT */
 
-BEGIN_OPTION_TABLE
-  {"device", "device", 'd', 0, 0,
-   &opt_tuneDevice, NULL,
-   "Name of tune device."},
+BEGIN_OPTION_TABLE(programOptions)
+  { .letter = 'v',
+    .word = "volume",
+    .argument = "loudness",
+    .setting.string = &opt_outputVolume,
+    .description = "Output volume (percentage)."
+  },
 
-#ifdef ENABLE_MIDI_SUPPORT
-  {"instrument", "instrument", 'i', 0, 0,
-   &opt_midiInstrument, NULL,
-   "Name of MIDI instrument."},
-#endif /* ENABLE_MIDI_SUPPORT */
-
-#ifdef ENABLE_MIDI_SUPPORT
-  {"midi-device", "device", 'm', 0, 0,
-   &opt_midiDevice, NULL,
-   "Device specifier for the Musical Instrument Digital Interface."},
-#endif /* ENABLE_MIDI_SUPPORT */
+  { .letter = 'd',
+    .word = "device",
+    .argument = "device",
+    .setting.string = &opt_tuneDevice,
+    .description = "Name of tune device."
+  },
 
 #ifdef ENABLE_PCM_SUPPORT
-  {"pcm-device", "device", 'p', 0, 0,
-   &opt_pcmDevice, NULL,
-   "Device specifier for soundcard digital audio."},
+  { .letter = 'p',
+    .word = "pcm-device",
+    .flags = OPT_Hidden,
+    .argument = "device",
+    .setting.string = &opt_pcmDevice,
+    .description = "Device specifier for soundcard digital audio."
+  },
 #endif /* ENABLE_PCM_SUPPORT */
 
-  {"volume", "loudness", 'v', 0, 0,
-   &opt_outputVolume, NULL,
-   "Output volume (percentage)."},
+#ifdef ENABLE_MIDI_SUPPORT
+  { .letter = 'm',
+    .word = "midi-device",
+    .flags = OPT_Hidden,
+    .argument = "device",
+    .setting.string = &opt_midiDevice,
+    .description = "Device specifier for the Musical Instrument Digital Interface."
+  },
+
+  { .letter = 'i',
+    .word = "instrument",
+    .argument = "instrument",
+    .setting.string = &opt_midiInstrument,
+    .description = "Name of MIDI instrument."
+  },
+#endif /* ENABLE_MIDI_SUPPORT */
 END_OPTION_TABLE
 
 static const char *deviceNames[] = {"beeper", "pcm", "midi", "fm", NULL};
 
 #ifdef ENABLE_MIDI_SUPPORT
-static unsigned char
-instrumentArgument (const char *argument) {
-  size_t argumentLength = strlen(argument);
+static int
+validateInstrument (unsigned char *value, const char *string) {
+  size_t stringLength = strlen(string);
   unsigned char instrument;
   for (instrument=0; instrument<midiInstrumentCount; ++instrument) {
     const char *component = midiInstrumentTable[instrument];
     size_t componentLeft = strlen(component);
-    const char *word = argument;
-    size_t wordLeft = argumentLength;
+    const char *word = string;
+    size_t wordLeft = stringLength;
     {
       const char *delimiter = memchr(component, '(', componentLeft);
       if (delimiter) componentLeft = delimiter - component;
@@ -96,7 +109,10 @@ instrumentArgument (const char *argument) {
     while (1) {
       while (*component == ' ') component++, componentLeft--;
       if ((componentLeft == 0) != (wordLeft == 0)) break; 
-      if (!componentLeft) return instrument;
+      if (!componentLeft) {
+        *value = instrument;
+        return 1;
+      }
       {
         size_t wordLength = wordLeft;
         size_t componentLength = componentLeft;
@@ -110,8 +126,7 @@ instrumentArgument (const char *argument) {
       }
     }
   }
-  LogPrint(LOG_ERR, "invalid instrument: %s", argument);
-  exit(2);
+  return 0;
 }
 #endif /* ENABLE_MIDI_SUPPORT */
 
@@ -124,46 +139,95 @@ main (int argc, char *argv[]) {
   unsigned char midiInstrument;
 #endif /* ENABLE_MIDI_SUPPORT */
 
-  processOptions(optionTable, optionCount,
-                 "tunetest", &argc, &argv,
-                 NULL, NULL, NULL,
-                 "{note duration} ...");
+  {
+    static const OptionsDescriptor descriptor = {
+      OPTION_TABLE(programOptions),
+      .applicationName = "tunetest",
+      .argumentsSummary = "{note duration} ..."
+    };
+    processOptions(&descriptor, &argc, &argv);
+  }
 
   if (opt_tuneDevice && *opt_tuneDevice) {
-    tuneDevice = wordArgument(opt_tuneDevice, deviceNames, "device");
+    unsigned int device;
+    if (!validateChoice(&device, opt_tuneDevice, deviceNames)) {
+      LogPrint(LOG_ERR, "%s: %s", "invalid tune device", opt_tuneDevice);
+      exit(2);
+    }
+    tuneDevice = device;
   } else {
     tuneDevice = getDefaultTuneDevice();
   }
 
 #ifdef ENABLE_MIDI_SUPPORT
   if (opt_midiInstrument && *opt_midiInstrument) {
-    midiInstrument = instrumentArgument(opt_midiInstrument);
+    if (!validateInstrument(&midiInstrument, opt_midiInstrument)) {
+      LogPrint(LOG_ERR, "%s: %s", "invalid musical instrument", opt_midiInstrument);
+      exit(2);
+    }
   } else {
     midiInstrument = 0;
   }
 #endif /* ENABLE_MIDI_SUPPORT */
 
   if (opt_outputVolume && *opt_outputVolume) {
-    outputVolume = integerArgument(opt_outputVolume, 0, 100, "level");
+    static const int minimum = 0;
+    static const int maximum = 100;
+    int volume;
+    if (!validateInteger(&volume, opt_outputVolume, &minimum, &maximum)) {
+      LogPrint(LOG_ERR, "%s: %s", "invalid volume percentage", opt_outputVolume);
+      exit(2);
+    }
+    outputVolume = volume;
   } else {
     outputVolume = 50;
   }
 
   if (!argc) {
     LogPrint(LOG_ERR, "missing tune.");
-  } else if (argc % 2) {
-    LogPrint(LOG_ERR, "missing duration.");
-  } else {
+    exit(2);
+  }
+
+  if (argc % 2) {
+    LogPrint(LOG_ERR, "missing note duration.");
+    exit(2);
+  }
+
+  {
     unsigned int count = argc / 2;
     TuneElement *elements = mallocWrapper((sizeof(*elements) * count) + 1);
     TuneElement *element = elements;
 
     while (argc) {
-      short note = integerArgument(*argv++, 1, 127, "note");
-      short duration = integerArgument(*argv++, 1, 255, "duration");
-      TuneElement te = TUNE_NOTE(duration, note);
-      argc -= 2;
-      *(element++) = te;
+      int note;
+      int duration;
+
+      {
+        static const int minimum = 0X01;
+        static const int maximum = 0X7F;
+        const char *argument = *argv++;
+        if (!validateInteger(&note, argument, &minimum, &maximum)) {
+          LogPrint(LOG_ERR, "%s: %s", "invalid note number", argument);
+          exit(2);
+        }
+        --argc;
+      }
+
+      {
+        static const int minimum = 1;
+        static const int maximum = 255;
+        const char *argument = *argv++;
+        if (!validateInteger(&duration, argument, &minimum, &maximum)) {
+          LogPrint(LOG_ERR, "%s: %s", "invalid note duration", argument);
+          exit(2);
+        }
+        --argc;
+      }
+
+      {
+        TuneElement te = TUNE_NOTE(duration, note);
+        *(element++) = te;
+      }
     }
 
     {
@@ -203,9 +267,8 @@ main (int argc, char *argv[]) {
     }
 
     free(elements);
-    return 0;
   }
-  return 2;
+  return 0;
 }
 
 void
