@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -17,8 +17,6 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -393,7 +391,7 @@ usbDeallocateEndpoint (void *item, void *data) {
   }
 
   if (endpoint->extension) {
-    usbDeallocateEndpointExtension(endpoint);
+    usbDeallocateEndpointExtension(endpoint->extension);
     endpoint->extension = NULL;
   }
 
@@ -456,7 +454,7 @@ usbGetEndpoint (UsbDevice *device, unsigned char endpointAddress) {
       endpoint->extension = NULL;
       if (usbAllocateEndpointExtension(endpoint)) {
         if (enqueueItem(device->endpoints, endpoint)) return endpoint;
-        usbDeallocateEndpointExtension(endpoint);
+        usbDeallocateEndpointExtension(endpoint->extension);
       }
 
       free(endpoint);
@@ -564,7 +562,7 @@ usbCloseDevice (UsbDevice *device) {
   }
 
   if (device->extension) {
-    usbDeallocateDeviceExtension(device);
+    usbDeallocateDeviceExtension(device->extension);
     device->extension = NULL;
   }
 
@@ -573,11 +571,12 @@ usbCloseDevice (UsbDevice *device) {
 }
 
 static UsbDevice *
-usbOpenDevice (void *extension) {
+usbOpenDevice (UsbDeviceExtension *extension) {
   UsbDevice *device;
   if ((device = malloc(sizeof(*device)))) {
     memset(device, 0, sizeof(*device));
     device->extension = extension;
+    device->serial = NULL;
 
     if ((device->endpoints = newQueue(usbDeallocateEndpoint, NULL))) {
       if ((device->inputFilters = newQueue(usbDeallocateInputFilter, NULL))) {
@@ -597,7 +596,7 @@ usbOpenDevice (void *extension) {
 }
 
 UsbDevice *
-usbTestDevice (void *extension, UsbDeviceChooser chooser, void *data) {
+usbTestDevice (UsbDeviceExtension *extension, UsbDeviceChooser chooser, void *data) {
   UsbDevice *device;
   if ((device = usbOpenDevice(extension))) {
     LogPrint(LOG_DEBUG, "USB: testing: vendor=%04X product=%04X",
@@ -887,7 +886,8 @@ static int
 usbFtdiInputFilter (UsbInputFilterData *data) {
   const int count = 2;
   if (data->length > count) {
-    memmove(data->buffer, data->buffer+count, data->length-=count);
+    unsigned char *buffer = data->buffer;
+    memmove(buffer, buffer+count, data->length-=count);
   } else {
     data->length = 0;
   }
@@ -1051,76 +1051,96 @@ static const UsbSerialOperations usbFtdiOperations_FT232BM = {
 
 const UsbSerialOperations *
 usbGetSerialOperations (UsbDevice *device) {
-  typedef struct {
-    uint16_t vendor;
-    uint16_t product;
-    const UsbSerialOperations *operations;
-    UsbInputFilter inputFilter;
-  } UsbSerialAdapter;
-  static const UsbSerialAdapter usbSerialAdapters[] = {
-    { /* HandyTech GoHubs */
-      0X0921, 0X1200,
-      &usbBelkinOperations,
-      NULL
-    }
-    ,
-    { /* HandyTech FTDI */
-      0X0403, 0X6001,
-      &usbFtdiOperations_FT8U232AM,
-      usbFtdiInputFilter
-    }
-    ,
-    { /* Papenmeier FTDI */
-      0X0403, 0XF208,
-      &usbFtdiOperations_FT232BM,
-      usbFtdiInputFilter
-    }
-    ,
-    { /* Baum FTDI: 24 cells  */
-      0X0403, 0XFE71,
-      &usbFtdiOperations_FT232BM,
-      usbFtdiInputFilter
-    }
-    ,
-    { /* Baum FTDI: 40 cells  */
-      0X0403, 0XFE72,
-      &usbFtdiOperations_FT232BM,
-      usbFtdiInputFilter
-    }
-    ,
-    { /* Baum FTDI: 32 cells  */
-      0X0403, 0XFE73,
-      &usbFtdiOperations_FT232BM,
-      usbFtdiInputFilter
-    }
-    ,
-    { /* Baum FTDI: 64 cells  */
-      0X0403, 0XFE74,
-      &usbFtdiOperations_FT232BM,
-      usbFtdiInputFilter
-    }
-    ,
-    { /* Baum FTDI: 80 cells  */
-      0X0403, 0XFE75,
-      &usbFtdiOperations_FT232BM,
-      usbFtdiInputFilter
-    }
-    ,
-    {}
-  };
-  const UsbSerialAdapter *sa = usbSerialAdapters;
-  while (sa->vendor) {
-    if (sa->vendor == device->descriptor.idVendor) {
-      if (!sa->product || (sa->product == device->descriptor.idProduct)) {
-        if (!sa->inputFilter || usbAddInputFilter(device, sa->inputFilter)) {
-          return sa->operations;
-        }
-        break;
+  if (!device->serial) {
+    typedef struct {
+      uint16_t vendor;
+      uint16_t product;
+      const UsbSerialOperations *operations;
+      UsbInputFilter inputFilter;
+    } UsbSerialAdapter;
+
+    static const UsbSerialAdapter usbSerialAdapters[] = {
+      { /* HandyTech GoHubs */
+        0X0921, 0X1200,
+        &usbBelkinOperations,
+        NULL
       }
+      ,
+      { /* HandyTech FTDI */
+        0X0403, 0X6001,
+        &usbFtdiOperations_FT8U232AM,
+        usbFtdiInputFilter
+      }
+      ,
+      { /* Papenmeier FTDI */
+        0X0403, 0XF208,
+        &usbFtdiOperations_FT232BM,
+        usbFtdiInputFilter
+      }
+      ,
+      { /* Baum FTDI: 24 cells  */
+        0X0403, 0XFE71,
+        &usbFtdiOperations_FT232BM,
+        usbFtdiInputFilter
+      }
+      ,
+      { /* Baum FTDI: 40 cells  */
+        0X0403, 0XFE72,
+        &usbFtdiOperations_FT232BM,
+        usbFtdiInputFilter
+      }
+      ,
+      { /* Baum FTDI: 32 cells  */
+        0X0403, 0XFE73,
+        &usbFtdiOperations_FT232BM,
+        usbFtdiInputFilter
+      }
+      ,
+      { /* Baum FTDI: 64 cells  */
+        0X0403, 0XFE74,
+        &usbFtdiOperations_FT232BM,
+        usbFtdiInputFilter
+      }
+      ,
+      { /* Baum FTDI: 80 cells  */
+        0X0403, 0XFE75,
+        &usbFtdiOperations_FT232BM,
+        usbFtdiInputFilter
+      }
+      ,
+      {0, 0}
+    };
+    const UsbSerialAdapter *sa = usbSerialAdapters;
+
+    while (sa->vendor) {
+      if (sa->vendor == device->descriptor.idVendor) {
+        if (!sa->product || (sa->product == device->descriptor.idProduct)) {
+          if (sa->inputFilter && !usbAddInputFilter(device, sa->inputFilter)) goto error;
+          device->serial = sa->operations;
+          break;
+        }
+      }
+
+      ++sa;
     }
-    ++sa;
+
+    if (!device->serial)
+      LogPrint(LOG_WARNING, "USB: unsupported serial adapter: vendor=%04X product=%04X",
+               device->descriptor.idVendor, device->descriptor.idProduct);
   }
-  return NULL;
+
+error:
+  return device->serial;
+}
+
+int
+usbSetSerialParameters (UsbDevice *device, const SerialParameters *parameters) {
+  const UsbSerialOperations *serial = usbGetSerialOperations(device);
+  if (!serial) return 0;
+  if (!serial->setBaud(device, parameters->baud)) return 0;
+  if (!serial->setFlowControl(device, parameters->flow)) return 0;
+  if (!serial->setDataFormat(device, parameters->data, parameters->stop, parameters->parity)) return 0;
+  return 1;
 }
 
 typedef struct {
@@ -1138,22 +1158,15 @@ usbChooseChannel (UsbDevice *device, void *data) {
     if (USB_IS_PRODUCT(descriptor, definition->vendor, definition->product)) {
       if (!usbVerifySerialNumber(device, choose->serialNumber)) break;
 
+      if (definition->disableAutosuspend) usbDisableAutosuspend(device);
+
       if (usbConfigureDevice(device, definition->configuration)) {
         if (usbOpenInterface(device, definition->interface, definition->alternative)) {
-          const UsbSerialOperations *serial = usbGetSerialOperations(device);
           int ok = 1;
 
-          if (definition->baud) {
-            if (serial) {
-              if (!serial->setBaud(device, definition->baud)) ok = 0;
-              if (!serial->setFlowControl(device, definition->flowControl)) ok = 0;
-              if (!serial->setDataFormat(device, definition->dataBits, definition->stopBits, definition->parity)) ok = 0;
-            } else {
-              LogPrint(LOG_WARNING, "Unsupported serial adapter: vendor=%04X product=%04X",
-                       definition->vendor, definition->product);
+          if (definition->serial)
+            if (!usbSetSerialParameters(device, definition->serial))
               ok = 0;
-            }
-          }
 
           if (ok) {
             choose->definition = definition;
@@ -1185,7 +1198,7 @@ usbFindChannel (const UsbChannelDefinition *definitions, const char *device) {
       return channel;
     } else {
       LogPrint(LOG_DEBUG, "USB device not found%s%s",
-               (*device? ": ": "."),
+               (*device? ": ": ""),
                device);
     }
 
