@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -17,8 +17,6 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <errno.h>
 
@@ -33,6 +31,13 @@ usbResetDevice (UsbDevice *device) {
   return 0;
 }
 
+int
+usbDisableAutosuspend (UsbDevice *device) {
+  errno = ENOSYS;
+  LogError("USB device autosuspend disable");
+  return 0;
+}
+
 int
 usbSetConfiguration (
   UsbDevice *device,
@@ -186,11 +191,11 @@ usbAllocateEndpointExtension (UsbEndpoint *endpoint) {
 }
 
 void
-usbDeallocateEndpointExtension (UsbEndpoint *endpoint) {
+usbDeallocateEndpointExtension (UsbEndpointExtension *eptx) {
 }
 
 void
-usbDeallocateDeviceExtension (UsbDevice *device) {
+usbDeallocateDeviceExtension (UsbDeviceExtension *devx) {
 }
 
 UsbDevice *
