@@ -1,8 +1,7 @@
 /*
  * XBrlAPI - A background process tinkering with X for proper BrlAPI behavior
  *
- * Copyright (C) 2003-2005 by Samuel Thibault <Samuel.Thibault@ens-lyon.org>
- * All rights reserved.
+ * Copyright (C) 2003-2007 by Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  *
  * XBrlAPI comes with ABSOLUTELY NO WARRANTY.
  *
@@ -20,19 +19,12 @@
  */
 
 #include "prologue.h"
-#include <stdlib.h>
-#include <unistd.h>
+
 #include <stdio.h>
 #include <stdarg.h>
-#include <inttypes.h>
-#include <getopt.h>
 #include <signal.h>
 #include <string.h>
-#ifdef WINDOWS
-#include <ws2tcpip.h>
-#else /* WINDOWS */
-#include <netinet/in.h>
-#endif /* WINDOWS */
+
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #else /* HAVE_SYS_SELECT_H */
@@ -53,18 +45,50 @@
 #warning key press simulation not supported on this installation
 #endif /* HAVE_X11_EXTENSIONS_XTEST_H && HAVE_X11_EXTENSIONS_XKB_H */
 
-#include "api.h"
-#include "brldefs.h"
+#define BRLAPI_NO_DEPRECATED
+#include "brlapi.h"
+
+#include "misc.h"
+#include "options.h"
 
 //#define DEBUG
 #ifdef DEBUG
 #define debugf(fmt, ...) fprintf(stderr, fmt, ## __VA_ARGS__)
-#else
+#else /* DEBUG */
 #define debugf(fmt, ...) (void)0
-#endif
+#endif /* DEBUG */
+
+/******************************************************************************
+ * option handling
+ */
+
+static char *auth;
+static char *host;
+static char *xDisplay;
+
+BEGIN_OPTION_TABLE(programOptions)
+  { .letter = 'b',
+    .word = "brlapi",
+    .argument = strtext("[host][:port]"),
+    .setting.string = &host,
+    .description = strtext("BrlAPI host and/or port to connect to")
+  },
+
+  { .letter = 'a',
+    .word = "auth",
+    .argument = strtext("file"),
+    .setting.string = &auth,
+    .description = strtext("BrlAPI authorization/authentication string")
+  },
+
+  { .letter = 'd',
+    .word = "display",
+    .argument = strtext("display"),
+    .setting.string = &xDisplay,
+    .description = strtext("X display to connect to")
+  },
+END_OPTION_TABLE
 
-#define _(s) (s)
-#define _n(s,sp,n) ((n)>1?(sp):(s))
 /******************************************************************************
  * error handling
  */
@@ -92,12 +116,13 @@ void fatal_errno(const char *msg, const char *fmt, ...) {
 }
 
 void fatal(const char *fmt, ...) {
-  va_list va;
-  va_start(va,fmt);
-  if (fmt)
+  if (fmt) {
+    va_list va;
+    va_start(va,fmt);
     vfprintf(stderr,fmt,va);
+    va_end(va);
+  }
   exit(1);
-  va_end(va);
 }
 
 /******************************************************************************
@@ -116,35 +141,49 @@ void api_cleanExit(int foo) {
   exit(0);
 }
 
-void tobrltty_init(char *authKey, char *hostName) {
-  brlapi_settings_t settings;
+void tobrltty_init(char *auth, char *host) {
+  brlapi_connectionSettings_t settings;
   unsigned int x,y;
-  settings.hostName=hostName;
-  settings.authKey=authKey;
+  settings.host=host;
+  settings.auth=auth;
 
   signal(SIGTERM,api_cleanExit);
-  signal(SIGHUP,api_cleanExit);
   signal(SIGINT,api_cleanExit);
+#ifdef SIGHUP
+  signal(SIGHUP,api_cleanExit);
+#endif /* SIGHUP */
+#ifdef SIGQUIT
   signal(SIGQUIT,api_cleanExit);
+#endif /* SIGQUIT */
+#ifdef SIGPIPE
   signal(SIGPIPE,api_cleanExit);
+#endif /* SIGPIPE */
 
-  if ((brlapi_fd = brlapi_initializeConnection(&settings,&settings))<0)
-    fatal_brlapi_errno("initializeConnection",_("couldn't connect to brltty at %s\n"),settings.hostName);
+  if ((brlapi_fd = brlapi_openConnection(&settings,&settings))<0)
+    fatal_brlapi_errno("openConnection",gettext("cannot connect to brltty at %s\n"),settings.host);
 
   if (brlapi_getDisplaySize(&x,&y)<0)
     fatal_brlapi_errno("getDisplaySize",NULL);
 }
 
-void vtno(int vtno) {
-  if (brlapi_getTty(vtno,NULL)<0)
-    fatal_brlapi_errno("getTty",_("getting tty %d\n"),vtno);
-  if (brlapi_ignoreKeyRange(0,BRL_KEYCODE_MAX)<0)
-    fatal_brlapi_errno("ignoreKeys",_("ignoring every key %d\n"),vtno);
+static int getXVTnb(void);
+
+void getVT(void) {
+  if (getenv("WINDOWPATH")) {
+    if (brlapi_enterTtyModeWithPath(NULL,0,NULL)<0)
+      fatal_brlapi_errno("geTtyPath",gettext("cannot get tty\n"));
+  } else {
+    int vtno = getXVTnb();
+    if (brlapi_enterTtyMode(vtno,NULL)<0)
+      fatal_brlapi_errno("enterTtyMode",gettext("cannot get tty %d\n"),vtno);
+  }
+  if (brlapi_ignoreAllKeys()<0)
+    fatal_brlapi_errno("ignoreAllKeys",gettext("cannot ignore keys\n"));
 #ifdef CAN_SIMULATE_KEY_PRESSES
-  if (brlapi_unignoreKeyRange(BRL_BLK_PASSCHAR,BRL_BLK_PASSCHAR|BRL_MSK_ARG))
-    fatal_brlapi_errno("unignoreKeyRange",NULL);
-  if (brlapi_unignoreKeyRange(BRL_BLK_PASSKEY, BRL_BLK_PASSKEY |BRL_MSK_ARG))
-    fatal_brlapi_errno("unignoreKeyRange",NULL);
+  /* All X keysyms with any modifier */
+  brlapi_keyCode_t cmd = BRLAPI_KEY_TYPE_SYM;
+  if (brlapi_acceptKeys(brlapi_rangeType_type, &cmd, 1))
+    fatal_brlapi_errno("acceptKeys",NULL);
 #endif /* CAN_SIMULATE_KEY_PRESSES */
 }
 
@@ -160,14 +199,14 @@ void api_setName(const char *wm_name) {
 
   if (brlapi_writeText(0,wm_name)<0) {
     brlapi_perror("writeText");
-    fprintf(stderr,_("setting name %s\n"),wm_name);
+    fprintf(stderr,gettext("cannot write window name %s\n"),wm_name);
   }
 }
 
 void api_setFocus(int win) {
   debugf("%#010x (%d) got focus\n",win,win);
   if (brlapi_setFocus(win)<0)
-    fatal_brlapi_errno("setFocus",_("setting focus to %#010x\n"),win);
+    fatal_brlapi_errno("setFocus",gettext("cannot set focus to %#010x\n"),win);
 }
 
 /******************************************************************************
@@ -231,12 +270,12 @@ static int ErrorHandler(Display *dpy, XErrorEvent *ev) {
   }
   if (!XGetErrorText(dpy, ev->error_code, buffer, sizeof(buffer)))
     fatal("XGetErrorText");
-  fprintf(stderr,_("X Error %d, %s on display %s\n"), ev->type, buffer, XDisplayName(Xdisplay));
-  fprintf(stderr,_("resource %#010lx, req %u:%u\n"),ev->resourceid,ev->request_code,ev->minor_code);
+  fprintf(stderr,gettext("X Error %d, %s on display %s\n"), ev->type, buffer, XDisplayName(Xdisplay));
+  fprintf(stderr,gettext("resource %#010lx, req %u:%u\n"),ev->resourceid,ev->request_code,ev->minor_code);
   exit(1);
 }
 
-static void getVTnb(void) {
+static int getXVTnb(void) {
   Window root;
   Atom property;
   Atom actual_type;
@@ -244,34 +283,36 @@ static void getVTnb(void) {
   unsigned long nitems;
   unsigned long bytes_after;
   unsigned char *buf;
+  int vt = -1;
 
   root=DefaultRootWindow(dpy);
 
   if ((property=XInternAtom(dpy,"XFree86_VT",False))==None)
-    fatal(_("no XFree86_VT atom\n"));
+    fatal(gettext("no XFree86_VT atom\n"));
   
   if (XGetWindowProperty(dpy,root,property,0,1,False,AnyPropertyType,
     &actual_type, &actual_format, &nitems, &bytes_after, &buf))
-    fatal(_("couldn't get root window XFree86_VT property\n"));
+    fatal(gettext("cannot get root window XFree86_VT property\n"));
 
   if (nitems<1)
-    fatal(_("no items for VT number\n"));
+    fatal(gettext("no items for VT number\n"));
   if (nitems>1)
-    fprintf(stderr,_("several items for VT number ?!\n"));
+    fprintf(stderr,gettext("more than one item for VT number\n"));
   switch (actual_type) {
   case XA_CARDINAL:
   case XA_INTEGER:
   case XA_WINDOW:
     switch (actual_format) {
-    case 8:  vtno(*(uint8_t *)buf); break;
-    case 16: vtno(*(uint16_t *)buf); break;
-    case 32: vtno(*(uint32_t *)buf); break;
-    default: fatal(_("Bad format for VT number\n"));
+    case 8:  vt = (*(uint8_t *)buf); break;
+    case 16: vt = (*(uint16_t *)buf); break;
+    case 32: vt = (*(uint32_t *)buf); break;
+    default: fatal(gettext("bad format for VT number\n"));
     }
     break;
-  default: fatal(_("Bad type for VT number\n"));
+  default: fatal(gettext("bad type for VT number\n"));
   }
   if (!XFree(buf)) fatal("XFree(VTnobuf)");
+  return vt;
 }
 
 static int grabWindow(Window win,int level) {
@@ -303,7 +344,9 @@ static char *getWindowTitle(Window win) {
       &actual_type,&actual_format,&nitems,&bytes_after,&wm_name))
       return NULL; /* window disappeared */
     wm_name_size+=bytes_after;
-  } while (bytes_after && ({if (!XFree(wm_name)) fatal("tempo_XFree(wm_name)"); 1;}));
+    if (!bytes_after) break;
+    if (!XFree(wm_name)) fatal("tempo_XFree(wm_name)");
+  } while (1);
   if (actual_type==None) return NULL;
   else {
     debugf("type %lx name %s",actual_type,wm_name);
@@ -348,11 +391,24 @@ static void setFocus(Window win) {
   api_setFocus((uint32_t)win);
 
   if (!(window=window_of_Window(win))) {
-    fprintf(stderr,_("didn't grab window %#010lx but got focus !\n"),win);
+    fprintf(stderr,gettext("didn't grab window %#010lx but got focus\n"),win);
     api_setName("unknown");
   } else setName(window);
 }
 
+#ifdef CAN_SIMULATE_KEY_PRESSES
+static int tryModifiers(KeyCode keycode, unsigned int *modifiers, unsigned int modifiers_try, KeySym keysym) {
+  KeySym keysymRet;
+  unsigned int modifiersRet;
+  if (!XkbLookupKeySym(dpy, keycode, *modifiers | modifiers_try, &modifiersRet, &keysymRet))
+    return 0;
+  if (keysymRet != keysym)
+    return 0;
+  *modifiers |= modifiers_try;
+  return 1;
+}
+#endif /* CAN_SIMULATE_KEY_PRESSES */
+
 void toX_f(const char *display) {
   Window root;
   XEvent ev;
@@ -362,7 +418,7 @@ void toX_f(const char *display) {
   int maxfd;
 #ifdef CAN_SIMULATE_KEY_PRESSES
   int res;
-  brl_keycode_t code;
+  brlapi_keyCode_t code;
   unsigned int keysym, keycode, modifiers;
   Bool haveXTest;
   int eventBase, errorBase, majorVersion, minorVersion;
@@ -370,32 +426,41 @@ void toX_f(const char *display) {
 
   Xdisplay = display;
   if (!Xdisplay) Xdisplay=getenv("DISPLAY");
-  if (!(dpy=XOpenDisplay(Xdisplay))) fatal(_("couldn't connect to %s\n"),Xdisplay);
+  if (!(dpy=XOpenDisplay(Xdisplay))) fatal(gettext("cannot connect to display %s\n"),Xdisplay);
 
-  if (!XSetErrorHandler(ErrorHandler)) fatal(_("strange old error handler\n"));
+  if (!XSetErrorHandler(ErrorHandler)) fatal(gettext("strange old error handler\n"));
 
 #ifdef CAN_SIMULATE_KEY_PRESSES
   haveXTest = XTestQueryExtension(dpy, &eventBase, &errorBase, &majorVersion, &minorVersion);
 #endif /* CAN_SIMULATE_KEY_PRESSES */
 
+  {
+    int foo;
+    int major = XkbMajorVersion, minor = XkbMinorVersion;
+    if (!XkbLibraryVersion(&major, &minor))
+      fatal(gettext("Incompatible XKB library\n"));
+    if (!XkbQueryExtension(dpy, &foo, &foo, &foo, &major, &minor))
+      fatal(gettext("Incompatible XKB server support\n"));
+  }
+
   X_fd = XConnectionNumber(dpy);
   FD_ZERO(&fds);
   FD_SET(brlapi_fd, &fds);
   FD_SET(X_fd, &fds);
   maxfd = X_fd>brlapi_fd ? X_fd+1 : brlapi_fd+1;
 
-  getVTnb();
+  getVT();
   
   for (i=0;i<ScreenCount(dpy);i++) {
     root=RootWindow(dpy,i);
-    if (!grabWindows(root,0)) fatal(_("Couldn't grab windows on screen %d\n"),i);
+    if (!grabWindows(root,0)) fatal(gettext("cannot grab windows on screen %d\n"),i);
   }
 
   {
     Window win;
     int revert_to;
     if (!XGetInputFocus(dpy,&win,&revert_to))
-      fatal(_("failed to get first Focus\n"));
+      fatal(gettext("failed to get first focus\n"));
     setFocus(win);
   }
   while(1) {
@@ -442,7 +507,7 @@ void toX_f(const char *display) {
 	if (!grabWindow(win,0)) break; /* window already disappeared ! */
 	debugf("win %#010lx created\n",win);
 	if (!(window = window_of_Window(ev.xcreatewindow.parent))) {
-	  fprintf(stderr,_("didn't grab parent of %#010lx ?!\n"),win);
+	  fprintf(stderr,gettext("didn't grab parent of %#010lx\n"),win);
 	  add_window(win,None,getWindowTitle(win));
 	} else add_window(win,window->root,getWindowTitle(win));
       } break;
@@ -459,18 +524,21 @@ void toX_f(const char *display) {
 	  debugf("WM_NAME property of %#010lx changed\n",win);
 	  struct window *window;
 	  if (!(window=window_of_Window(win))) {
-	    fprintf(stderr,_("didn't grab window %#010lx\n"),win);
+	    fprintf(stderr,gettext("didn't grab window %#010lx\n"),win);
 	    add_window(win,None,getWindowTitle(win));
 	  } else {
 	    if (window->wm_name)
-	      if (!XFree(window->wm_name)) fatal(_("XFree(wm_name) for change"));
+	      if (!XFree(window->wm_name)) fatal(gettext("XFree(wm_name) for change"));
 	    if ((window->wm_name=getWindowTitle(win))) {
 	      if (win==curWindow)
 		api_setName(window->wm_name);
-	    } else fprintf(stderr,_("uhu, window %#010lx changed to NULL name ?!\n"),win);
+	    } else fprintf(stderr,gettext("window %#010lx changed to NULL name\n"),win);
 	  }
 	}
 	break;
+      case MappingNotify:
+	XRefreshKeyboardMapping(&ev.xmapping);
+	break;
       /* ignored events */
       case UnmapNotify:
       case MapNotify:
@@ -484,90 +552,53 @@ void toX_f(const char *display) {
 	break;
 
       /* "shouldn't happen" events */
-      default: fprintf(stderr,_("unhandled %d type\n"),ev.type); break;
+      default: fprintf(stderr,gettext("unhandled event type: %d\n"),ev.type); break;
       }
     }
 #ifdef CAN_SIMULATE_KEY_PRESSES
     if (haveXTest && FD_ISSET(brlapi_fd,&readfds)) {
       while ((res = brlapi_readKey(0, &code)==1)) {
-	modifiers = 0;
-	if (code & BRL_FLG_CHAR_CONTROL) modifiers |= ControlMask;
-	if (code & BRL_FLG_CHAR_META) modifiers |= Mod1Mask;
-	if (code & BRL_FLG_CHAR_UPPER) modifiers |= ShiftMask;
-	if (code & BRL_FLG_CHAR_SHIFT) modifiers |= ShiftMask;
-
-	switch (code&BRL_MSK_BLK) {
-	  case BRL_BLK_PASSCHAR:
-	    if (((code&BRL_MSK_ARG) >= 'A') && ((code&BRL_MSK_ARG) <= 'Z')) {
-	      code = code + 'a' - 'A';
-	      modifiers |= ShiftMask;
-	    }
-	    keysym = code&BRL_MSK_ARG;
-	    break;
-
-	  case BRL_BLK_PASSKEY:
-	    switch (code&BRL_MSK_ARG) {
-	      case BRL_KEY_ENTER:         keysym = XK_KP_Enter;  break;
-	      case BRL_KEY_TAB:           keysym = XK_Tab;       break;
-	      case BRL_KEY_BACKSPACE:     keysym = XK_BackSpace; break;
-	      case BRL_KEY_ESCAPE:        keysym = XK_Escape;    break;
-	      case BRL_KEY_CURSOR_LEFT:   keysym = XK_Left;      break;
-	      case BRL_KEY_CURSOR_RIGHT:  keysym = XK_Right;     break;
-	      case BRL_KEY_CURSOR_UP:     keysym = XK_Up;        break;
-	      case BRL_KEY_CURSOR_DOWN:   keysym = XK_Down;      break;
-	      case BRL_KEY_PAGE_UP:       keysym = XK_Page_Up;   break;
-	      case BRL_KEY_PAGE_DOWN:     keysym = XK_Page_Down; break;
-	      case BRL_KEY_HOME:          keysym = XK_Home;      break;
-	      case BRL_KEY_END:           keysym = XK_End;       break;
-	      case BRL_KEY_INSERT:        keysym = XK_Insert;    break;
-	      case BRL_KEY_DELETE:        keysym = XK_Delete;    break;
-	      case BRL_KEY_FUNCTION + 0:  keysym = XK_F1;        break;
-	      case BRL_KEY_FUNCTION + 1:  keysym = XK_F2;        break;
-	      case BRL_KEY_FUNCTION + 2:  keysym = XK_F3;        break;
-	      case BRL_KEY_FUNCTION + 3:  keysym = XK_F4;        break;
-	      case BRL_KEY_FUNCTION + 4:  keysym = XK_F5;        break;
-	      case BRL_KEY_FUNCTION + 5:  keysym = XK_F6;        break;
-	      case BRL_KEY_FUNCTION + 6:  keysym = XK_F7;        break;
-	      case BRL_KEY_FUNCTION + 7:  keysym = XK_F8;        break;
-	      case BRL_KEY_FUNCTION + 8:  keysym = XK_F9;        break;
-	      case BRL_KEY_FUNCTION + 9:  keysym = XK_F10;       break;
-	      case BRL_KEY_FUNCTION + 10: keysym = XK_F11;       break;
-	      case BRL_KEY_FUNCTION + 11: keysym = XK_F12;       break;
-	      case BRL_KEY_FUNCTION + 12: keysym = XK_F13;       break;
-	      case BRL_KEY_FUNCTION + 13: keysym = XK_F14;       break;
-	      case BRL_KEY_FUNCTION + 14: keysym = XK_F15;       break;
-	      case BRL_KEY_FUNCTION + 15: keysym = XK_F16;       break;
-	      case BRL_KEY_FUNCTION + 16: keysym = XK_F17;       break;
-	      case BRL_KEY_FUNCTION + 17: keysym = XK_F18;       break;
-	      case BRL_KEY_FUNCTION + 18: keysym = XK_F19;       break;
-	      case BRL_KEY_FUNCTION + 19: keysym = XK_F20;       break;
-	      case BRL_KEY_FUNCTION + 20: keysym = XK_F21;       break;
-	      case BRL_KEY_FUNCTION + 21: keysym = XK_F22;       break;
-	      case BRL_KEY_FUNCTION + 22: keysym = XK_F23;       break;
-	      case BRL_KEY_FUNCTION + 23: keysym = XK_F24;       break;
-	      case BRL_KEY_FUNCTION + 24: keysym = XK_F25;       break;
-	      case BRL_KEY_FUNCTION + 25: keysym = XK_F26;       break;
-	      case BRL_KEY_FUNCTION + 26: keysym = XK_F27;       break;
-	      case BRL_KEY_FUNCTION + 27: keysym = XK_F28;       break;
-	      case BRL_KEY_FUNCTION + 28: keysym = XK_F29;       break;
-	      case BRL_KEY_FUNCTION + 29: keysym = XK_F30;       break;
-	      case BRL_KEY_FUNCTION + 30: keysym = XK_F31;       break;
-	      case BRL_KEY_FUNCTION + 31: keysym = XK_F32;       break;
-	      case BRL_KEY_FUNCTION + 32: keysym = XK_F33;       break;
-	      case BRL_KEY_FUNCTION + 33: keysym = XK_F34;       break;
-	      case BRL_KEY_FUNCTION + 34: keysym = XK_F35;       break;
-	      default:
-		fprintf(stderr,"unexpected key: %" PRIX32 "\n",code);
-		continue;
-	    }
-	  default:
-	    fprintf(stderr,"unexpected block type %" PRIX32 "\n",code&BRL_MSK_BLK);
-	    continue;
+	if (((code & BRLAPI_KEY_TYPE_MASK) != BRLAPI_KEY_TYPE_SYM)) {
+	  fprintf(stderr,gettext("unexpected block type %" BRLAPI_PRIxKEYCODE "\n"),code);
+	  continue;
 	}
+
+	modifiers = ((code & BRLAPI_KEY_FLAGS_MASK) >> BRLAPI_KEY_FLAGS_SHIFT) & 0xFF;
+	keysym = code & BRLAPI_KEY_CODE_MASK;
 	keycode = XKeysymToKeycode(dpy,keysym);
+	if (keycode == NoSymbol) {
+	  fprintf(stderr,gettext("Couldn't translate keysym %08X to keycode.\n"),keysym);
+	  continue;
+	}
+
+        {
+          static const unsigned int tryTable[] = {
+            0,
+            ShiftMask,
+            Mod2Mask,
+            Mod3Mask,
+            Mod4Mask,
+            Mod5Mask,
+            ShiftMask|Mod2Mask,
+            ShiftMask|Mod3Mask,
+            ShiftMask|Mod4Mask,
+            ShiftMask|Mod5Mask,
+            0
+          };
+          const unsigned int *try = tryTable;
+
+          do {
+            if (tryModifiers(keycode, &modifiers, *try, keysym)) goto foundModifiers;
+          } while (*++try);
+
+	  fprintf(stderr,gettext("Couldn't find modifiers to apply to %d for getting keysym %08X\n"),keycode,keysym);
+	  continue;
+        }
+      foundModifiers:
+
+	debugf("key %08X: (%d,%x)\n", keysym, keycode, modifiers);
 	if (modifiers)
 	  XkbLockModifiers(dpy, XkbUseCoreKbd, modifiers, modifiers);
-	debugf("key %x\n",code);
 	XTestFakeKeyEvent(dpy,keycode,True,CurrentTime);
 	XTestFakeKeyEvent(dpy,keycode,False,CurrentTime);
 	if (modifiers)
@@ -584,55 +615,18 @@ void toX_f(const char *display) {
  * main 
  */
 
-static char *keypath=NULL,*hostname=NULL;
-
-static struct option longopts[] = {
-  { "help",	0, NULL, 'h' },
-  { "keypath",  1, NULL, 'k' },
-  { "hostname", 1, NULL, 'H' },
-  { "display",  1, NULL, 'd' },
-  { NULL,       0, NULL,  0  },
-};
-
-static void usage(const char *name) {
-  printf(_("usage: %s [opts...]\n"),name);
-  printf(_("  --help                   get what you're reading\n"));
-  printf(_("  --keypath path           set path to authentication key\n"));
-  printf(_("  --hostname [host][:port] set local hostname and port to connect to\n"));
-  printf(_("  --display display        set display\n"));
-}
-
 int main(int argc, char *argv[]) {
-  int c;
-  const char *display = NULL;
-
-  while ((c=getopt_long(argc,argv,":h:H:k:d:f",longopts,NULL))!=EOF)
-  switch (c) {
-  case 'h':
-    usage(argv[0]);
-    exit(0);
-  case 'k':
-    keypath=optarg;
-    break;
-  case 'H':
-    hostname=optarg;
-    break;
-  case 'd':
-    display=optarg;
-    break;
-  case ':':
-    fprintf(stderr,_("missing argument to -%c\n"),optopt);
-    usage(argv[0]);
-    exit(1);
-  case '?':
-    fprintf(stderr,_("bad option %c\n"),optopt);
-    usage(argv[0]);
-    exit(1);
+  {
+    static const OptionsDescriptor descriptor = {
+      OPTION_TABLE(programOptions),
+      .applicationName = "xbrlapi"
+    };
+    processOptions(&descriptor, &argc, &argv);
   }
 
-  tobrltty_init(keypath,hostname);
+  tobrltty_init(auth,host);
 
-  toX_f(display);
+  toX_f(xDisplay);
 
   return 0;
 }
