@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to AT-SPI terminals
- *          for a blind person using a refreshable braille display.
+ * BRLTTY - A background process providing access to the console screen (when in
+ *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -17,8 +17,6 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -27,22 +25,27 @@
 #include <pthread.h>
 #include <limits.h>
 #include <langinfo.h>
-#include <semaphore.h>
 #include <locale.h>
+#ifdef __MINGW32__
+#include "win_pthread.h"
+#else /* __MINGW32__ */
+#include <semaphore.h>
+#endif /* __MINGW32__ */
 
 #include <cspi/spi.h>
 #include <X11/keysym.h>
 #include <X11/Xlib.h>
 
-#include "Programs/misc.h"
-#include "Programs/brldefs.h"
+#include "misc.h"
+#include "brldefs.h"
+#include "charset.h"
 
 typedef enum {
   PARM_TYPE
 } ScreenParameters;
 #define SCRPARMS "type"
 
-#include "Programs/scr_driver.h"
+#include "scr_driver.h"
 
 static int typeText = 1, typeTerminal = 1, typeAll = 0;
 static AccessibleText *curTerm;
@@ -201,7 +204,7 @@ static void delRows(long pos, long num) {
 }
 
 static int
-prepare_AtSpiScreen (char **parameters) {
+processParameters_AtSpiScreen (char **parameters) {
   if (*parameters[PARM_TYPE]) {
     static const char *const choices[] = {"text"   , "terminal"   , "all"   , NULL}; 
     static       int  *const flags  [] = {&typeText, &typeTerminal, &typeAll, NULL};
@@ -219,7 +222,7 @@ prepare_AtSpiScreen (char **parameters) {
         const char *type = types[index];
         unsigned int choice;
 
-        if (validateChoice(&choice, "widget type", type, choices)) {
+        if (validateChoice(&choice, type, choices)) {
           int *flag = flags[choice];
           if ((flag == &typeAll) && (index > 0)) {
             LogPrint(LOG_WARNING, "widget type is mutually exclusive: %s", type);
@@ -228,6 +231,8 @@ prepare_AtSpiScreen (char **parameters) {
           } else {
             *flag = 1;
           }
+        } else {
+          LogPrint(LOG_WARNING, "%s: %s", "invalid widget type", type);
         }
       }
     }
@@ -532,7 +537,7 @@ static void *doAtSpiScreenOpen(void *arg) {
 }
 
 static int
-open_AtSpiScreen (void) {
+construct_AtSpiScreen (void) {
   sem_t SPI_init_sem;
   sem_init(&SPI_init_sem,0,0);
   XInitThreads();
@@ -540,35 +545,36 @@ open_AtSpiScreen (void) {
     LogPrint(LOG_ERR,"main SPI thread failed to be launched");
     return 0;
   }
-  sem_wait(&SPI_init_sem);
+  do {
+    errno = 0;
+  } while (sem_wait(&SPI_init_sem) == -1 && errno == EINTR);
+  if (errno) {
+    LogError("SPI initialization wait failed");
+    return 0;
+  }
   LogPrint(LOG_DEBUG,"SPI initialized");
   return 1;
 }
 
-static int
-setup_AtSpiScreen (void) {
-  return 1;
-}
-
 static void
-close_AtSpiScreen (void) {
+destruct_AtSpiScreen (void) {
   SPI_event_quit();
   pthread_join(SPI_main_thread,NULL);
   LogPrint(LOG_DEBUG,"SPI stopped");
 }
 
 static int
-selectvt_AtSpiScreen (int vt) {
+selectVirtualTerminal_AtSpiScreen (int vt) {
   return 0;
 }
 
 static int
-switchvt_AtSpiScreen (int vt) {
+switchVirtualTerminal_AtSpiScreen (int vt) {
   return 0;
 }
 
 static int
-currentvt_AtSpiScreen (void) {
+currentVirtualTerminal_AtSpiScreen (void) {
   return curTerm? 0: -1;
 }
 
@@ -583,28 +589,29 @@ describe_AtSpiScreen (ScreenDescription *description) {
     description->posx = curPosX;
     description->posy = curPosY;
   } else {
+    description->unreadable = nonatspi;
     description->rows = 1;
     description->cols = strlen(nonatspi);
     description->posx = 0;
     description->posy = 0;
   }
   pthread_mutex_unlock(&updateMutex);
-  description->no = currentvt_AtSpiScreen();
+  description->number = currentVirtualTerminal_AtSpiScreen();
 }
 
 static int
-read_AtSpiScreen (ScreenBox box, unsigned char *buffer, ScreenMode mode) {
+read_AtSpiScreen (ScreenBox box, unsigned char *buffer, ScreenCharacterProperty property) {
   long x,y;
-  wchar_t c;
+  int c;
   if (box.height<0 || box.width<0) return 0;
-  if (mode == SCR_ATTRIB) {
+  if (property == SCR_ATTRIB) {
     memset(buffer,0x07,box.height*box.width);
     return 1;
   }
   memset(buffer,' ',box.height*box.width);
   pthread_mutex_lock(&updateMutex);
   if (!curTerm) {
-    strncpy((char *)buffer, nonatspi+box.left, box.width);
+    setScreenMessage(&box, buffer, property, nonatspi);
     goto out;
   }
   if (box.top+box.height>curNumRows)
@@ -614,11 +621,9 @@ read_AtSpiScreen (ScreenBox box, unsigned char *buffer, ScreenMode mode) {
       if (curRowLengths[box.top+y])
 	for (x=0; x<box.width; x++) {
 	  if (box.left+x<curRowLengths[box.top+y] - (curRows[box.top+y][curRowLengths[box.top+y]-1]=='\n')) {
-	    if ((c = curRows[box.top+y][box.left+x])<0x100)
-	      /* latin1 only */
-	      buffer[y*box.width+x] = c;
-	    else
-	      buffer[y*box.width+x] = '?';
+	    if ((c = convertWcharToChar(curRows[box.top+y][box.left+x])) == EOF)
+	      c = '?';
+	    buffer[y*box.width+x] = c;
 	  }
 	}
 out:
@@ -627,11 +632,12 @@ out:
 }
 
 static int
-insert_AtSpiScreen (ScreenKey key) {
+insertKey_AtSpiScreen (ScreenKey key) {
   long keysym;
   int modMeta=0, modControl=0;
 
   if (key < SCR_KEY_ENTER) {
+    wint_t wc;
     if (key & SCR_KEY_MOD_META) {
       key &= ~SCR_KEY_MOD_META;
       modMeta = 1;
@@ -642,7 +648,13 @@ insert_AtSpiScreen (ScreenKey key) {
       modControl = 1;
     }
 
-    keysym = key;
+    wc = convertCharToWchar(key);
+    if (wc == WEOF)
+      keysym = key; /* let's hope this is more or less correct */
+    else if (wc < 0x100)
+      keysym = wc; /* latin1 character */
+    else
+      keysym = 0x1000000 | wc;
   } else {
     switch (key) {
       case SCR_KEY_ENTER:         keysym = XK_KP_Enter;  break;
@@ -735,18 +747,10 @@ insert_AtSpiScreen (ScreenKey key) {
 }
 
 static int
-execute_AtSpiScreen (int command) {
-  int blk = command & BRL_MSK_BLK;
-  int arg
-#ifdef HAVE_ATTRIBUTE_UNUSED
-      __attribute__((unused))
-#endif /* HAVE_ATTRIBUTE_UNUSED */
-      = command & BRL_MSK_ARG;
+executeCommand_AtSpiScreen (int command) {
+  int blk UNUSED = command & BRL_MSK_BLK;
+  int arg UNUSED = command & BRL_MSK_ARG;
 
-  switch (blk) {
-    case BRL_BLK_PASSAT2:
-      break;
-  }
   return 0;
 }
 
@@ -755,13 +759,12 @@ scr_initialize (MainScreen *main) {
   initializeRealScreen(main);
   main->base.describe = describe_AtSpiScreen;
   main->base.read = read_AtSpiScreen;
-  main->base.insert = insert_AtSpiScreen;
-  main->base.selectvt = selectvt_AtSpiScreen;
-  main->base.switchvt = switchvt_AtSpiScreen;
-  main->base.currentvt = currentvt_AtSpiScreen;
-  main->base.execute = execute_AtSpiScreen;
-  main->prepare = prepare_AtSpiScreen;
-  main->open = open_AtSpiScreen;
-  main->setup = setup_AtSpiScreen;
-  main->close = close_AtSpiScreen;
+  main->base.insertKey = insertKey_AtSpiScreen;
+  main->base.selectVirtualTerminal = selectVirtualTerminal_AtSpiScreen;
+  main->base.switchVirtualTerminal = switchVirtualTerminal_AtSpiScreen;
+  main->base.currentVirtualTerminal = currentVirtualTerminal_AtSpiScreen;
+  main->base.executeCommand = executeCommand_AtSpiScreen;
+  main->processParameters = processParameters_AtSpiScreen;
+  main->construct = construct_AtSpiScreen;
+  main->destruct = destruct_AtSpiScreen;
 }
